program calc
  use string
  use RPN
  implicit none

  type(RPN_STACK), dimension(RPN_MaxStacks)    :: Stacks
  type(RPN_CONST), dimension(RPN_MaxConstants) :: Constants
  type(RPN_EQN)  , dimension(RPN_MaxEquations) :: Eqns

  integer :: i, ip, ierr, status, nlen
  logical :: cmplx
  character(len=128) :: lines, c128

  c128 =    '                            '                       &
       & // '                                                  ' &
       & // '                                                  '
  cmplx = .false.

  Eqns%current = 1

  call help
  do
     lines = c128
     write(6,*) 'Equations written in RPN ( or help )'
     if( Stacks(1)%current > 1 ) then
        write(6,*) '(STACK) '
        do i = Stacks(1)%current, 2, -1
           write(6,*) i-1, Stacks(i)%val
        end do
     else
        write(6,*) '(NO STACK)'
     end if
     read(5,'(a)', IOSTAT=status) lines
     if( status /= 0 ) exit
     call ToUpper( lines )

     if( lines(1:4) == 'QUIT' ) then
        exit

     else if( lines(1:3) == 'CLS' ) then
        call RPN_RestartStacks( Stacks, RPN_MaxStacks )
        write(6,*)
        cycle

     else if( lines(1:5) == 'PRINT' ) then
        call ShowDefinedEquations( Eqns, RPN_MaxEquations )
        cycle

     else if( lines(1:4) == 'HELP' ) then
        call help
        cycle
     end if

     nlen = len_trim( lines )
     if( lines(nlen-2:nlen) == 'DEF' ) then
        call DefEquations( lines, Eqns, RPN_MaxEquations )
        cycle
     end if

     ierr = RPN_CALCULATION( Stacks   , RPN_MaxStacks,    &
          &                  Constants, RPN_MaxConstants, &
          &                  Eqns     , RPN_MaxEquations, &
          &                  lines                        )

     if( ierr == 0 ) then
        if(         lines(nlen-3:nlen) == 'CADD'                           &
             & .or. lines(nlen-3:nlen) == 'CSUB'                           &
             & .or. lines(nlen-3:nlen) == 'CMUL'                           &
             & .or. lines(nlen-3:nlen) == 'CDIV'                           &
             & .or. lines(nlen-3:nlen) == 'CINV'                           &
             & .or. lines(nlen-3:nlen) == 'CPOW'                           &
             & .or. lines(nlen-4:nlen) == 'CIPOW'                          &
             & .or. lines(nlen-4:nlen) == 'CONJG' ) then
           cmplx = .true.
        end if

        if( cmplx .eqv. .false. ) then
           if( stacks(1)%current > 1 ) then
              write(6,*)
              write(6,*) '***RESULT***'
              i = Stacks(1)%current
              write(6,*) trim( lines ), ' = ', Stacks(i)%val(1,1,1,1)
           end if
        else
           if( stacks(1)%current > 2 ) then
              write(6,*)
              write(6,*) '***RESULT (REAL , IMAGINARY)***'
              i = Stacks(1)%current
              write(6,*) trim( lines ), ' = ( ',                            &
                   & Stacks(i-1)%val(1,1,1,1), ' , ' ,                      &
                   & Stacks(i)%val(1,1,1,1), ' )'
           end if
        end if

        cmplx = .false.
     else
        write(6,*)
        write(6,*) '***ERROR***', ierr
        call RPN_RestartStacks( Stacks, RPN_MaxStacks )
     end if

     write(6,*)
  end do

  stop
contains
  subroutine DefEquations( lines, Eqns, MaxEquations )
    character(len=*), intent(IN)    :: lines
    integer         , intent(IN)    :: MaxEquations
    type(RPN_EQN)   , intent(INOUT) :: Eqns(MaxEquations)

    integer :: i
    integer :: n, ip, DataType, status
    integer :: is1, is2, nstr, nlen
    integer :: nc
    integer :: clen
    
    character(len=128) :: tmp
    character(len=28)  :: cname

    ip = 0
    n  = Eqns(1)%current
    nlen = len_trim( lines )
    cname = '                           '

    DataType = InquireStringType( lines, ip )
    if( DataType == TypeEnd .or. DataType == TypeError ) return

    status = InquireCharacter( lines, ip, nstr, cname )
    clen = len_trim( cname )
    nc   = n + 1
    do i = 1, n
       if( Eqns(i)%name(1:clen+1) == cname(1:clen+1) ) then
          nc = i
          Eqns(nc)%def = ''
          exit
       end if
    end do

    if( nc > n ) then
       Eqns(nc)%name = cname
    end if

!    n   = n + 1
!    status = InquireCharacter( lines, ip, nstr, Eqns(n)%name )

    DataType = InquireStringType( lines, ip )
    if( DataType == TypeEnd .or. DataType == TypeError ) then
       Eqns(nc)%name = '                            '
       return
    end if

    is1 = ip
    status = InquireCharacter( lines, ip, nstr, tmp )

    do
       DataType = InquireStringType( lines, ip )
       if( DataType == TypeEnd .or. DataType == TypeError ) exit
       status = InquireCharacter( lines, ip, nstr, tmp )
       is2 = ip
    end do

    if( lines(nlen-4:nlen) == '= DEF' ) then
       Eqns(nc)%def = lines(is1:nlen-3)
    else
       Eqns(nc)%def = lines(is1:nlen-3) // ' ='
    end if

    if( nc > n ) Eqns(1)%current = nc

    return
  end subroutine DefEquations

  subroutine ShowDefinedEquations( Eqns, MaxEquations )
    integer      , intent(IN) :: MaxEquations
    type(RPN_EQN), intent(IN) :: Eqns(MaxEquations)

    integer :: i

    if( Eqns(1)%current == 1 ) return

    write(6,*)
    write(6,*) ' [DEFINED EQUATIONS AND CONSTANTS]'
    write(6,*) '========================================================='
    do i = 2, Eqns(1)%current
       write(6,'(a10,a2,a66)') trim( Eqns(i)%name ), ' ', trim( Eqns(i)%def )
    end do
    write(6,*) '========================================================='
    write(6,*)

    return
  end subroutine ShowDefinedEquations
end program calc

subroutine help
  write(6,*)
  write(6,*) '==============================================================='
  write(6,*) ' cls: refresh stack             quit: end program '
  write(6,*) 'help: show this help            print: show eqs. and consts.'
  write(6,*) '==============================================================='
  write(6,*) '  ADD      A + B                SUB    A - B'
  write(6,*) '  MUL      A * B                DIV    A * B'
  write(6,*) '  POW      A^B                  ABS    |A|'
  write(6,*) '  IPOW     A^B (B:integer)'
  write(6,*) '  LOG      log(A)               LOG10  log10(A)'
  write(6,*) '  SQRT     sqrt(A)              INV    1/A'
  write(6,*) '  NEG      -A                   EXP    exp(A)'
  write(6,*) '  R2       A*A + B*B'
  write(6,*) '  D2R      deg->rad(A)          R2D    rad->deg(A)'
  write(6,*) '  SIN      sin(A)               ASIN   asin(A)'
  write(6,*) '  COS      cos(A)               ACOS   acos(A)'
  write(6,*) '  TAN      tan(A)               ATAN   atan(A)'
  write(6,*) '  ATAN2    atan2(A,B)'
  write(6,*) '  COMPBIG                       COMPSMALL'
  write(6,*) '  COPY                          SWAP'
  write(6,*) '  POP                           SHIFT'
  write(6,*) '  CADD     (A+Bi)+(C+Di)        CSUB   (A+Bi)-(C+Di)'
  write(6,*) '  CMUL     (A+Bi)*(C+Di)        CDIV   (A+Bi)/(C+Di)'
  write(6,*) '  CINV     1/(A+Bi)             CCOPY'
  write(6,*) '  CPOW     (A+Bi)^(C+Di)        CABS   |A+Bi|(=SQRT(A*A+B*B))'
  write(6,*) '  CIPOW    (A+Bi)^C (C:integer)'
  write(6,*) '  CONJG    complex conjugate of (A+Bi)'
  write(6,*) '==============================================================='
  write(6,*) ' You can define any variables and equations,         '
  write(6,*) '    ex.1) A 2.0 = DEF'
  write(6,*) '    ex.2) WSPD [X] [Y] R2 SQRT = DEF'
  write(6,*) ' The first is an example to define a constant value.'
  write(6,*) ' The second is an example to define an equation making'
  write(6,*) ' use of already defined variables ( X, Y ).'
  write(6,*) '==============================================================='
  write(6,*)
  return
end subroutine help

subroutine ReadFromData( operation, stacks, MaxStacks, status )
  use string
  use ModelInfo
  use namelist
  use Endian
  use RPN
  implicit none
  integer                              , intent(IN)    :: MaxStacks
  type(RPN_STACK), dimension(MaxStacks), intent(INOUT) :: stacks
  character(len=*)                     , intent(IN)    :: operation
  integer                              , intent(INOUT) :: status

  status = 0
  return
end subroutine ReadFromData

subroutine ReadFuncData( operation, stacks, MaxStacks, status )
  use string
  use ModelInfo
  use namelist
  use Endian
  use RPN
  implicit none
  integer         , intent(IN)    :: MaxStacks
  type(RPN_STACK) , intent(INOUT) :: stacks(MaxStacks)
  character(len=*), intent(IN)    :: operation
  integer         , intent(INOUT) :: status

  integer :: i
  integer :: ip
  integer :: is
  integer :: ipos

  integer :: ix
  integer :: jx
  integer :: kx
  integer :: it

  character(len=20) :: FuncName
  character(len=60) :: FuncArg
  character(len=60) :: fname

  ip = 6

  status = AnalyzeFuncComponent( trim( operation ), ',', FuncName, ip, ipos )

  if( FuncName(1:6) == 'BREAD ' ) then
     status = RPN_FUNC_ARG_ERROR
     if( ipos /= 0 ) return

     do i = 1, 5
        is = 1
        FuncArg = '                              '                          &
             & // '                              '
        status = AnalyzeFuncComponent( trim( operation ),                   &
             &                         ',', FuncArg, ip, ipos )
        if( status /= STRING_NOERROR ) return
        if( i < 5 .and. ipos /= 0 ) return

        select case( i )
        case ( 1 )
           status = InquireNumber( trim( FuncArg ), is, ix )
        case ( 2 )
           status = InquireNumber( trim( FuncArg ), is, jx )
        case ( 3 )
           status = InquireNumber( trim( FuncArg ), is, kx )
        case ( 4 )
           status = InquireNumber( trim( FuncArg ), is, it )
        case ( 5 )
           fname = '                              '                         &
                & // '                              '
           fname = trim( FuncArg )
        end select

     end do

     write(6,*) ix, jx, kx, it, trim(fname)
  end if

  status = 0
  return
end subroutine ReadFuncData
