subroutine CrossSimilarity( stacks, MaxStacks )
  use RPN
  use ModelInfo
  use namelist
  implicit none
  integer                              , intent(IN)    :: MaxStacks
  type(RPN_STACK), dimension(MaxStacks), intent(INOUT) :: stacks

  integer :: ic
  integer :: i, j, k, t, ii, jj, m
  integer :: i1, iw, i2, j1, jw, j2, ieo, jeo 
  integer :: ixi, jxi, kxi, iti
  integer :: ixt, jxt, kxt, itt
  integer :: ixw, jxw, kxw, itw

  real :: cnt_max, delta, x, y, xt, yt
  real :: tmat(3,3), tmat_tmp(3,3)
  real :: IDW_rate, IDW_rate_tmp
  real, dimension(:,:,:,:), allocatable :: wbox, res
  real, dimension(:,:)    , allocatable :: cnt, ave_w, sig_w
  real, dimension(:,:)    , allocatable :: ave_t, sig_t
  integer, dimension(:,:) , allocatable :: cnt_w, cnt_t
  
  ic = stacks(1)%current
  if( ic < 3 ) return
  m = stacks(ic)%model

  IDW_rate     = MODEL(m)%dy / MODEL(m)%dx
  IDW_rate_tmp = IDW_rate

  ixi = stacks(ic)%ix
  jxi = stacks(ic)%jx
  kxi = stacks(ic)%kx
  iti = stacks(ic)%it
  ixt = stacks(ic-1)%ix
  jxt = stacks(ic-1)%jx
  kxt = stacks(ic-1)%kx
  itt = stacks(ic-1)%it
  ixw = input_wind(3)
  jxw = input_wind(4)
  kxw = kxt
  itw = itt
  ieo = 1 - mod( ixw, 2 )
  jeo = 1 - mod( jxw, 2 )

  tmat      = 0.
  tmat(1,1) = 1.
  tmat(2,2) = 1.
  tmat(3,3) = 1.

  cnt_max = real( ixw * jxw ) * count_ratio

  if( kxi /= kxt .or. iti /= itt ) return

  allocate( wbox(ixw,jxw,kxw,itw) )
  allocate(  res(ixt,jxt,kxt,itt) )
  allocate(  cnt(        kxw,itw) )
  allocate( ave_w(       kxi,iti) )
  allocate( cnt_w(       kxi,iti) )
  allocate( sig_w(       kxi,iti) )
  allocate( ave_t(       kxi,iti) )
  allocate( cnt_t(       kxi,iti) )
  allocate( sig_t(       kxi,iti) )

  ! read target window
  wbox = stacks(ic)%undef
  ave_w = 0.
  cnt_w = 0
  i1   = input_wind(1)
  i2   = i1 + ixw - 1
  j1   = input_wind(2)
  j2   = j1 + jxw - 1

  if( wind_trans < 0 .or. wind_trans > 3 ) wind_trans = 0
  if( wind_trans == 0 ) then
     tmat(1,3) = - ( i1 + real( ixw - 1 ) / 2. )
     tmat(2,3) = - ( j1 + real( jxw - 1 ) / 2. )
  else
     do i = 1, wind_trans
        j = 2*i - 1
        tmat_tmp      = 0.
        tmat_tmp(3,3) = 1.
        tmat_tmp(1,1) = wind_tmat(j  ,1)
        tmat_tmp(1,2) = wind_tmat(j+1,1)
        tmat_tmp(2,1) = wind_tmat(j  ,2)
        tmat_tmp(2,2) = wind_tmat(j+1,2)

        tmat = matmul( matrix_a=tmat, matrix_b=tmat_tmp )
     end do

     tmat_tmp = 0.
     tmat_tmp(1,1) = 0.
     tmat_tmp(2,2) = 0.
     tmat_tmp(3,3) = 0.
     tmat_tmp(1,3) = - ( i1 + real( ixw - 1 ) / 2. )
     tmat_tmp(2,3) = - ( j1 + real( jxw - 1 ) / 2. )

     tmat = matmul( matrix_a=tmat, matrix_b=tmat_tmp )
  end if

  delta = tmat(1,1)*tmat(2,2) - tmat(2,1)*tmat(1,2)
  if( abs( delta ) < 1.e-4 ) then
     tmat = 0.
     tmat(1,1) = 0.
     tmat(2,2) = 0.
     tmat(3,3) = 0.
     tmat(1,3) = i1 + real( ixw - 1 ) / 2.
     tmat(2,3) = j1 + real( jxw - 1 ) / 2.
  else
     tmat_tmp = 0.
     tmat_tmp(1,1) = tmat(2,2)
     tmat_tmp(1,2) = - tmat(1,2)
     tmat_tmp(1,3) = tmat(1,2)*tmat(2,3) - tmat(2,2)*tmat(1,3)
     tmat_tmp(2,1) = - tmat(2,1)
     tmat_tmp(2,2) = tmat(1,1)
     tmat_tmp(2,3) = tmat(2,1)*tmat(1,3) - tmat(1,1)*tmat(2,3)
     tmat_tmp(3,3) = delta
     tmat = tmat_tmp / delta
  end if

  write(6,*) '*+++DEBUG***'
  write(6,*) tmat(1,:)
  write(6,*) tmat(2,:)
  write(6,*) tmat(3,:)

!!$  do t = 1, iti
!!$     do k = 1, kxi
!!$        do j = 1, jxw
!!$           do i = 1, ixw
!!$              xt = real(i-1) - real(ixw-1)/2.
!!$              yt = real(j-1) - real(jxw-1)/2.
!!$              x = tmat(1,1)*xt + tmat(1,2)*yt + tmat(1,3)
!!$              y = tmat(2,1)*xt + tmat(2,2)*yt + tmat(2,3)
!!$              wbox(i,j,k,t) = IDW( x, y, itparg1, ic, IDW_rate_tmp )
!!$              if( wbox(i,j,k,t) /= stacks(ic)%undef ) then
!!$                 ave_w(k,t) = ave_w(k,t) + wbox(i,j,k,t)
!!$                 cnt_w(k,t) = cnt_w(k,t) + 1
!!$              end if
!!$           end do
!!$        end do
!!$     end do
!!$  end do
!!$
  do t = 1, iti
     do k = 1, kxi
        do j = j1, j2
           do i = i1, i2
              if( stacks(ic)%val(i,j,k,t) /= stacks(ic)%undef ) then
                 wbox(i-i1+1,j-j1+1,k,t) = stacks(ic)%val(i,j,k,t)
                 ave_w(k,t) = ave_w(k,t) + stacks(ic)%val(i,j,k,t)
                 cnt_w(k,t) = cnt_w(k,t) + 1
              end if
           end do
        end do
     end do
  end do

  ! average
  do t = 1, iti
     do k = 1, kxi
        if( cnt_w(k,t) >= cnt_max ) then
           ave_w(k,t) = ave_w(k,t) / real( cnt_w(k,t) )
        else
           ave_w(k,t) = stacks(ic)%undef
        end if
     end do
  end do

  ! deviation
  sig_w = 0.
  do t = 1, iti
     do k = 1, kxi
        do j = 1, jxw
           do i = 1, ixw
              if( wbox(i,j,k,t) /= stacks(ic)%undef ) then
                 sig_w(k,t) = sig_w(k,t) + ( ave_w(k,t) - wbox(i,j,k,t) )**2
              end if
           end do
        end do

        if( ave_w(k,t) == stacks(ic)%undef ) then
           sig_w(k,t) = stacks(ic)%undef
        end if
     end do
  end do

  ! 
  res = 0.
  if( similarity == 1 ) then
   do j = 1, jxt
        do i = 1, ixt
           cnt = 0
           do jj = 1, jxw
              jw = j - ( jxw + 1 ) / 2 + jj
              if( jw < 1 .or. jw > jxt ) cycle

              do ii = 1, ixw
                 iw = i - ( ixw + 1 ) / 2 + ii
                 if( iw < 1 .or. iw > ixt ) cycle

                 do t = 1, itt
                    do k = 1, kxt
                       if( stacks(ic-1)%val(iw,jw,k,t) /= stacks(ic-1)%undef .and. &
                            & wbox(ii,jj,k,t) /= stacks(ic)%undef ) then
                          res(i,j,k,t) = res(i,j,k,t) + abs( stacks(ic-1)%val(iw,jw,k,t) - wbox(ii,jj,k,t) )
                          cnt(k,t) = cnt(k,t) + 1.
                       end if
                    end do
                 end do

              end do
           end do
           
           do t = 1, itt
              do k = 1, kxt
                 if( cnt(k,t) >= cnt_max ) then
                    res(i,j,k,t) = res(i,j,k,t) / cnt(k,t)
                 else
                    res(i,j,k,t) = stacks(ic-1)%undef
                 end if
              end do
           end do

        end do
     end do

  else if( similarity == 2 ) then
     do j = 1, jxt
        do i = 1, ixt
           ave_t = 0.
           cnt_t = 0.
           sig_t = 0.
           do jj = 1, jxw
              jw = j - ( jxw + 1 ) / 2 + jj
              if( jw < 1 .or. jw > jxt ) cycle

              do ii = 1, ixw
                 iw = i - ( ixw + 1 ) / 2 + ii
                 if( iw < 1 .or. iw > ixt ) cycle

                 do t = 1, itt
                    do k = 1, kxt
                       if( stacks(ic-1)%val(iw,jw,k,t) &
                            & /= stacks(ic-1)%undef ) then
                          ave_t(k,t) = ave_t(k,t) + stacks(ic-1)%val(iw,jw,k,t)
                          cnt_t(k,t) = cnt_t(k,t) + 1
                       end if
                    end do
                 end do

              end do
           end do
           
           ! average
           do t = 1, itt
              do k = 1, kxt
                 if( cnt_t(k,t) >= cnt_max ) then
                    ave_t(k,t) = ave_t(k,t) / cnt_t(k,t)
                 else
                    ave_t(k,t) = stacks(ic-1)%undef
                 end if
              end do
           end do

           ! deviation
           do t = 1, itt
              do k = 1, kxt
                 do jj = 1, jxw
                    jw = j - ( jxw + 1 ) / 2 + jj
                    if( jw < 1 .or. jw > jxt ) cycle

                    do ii = 1, ixw
                       iw = i - ( ixw + 1 ) / 2 + ii
                       if( iw < 1 .or. iw > ixt ) cycle

                       if( stacks(ic-1)%val(iw,jw,k,t) &
                            & /= stacks(ic-1)%undef ) then
                          sig_t(k,t) = sig_t(k,t) + &
                               & (ave_t(k,t) - stacks(ic-1)%val(iw,jw,k,t))**2
                       end if
                    end do
                 end do

                 if( ave_t(k,t) == stacks(ic-1)%undef ) then
                    sig_t(k,t) = stacks(ic-1)%undef
                 end if

              end do
           end do

           do k = 1, kxw
              do t = 1, itw

                 do jj = 1, jxw
                    jw = j - ( jxw + 1 ) / 2 + jj
                    if( jw < 1 .or. jw > jxt ) cycle

                    do ii = 1, ixw
                       iw = i - ( ixw + 1 ) / 2 + ii
                       if( iw < 1 .or. iw > ixt ) cycle

                       if( stacks(ic-1)%val(iw,jw,k,t) /= stacks(ic-1)%undef .and. wbox(ii,jj,k,t) /= stacks(ic)%undef ) then
                          res(i,j,k,t) = res(i,j,k,t) + &
                               & ( stacks(ic-1)%val(iw,jw,k,t) - ave_t(k,t) ) &
                               & * ( wbox(ii,jj,k,t) - ave_w(k,t) )
                       end if

                    end do
                 end do

                 if( sig_w(k,t) /= stacks(ic)%undef .and. &
                      & sig_t(k,t) /= stacks(ic-1)%undef ) then
                    res(i,j,k,t) = res(i,j,k,t) / sqrt( sig_w(k,t) * sig_t(k,t) )
                 else
                    res(i,j,k,t) = stacks(ic-1)%undef
                 end if
              end do
           end do

        end do
     end do

  end if

  stacks(ic-1)%val = res

  deallocate(  wbox )
  deallocate(   res )
  deallocate(   cnt )
  deallocate( ave_w )
  deallocate( cnt_w )
  deallocate( sig_w )
  deallocate( ave_t )
  deallocate( cnt_t )
  deallocate( sig_t )

  call RPN_ClearStacks( stacks, Maxstacks )

  return
contains
  include "func_inner_idw.f90"
end subroutine CrossSimilarity
