integer function FuncBmask( stacks, MaxStacks, mode, mno,                   &
     &                      x1, y1, x2, y2, inval, outval )
  use RPN
  use ModelInfo
  use GeoGraphic
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mode
  integer        , intent(IN)    :: mno
  real           , intent(IN)    :: x1
  real           , intent(IN)    :: y1
  real           , intent(IN)    :: x2
  real           , intent(IN)    :: y2
  real           , intent(IN)    :: inval
  real           , intent(IN)    :: outval

  integer :: ix
  integer :: jx
  integer :: kx
  integer :: it
  integer :: i
  integer :: j
  integer :: is
  integer :: ie
  integer :: js
  integer :: je

  real :: x
  real :: y
  real :: undef
  real :: vin
  real :: vout

  real, dimension(:,:,:,:), allocatable :: bmask

!  integer :: ic, ico
!  integer :: i, j, k, t, n, ix, jx, kx, it
!  logical :: equal, inside, ref

  FuncBmask = RPN_FUNC_OPE_ERROR

  if( mno < 1 .or. mno > MODEL(1)%TotalInfo ) return
  if( mode < 1 .or. mode > 2 ) return

  ix = MODEL(mno)%ix
  jx = MODEL(mno)%jx
  kx = 1
  it = 1

  undef = MODEL(mno)%undef
  vin   = inval
  vout  = outval

  if( vin  == RPN_Undefined ) vin  = undef
  if( vout == RPN_Undefined ) vout = undef

  allocate( bmask(ix,jx,1,1) )

  do j = 1, jx
     do i = 1, ix
        bmask(i,j,1,1) = vout
     end do
  end do

  if( mode == 1 ) then
     is = 1
     ie = ix
     js = 1
     je = jx
     if( x1 > 1 .and. x1 < ix ) is = int( x1 + 1e-4 )
     if( x2 > 1 .and. x2 < ix ) ie = int( x2 + 1e-4 )
     if( y1 > 1 .and. y1 < jx ) js = int( y1 + 1e-4 )
     if( y2 > 1 .and. y2 < jx ) je = int( y2 + 1e-4 )
     
  else if( mode == 2 ) then
     is = 1
     ie = ix
     js = 1
     je = jx

     if( MODEL(mno)%cproj == 'LATLON' ) then
        x = MODEL(mno)%slon + real(x1-1)*MODEL(mno)%dx
        y = MODEL(mno)%slat + real(y1-1)*MODEL(mno)%dy
        if( x > 1 .and. x < ix ) is = int( x + 1e-4 )
        if( y > 1 .and. y < jx ) js = int( y + 1e-4 )

        x = MODEL(mno)%slon + real(x2-1)*MODEL(mno)%dx
        y = MODEL(mno)%slat + real(y2-1)*MODEL(mno)%dy
        if( x > 1 .and. x < ix ) ie = int( x + 1e-4 )
        if( y > 1 .and. y < jx ) je = int( y + 1e-4 )
        
     else
        call ll2xy( x1,                                                      &
             &      y1,                                                      &
             &      x,                                                       &
             &      y,                                                       &
             &      MODEL(mno)%cproj,                                        &
             &      MODEL(mno)%dx,                                           &
             &      MODEL(mno)%dy,                                           &
             &      MODEL(mno)%clon,                                         &
             &      MODEL(mno)%clat,                                         &
             &      MODEL(mno)%xc,                                           &
             &      MODEL(mno)%yc,                                           &
             &      tlat1=MODEL(mno)%tlat1,                                  &
             &      tlat2=MODEL(mno)%tlat2,                                  &
             &      earth=MODEL(mno)%er                                      )
        if( x > 1 .and. x < ix ) is = int( x + 1e-4 )
        if( y > 1 .and. y < jx ) js = int( y + 1e-4 )

        call ll2xy( x2,                                                      &
             &      y2,                                                      &
             &      x,                                                       &
             &      y,                                                       &
             &      MODEL(mno)%cproj,                                        &
             &      MODEL(mno)%dx,                                           &
             &      MODEL(mno)%dy,                                           &
             &      MODEL(mno)%clon,                                         &
             &      MODEL(mno)%clat,                                         &
             &      MODEL(mno)%xc,                                           &
             &      MODEL(mno)%yc,                                           &
             &      tlat1=MODEL(mno)%tlat1,                                  &
             &      tlat2=MODEL(mno)%tlat2,                                  &
             &      earth=MODEL(mno)%er                                      )
        if( x > 1 .and. x < ix ) ie = int( x + 1e-4 )
        if( y > 1 .and. y < jx ) je = int( y + 1e-4 )
     end if
  end if

  do j = js, je
     do i = is, ie
        bmask(i,j,1,1) = vin
     end do
  end do

  FuncBmask = RPN_PushStack( stacks,                                        &
       &                     MaxStacks,                                     &
       &                     bmask,                                         &
       &                     ix,                                            &
       &                     jx,                                            &
       &                     kx,                                            &
       &                     it,                                            &
       &                     undef=undef,                                   &
       &                     model=mno,                                     &
       &                     omodel=mno                                     )

  deallocate( bmask )
  FuncBmask = RPN_NOERROR
  return
end function FuncBmask
