integer function FuncCenterPoint( stacks, MaxStacks, mode )
  use RPN
  use ModelInfo
  use namelist
  use GeoGraphic
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mode

  integer :: ix, jx, kx, it
  integer :: id, jd, idx, jdx, iid, jjd
  integer :: i, j, k, t, tt
  integer :: i1, j1, i2, j2
  integer :: ic
  integer :: m
  integer :: iint
  integer :: jint

  real :: MaxDensity

  real :: Xs, Ys, X2, Y2, u1, u2, v1, v2
  real :: dx, dy, dt, coef, dtdx, dis
  real :: n, IDW_rate, itpmethod_tmp

  integer, dimension(:,:)    , allocatable :: iden, iden2
  real   , dimension(:,:,:,:), allocatable :: tmp

  logical :: debug

  debug = .false.

  ! default
  MaxDensity = 1.

  FuncCenterPoint = RPN_FUNC_OPE_ERROR

  ic = stacks(1)%current

  if( mode < 1 .or. mode > 3 ) return
  if( ( mode == 1 .or. mode == 2 ) .and. ic < 2 ) return
  if( mode == 3 .and. ic < 3 ) return


  ix = stacks(ic)%ix
  jx = stacks(ic)%jx
  kx = stacks(ic)%kx
  it = 1

  iint = ix / 50
  jint = jx / 50
  iint = max( iint, 1 )
  jint = max( jint, 1 )

  if( mode == 3 ) then
     if(    stacks(ic-1)%ix /= ix .or. &
          & stacks(ic-1)%jx /= jx .or. &
          & stacks(ic-1)%kx /= kx .or. &
          & stacks(ic-1)%it /= it ) return
  end if

  allocate( tmp(ix,jx,kx,it) )

  ! maximum point
  if( mode == 1 ) then
     tmp = stacks(ic)%undef

     do t = 1, it
        do k = 1, kx
           do j = 2, jx-1
              do i = 2, ix-1
                 if(    stacks(ic)%val(i-1,j+1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i  ,j+1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i+1,j+1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i-1,j  ,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i  ,j  ,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i+1,j  ,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i-1,j-1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i  ,j-1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i+1,j-1,k,t) /= stacks(ic)%undef ) then

                    if(    stacks(ic)%val(i,j,k,t) >=                   &
                         &            stacks(ic)%val(i-1,j+1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) >=                   &
                         &            stacks(ic)%val(i  ,j+1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) >=                   &
                         &            stacks(ic)%val(i+1,j+1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) >=                   &
                         &            stacks(ic)%val(i-1,j  ,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) >=                   &
                         &            stacks(ic)%val(i+1,j  ,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) >=                   &
                         &            stacks(ic)%val(i-1,j-1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) >=                   &
                         &            stacks(ic)%val(i  ,j-1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) >=                   &
                         &            stacks(ic)%val(i+1,j-1,k,t) ) then
                       tmp(i,j,k,t) = stacks(ic)%val(i,j,k,t)
                    end if
                 end if
              end do
           end do
        end do
     end do

     stacks(ic)%val = tmp

  ! minimum point
  else if( mode == 2 ) then
     tmp = stacks(ic)%undef

     do t = 1, it
        do k = 1, kx
           do j = 2, jx-1
              do i = 2, ix-1
                 if(    stacks(ic)%val(i-1,j+1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i  ,j+1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i+1,j+1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i-1,j  ,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i  ,j  ,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i+1,j  ,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i-1,j-1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i  ,j-1,k,t) /= stacks(ic)%undef .and. &
                      & stacks(ic)%val(i+1,j-1,k,t) /= stacks(ic)%undef ) then

                    if(    stacks(ic)%val(i,j,k,t) <=                   &
                         &            stacks(ic)%val(i-1,j+1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) <=                   &
                         &            stacks(ic)%val(i  ,j+1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) <=                   &
                         &            stacks(ic)%val(i+1,j+1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) <=                   &
                         &            stacks(ic)%val(i-1,j  ,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) <=                   &
                         &            stacks(ic)%val(i+1,j  ,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) <=                   &
                         &            stacks(ic)%val(i-1,j-1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) <=                   &
                         &            stacks(ic)%val(i  ,j-1,k,t) .and. &
                         & stacks(ic)%val(i,j,k,t) <=                   &
                         &            stacks(ic)%val(i+1,j-1,k,t) ) then
                       tmp(i,j,k,t) = stacks(ic)%val(i,j,k,t)
                    end if
                 end if
              end do
           end do
        end do
     end do

     stacks(ic)%val = tmp

  ! divergent point of stream line
  else if( mode == 3 ) then
     tmp = stacks(ic-1)%undef

     idx = MaxDensity * real( ix - 1 ) + 1
     jdx = MaxDensity * real( jx - 1 ) + 1

     allocate( iden(idx,jdx) )
     allocate( iden2(idx,jdx) )

     m  = stacks(ic)%model

     if( MODEL(m)%cproj == 'NOPROJ' .or. &
          & MODEL(m)%cproj == 'LATLON' ) then
        FuncCenterPoint = RPN_NOT_DEFINED
        return
     end if

     IDW_rate = MODEL(m)%dy / MODEL(m)%dx
     n        = itparg1
     itpmethod_tmp = itpmethod
     if( itpmethod == 1 ) itpmethod = 2
!     dt = MODEL(m)%dx / 1.
     dt = 10*60
     dx = MODEL(m)%dx
     dy = MODEL(m)%dy
     dtdx = dt/dx

     do t = 1, it
        do k = 1, kx
           iden = 0
           do j = 2, jx-1, jint
              do i = 2, ix-1, iint
                 if( stacks(ic)%val(i,j,k,t) == stacks(ic)%undef .or. &
                      & stacks(ic-1)%val(i,j,k,t) == stacks(ic-1)%undef ) cycle

                 dis = 5.

                 if( .not. debug ) then
                    Xs = real( i )
                    Ys = real( j )
                 else
                    Xs = real( 250 )
                    Ys = real( 372 )
                    Xs = real( i )
                    Ys = real( j )
                 end if

                 id = int( MaxDensity * Xs - MaxDensity + 1.5 )
                 jd = int( MaxDensity * Ys - MaxDensity + 1.5 )
                 if( iden(id,jd) > 0 ) cycle
                 iden2 = 0

                 tt = 0
                 if( debug ) write(88,'(f,f)') Xs, Ys

                 do
                    tt = tt + 1
                    if( tt > 10000 ) exit
                    u1 = IDW( Xs, Ys, n, ic-1, IDW_rate )
                    v1 = IDW( Xs, Ys, n, ic  , IDW_rate )

                    X2 = Xs + dis / sqrt( u1*u1 + v1*v1 ) * u1
                    Y2 = Ys + dis / sqrt( u1*u1 + v1*v1 ) * v1

                    if( X2 < 2 .or. X2 > ix-1 ) exit
                    if( Y2 < 2 .or. Y2 > jx-1 ) exit

                    u2 = IDW( X2, Y2, n, ic-1, IDW_rate )
                    v2 = IDW( X2, Y2, n, ic  , IDW_rate )

                    dis = dis * ( u1*u2 + v1*v2 ) / sqrt( u1*u1 + v1*v1 ) / sqrt( u2*u2 + v2*v2 )

                    dis = abs( dis )
                    X2   = Xs + dis / sqrt( u2*u2 + v2*v2 ) * u2
                    Y2   = Ys + dis / sqrt( u2*u2 + v2*v2 ) * v2
                    if( X2 < 2 .or. X2 > ix-1 ) exit
                    if( Y2 < 2 .or. Y2 > jx-1 ) exit

!                    dis = sqrt( (X2-Xs)*(X2-Xs) + (Y2-Ys)*(Y2-Ys) )
                    !if( (X2-Xs)*(X2-Xs)+(Y2-Ys)*(Y2-Ys) < 1.e-5 ) then
!                    if( (X2-Xs)*(X2-Xs)+(Y2-Ys)*(Y2-Ys) < 1.e-4 ) then
                    if( abs(dis) < 0.01 ) then
                       iden = iden + iden2
                       if( debug ) write(87,*) X2, Y2
                       tmp(int(X2+0.5),int(Y2+0.5),k,t) = 1.
                       exit
                    end if

                    id = int( MaxDensity * X2 - MaxDensity + 1.5 )
                    jd = int( MaxDensity * Y2 - MaxDensity + 1.5 )
                    if( iden(id,jd) > 0 ) exit

                    iden2(id,jd) = iden2(id,jd) + 1
                    if( iden2(id,jd) >= 10 ) then
                       Xs = 0.
                       Ys = 0.
                       X2 = 0.
                       Y2 = 0.
                       do jjd = jd-200, jd+200
                          do iid = id-200, id+200
!                       do jjd = 2, jjd-1
!                          do iid = 2, iid-1
                             if( iid > 1 .and. iid < ix .and. jjd > 1 .and. jjd < jx ) then
                                if( iden2(iid,jjd) > 6 ) then
                                   Xs = Xs + real(iid)  * iden2(iid,jjd)
                                   Ys = Ys + real(jjd)  * iden2(iid,jjd)
                                   X2 = X2 + iden2(iid,jjd)
                                   Y2 = Y2 + iden2(iid,jjd)
                                end if
                             end if
                          end do
                       end do
                       X2 = Xs / X2
                       Y2 = Ys / Y2
                       
                       if( X2 > 2 .and. X2 < ix-1 .and. &
                            & Y2 > 2 .and. Y2 < jx-1 ) then
                          iden = iden + iden2
                          if( debug ) write(87,*) X2, Y2
                          tmp(int(X2+0.5),int(Y2+0.5),k,t) = 1.
                          exit
                       end if
                    end if

                    Xs = X2
                    Ys = Y2

!                    if( abs( coef ) < 1.e-5 ) exit
                    if( Xs < 2 .or. Xs > ix-1 ) exit
                    if( Ys < 2 .or. Ys > jx-1 ) exit

                    if( debug .and. mod(tt,1) == 0 ) write(88,'(7f)') Xs, Ys, dis, real(iden2(id,jd))
                 end do

!                 if( debug ) stop
                 if( debug ) write(88,'(a)') '>'

              end do
           end do
        end do
     end do

     itpmethod = itpmethod_tmp

     stacks(ic-1)%val = tmp
     call RPN_ClearStacks( stacks, MaxStacks )

     deallocate( iden  )
     deallocate( iden2 )
  end if

  

  deallocate( tmp )
  FuncCenterPoint = RPN_NOERROR
  return
contains
  include 'func_inner_idw.f90'
end function FuncCenterPoint
