!
!  mode:  1 X/Y
!         2 LON/LAT (defined by map data)
!         3 LON/LAT (defined by last 2-stacks)
!
!  mode2:     1 meridional
!             2 zonal
!             3 great circle
!         other simply linear division
!
integer function FuncCross( stacks, MaxStacks, mode,  &
     &                      x1, y1, x2, y2, nc, mode2, no )
  use RPN
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mode
  real           , intent(IN)    :: x1
  real           , intent(IN)    :: y1
  real           , intent(IN)    :: x2
  real           , intent(IN)    :: y2
  integer        , intent(IN)    :: nc
  integer        , intent(IN)    :: mode2
  integer        , intent(IN)    :: no

  integer :: ic, ico
  integer :: i, j, k, t, n, nn, iv
  integer :: i1, j1, ii, jj
  integer :: ix, jx, kx, it

  real :: rate_i, rate_j
  real :: x, y, dnx, dny
  real :: a, b, c, d, e

  real, dimension(4,4)                :: stl
  real, dimension(:,:,:), allocatable :: val

  FuncCross = RPN_FUNC_OPE_ERROR

  ic = stacks(1)%current
  if( no < 1 ) return
  if( mode == 3 .and. ic < 3 + no ) return
  if( ic < 1 + no ) return
  if( nc < 3 ) return

  if( mode == 3 ) then
     ico = ic - 2
  else
     ico = ic
  end if

  ix = stacks(ico)%ix
  jx = stacks(ico)%jx
  kx = stacks(ico)%kx
  it = stacks(ico)%it
  if( mode == 3 ) then
     if( stacks(ic  )%ix /= ix .or. stacks(ic  )%jx /= jx ) return
     if( stacks(ic-1)%ix /= ix .or. stacks(ic-1)%jx /= jx ) return
  end if

  do iv = ico-no+1, ico-1
     if(    stacks(iv)%ix /= ix .or. &
          & stacks(iv)%jx /= jx .or. &
          & stacks(iv)%kx /= kx .or. &
          & stacks(iv)%it /= it      ) return
  end do

  if( mode == 1 ) then
     if(    x1 < 0 .or. x1 > ix .or. x2 < 0 .or. x2 > ix .or. &
          & y1 < 0 .or. y1 > jx .or. y2 < 0 .or. y2 > jx      ) return

     allocate( val(nc,kx,it) )

     dnx = ( x2 - x1 ) / real( nc - 1 )
     dny = ( y2 - y1 ) / real( nc - 1 )
     do iv = ico-no+1, ico
        do n = 1, nc
           x = x1 + real( n - 1 ) * dnx
           y = y1 + real( n - 1 ) * dny

           i1 = int( x + 0.00001 )
           j1 = int( y + 0.00001 )
           rate_i = x - real( i1 )
           rate_j = y - real( j1 )

           do t = 1, it
              KLOOP: do k = 1, kx
                 nn = 0
                 do j = 1, 4
                    jj = j1 + j - 2
                    do i = 1, 4
                       stl(i,j) = 0.
                       ii = i1 + i - 2
                       if( ii < 1 .or. ii > ix ) cycle
                       if( jj < 1 .or. jj > jx ) cycle
                       if( stacks(iv)%val(ii,jj,k,t) == stacks(iv)%undef ) then
                          val(n,k,t) = stacks(iv)%undef
                          cycle KLOOP
                       end if

                       stl(i,j) = stacks(iv)%val(ii,jj,k,t)
                       nn = nn + 1
                       if( stl(i,j) == 0. ) stl(i,j) = 1.e-20
                    end do
                 end do

                 a = oned( rate_i, stl(1,1), stl(2,1), stl(3,1), stl(4,1) )
                 b = oned( rate_i, stl(1,2), stl(2,2), stl(3,2), stl(4,2) )
                 c = oned( rate_i, stl(1,3), stl(2,3), stl(3,3), stl(4,3) )
                 d = oned( rate_i, stl(1,4), stl(2,4), stl(3,4), stl(4,4) )
                 e = oned( rate_j, a, b, c, d )

                 if( nn == 16 ) then
                    val(n,k,t) = e
                 else
                    a = oned( rate_j, stl(1,1), stl(1,2), stl(1,3), stl(1,4) )
                    b = oned( rate_j, stl(2,1), stl(2,2), stl(2,3), stl(2,4) )
                    c = oned( rate_j, stl(3,1), stl(3,2), stl(3,3), stl(3,4) )
                    d = oned( rate_j, stl(4,1), stl(4,2), stl(4,3), stl(4,4) )
                    val(n,k,t) = ( e + oned( rate_i, a, b, c, d ) ) * 0.5
                 end if
              end do KLOOP
           end do
        end do

        stacks(iv)%ix = nc
        stacks(iv)%jx =  1
!        if( associated( stacks(iv)%val ) ) nullify( stacks(iv)%val )
        if( associated( stacks(iv)%val ) ) deallocate( stacks(iv)%val )

        allocate( stacks(iv)%val(nc,1,kx,it) )
        do t = 1, it
           do k = 1, kx
              do i = 1, nc
                 stacks(iv)%val(i,1,k,t) = val(i,k,t)
              end do
           end do
        end do

     end do

  else if( mode == 2 ) then
     return

  else if( mode == 3 ) then
     return
  end if


  if( allocated( val ) ) deallocate( val )
  FuncCross = RPN_NOERROR
  return
contains
  real function oned( x, a, b, c, d )
    real, intent(INOUT) :: x
    real, intent(IN)    :: a, b, c, d

    oned = 0.
    if( x == 0. ) oned = b
    if( x == 1. ) oned = c
    if( b*c == 0. ) return

    if( a*d /= 0. ) then
       oned = ( 1.0 - x ) *                                                 &
            & ( b + x * ( 0.5 * ( c - a ) + x * ( 0.5 * ( c + a ) - b ) ) ) &
            & + x * ( c + ( 1.0 - x ) *                                     &
            & ( 0.5 * ( b - d ) + ( 1.0 - x ) * ( 0.5 * ( b + d ) - c ) ) )

    else
       oned = b * ( 1.0 - x ) + c * x
       if( a /= 0.0 ) &
            & oned = b + x * ( 0.5 * ( c - a ) + x * ( 0.5 * ( c + a ) - b ) )
       if( d /= 0.0 ) &
            & oned = c + ( 1.0 - x ) * ( 0.5 * (b - d) +(1.0-x)*(0.5*(b+d)-c) )
    end if
    return
  end function oned
end function FuncCross
