integer function FuncDDR( stacks, MaxStacks, mode, x, y, no )
  use RPN
  use ModelInfo
  use namelist
  use Geographic
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mode
  real           , intent(IN)    :: x
  real           , intent(IN)    :: y
  integer        , intent(IN)    :: no

  integer :: ic
  integer :: ix
  integer :: jx
  integer :: kx
  integer :: it
  integer :: i
  integer :: j
  integer :: k
  integer :: t
  integer :: m
  integer :: v
  integer :: itpmethod_tmp

  real :: x0
  real :: y0
  real :: x1
  real :: y1
  real :: x2
  real :: y2
  real :: dr
  real :: dr2
  real :: r2d
  real :: rtmp
  real :: v1
  real :: v2
  real :: IDW_rate
  real :: n
  
  real, dimension(:,:,:,:), allocatable :: dadr

  r2d = 57.2957795130823e0

  FuncDDR = RPN_FUNC_OPE_ERROR

  ic = stacks(1)%current
  if( ic < no+1 ) return

  m  = stacks(ic)%model
  ix = stacks(ic)%ix
  jx = stacks(ic)%jx
  kx = stacks(ic)%kx
  it = stacks(ic)%it

  n        = itparg1
  IDW_rate = MODEL(m)%dy / MODEL(m)%dx
  itpmethod_tmp = itpmethod
  if( abs(itpmethod) == 1 ) itpmethod = 2

  do i = ic-no+1, ic-1
     if(    stacks(i)%ix /= ix .or.                                         &
          & stacks(i)%jx /= jx .or.                                         &
          & stacks(i)%kx /= kx .or.                                         &
          & stacks(i)%it /= it .or.                                         &
          & stacks(i)%model /= m ) return
  end do

  ! memory allocation
  allocate( dadr(ix,jx,kx,it) )
  
  if( MODEL(m)%cproj(1:6) == 'LATLON' ) then
     FuncDDR = RPN_UNDER_CONSTRUCTION
     itpmethod = itpmethod_tmp
     deallocate( dadr )
     return

  else
     dr  = sqrt( MODEL(m)%dx*MODEL(m)%dx + MODEL(m)%dy*MODEL(m)%dy )
     dr2 = 2.0 * dr
     if( mode == 1 ) then
        x0 = x
        y0 = y
     else if( mode == 2 ) then
        call ll2xy( x,                                                      &
             &      y,                                                      &
             &      x0,                                                     &
             &      y0,                                                     &
             &      MODEL(m)%cproj,                                         &
             &      MODEL(m)%dx,                                            &
             &      MODEL(m)%dy,                                            &
             &      MODEL(m)%clon,                                          &
             &      MODEL(m)%clat,                                          &
             &      MODEL(m)%xc,                                            &
             &      MODEL(m)%yc,                                            &
             &      tlat1=MODEL(m)%tlat1,                                   &
             &      tlat2=MODEL(m)%tlat2,                                   &
             &      earth=MODEL(m)%er )
     else
        itpmethod = itpmethod_tmp
        deallocate( dadr )
        return
     end if

     do v = ic-no+1, ic
        do t = 1, it
        do k = 1, kx
           do i = 1, ix
              dadr(i,1,k,t) = stacks(v)%undef
              dadr(i,jx,k,t) = stacks(v)%undef
           end do
           do j = 1, jx
              dadr(1,j,k,t) = stacks(v)%undef
              dadr(ix,j,k,t) = stacks(v)%undef
           end do

        do j = 2, jx-1
        do i = 2, ix-1
           dadr(i,j,k,t) = stacks(v)%undef
           if(    i >= x0 - 1 .and. i <= x0 + 1 .and.                       &
                & j >= y0 - 1 .and. j <= y0 + 1 ) cycle
           if( stacks(v)%val(i,j,k,t) == stacks(v)%undef ) cycle

           rtmp = sqrt( (real(i)-x0)*(real(i)-x0) + (real(j)-y0)*(real(j)-y0) )
           x1 = real(i) + dr * ( real(i) - x0 ) / rtmp / MODEL(m)%dx
           y1 = real(j) + dr * ( real(j) - y0 ) / rtmp / MODEL(m)%dy
           x2 = real(i) - dr * ( real(i) - x0 ) / rtmp / MODEL(m)%dx
           y2 = real(j) - dr * ( real(j) - y0 ) / rtmp / MODEL(m)%dy

           if(     x1 < 1.0 .or. x1 > ix .or.                               &
                &  y1 < 1.0 .or. y1 > jx .or.                               &
                &  x2 < 1.0 .or. x2 > ix .or.                               &
                &  y2 < 1.0 .or. y2 > jx ) cycle

           v1 = IDW( x1, y1, n, v, IDW_rate )
           v2 = IDW( x2, y2, n, v, IDW_rate )


           if( v1 == stacks(v)%undef .or. v2 == stacks(v)%undef ) cycle

           dadr(i,j,k,t) = ( v1 - v2 ) / dr2
        end do
        end do
        end do
        end do

        do t = 1, it
        do k = 1, kx
        do j = 1, jx
        do i = 1, ix
           stacks(v)%val(i,j,k,t) = dadr(i,j,k,t)
        end do
        end do
        end do
        end do
     end do

  end if

  itpmethod = itpmethod_tmp
  FuncDDR = RPN_NOERROR


  deallocate( dadr )

  return
contains
  include "func_inner_idw.f90"
end function FuncDDR
