integer function FuncHconv( stacks, MaxStacks, mno, no )
  use RPN
  use ModelInfo
  use namelist
  use GeoGraphic
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mno
  integer        , intent(IN)    :: no

  integer :: ic, m, itpm
  integer :: im, is
  integer :: i, j, k, t
  integer :: ix, jx, kx, it
  integer :: ixo, jxo, kxo, ito
  integer :: ixs, jxs, jkx, its

  real :: lon, lat, x, y, xo, yo, n
  real :: d2r, IDW_rate, IDW_rate_tmp

  real, dimension(:,:,:,:), allocatable :: tmp


  FuncHconv = RPN_FUNC_OPE_ERROR

  d2r  = 0.01745329252   ! 3.14159265 / 180.
  ic   = stacks(1)%current
  n    = itparg1
  itpm = itpmethod
  if( itpmethod == 1  ) itpmethod = 3
  if( itpmethod == -1 ) itpmethod = 1
  if( ic < no+1 ) return
  if( MODEL(mno)%cproj == 'NOPROJ' ) return

  ix = MODEL(mno)%ix
  jx = MODEL(mno)%jx
  kx = MODEL(mno)%kx
  it = 1
  allocate( tmp(ix,jx,kx,it) )

  do im = 1, no
     is = ic - im + 1
     m = stacks(is)%model
     if( m == mno ) cycle

     ! old grid information
     ixo = MODEL(m)%ix
     jxo = MODEL(m)%jx
     if( stacks(is)%ix /= ixo ) return
     if( stacks(is)%jx /= jxo ) return
     kxo = stacks(is)%kx
     ito = stacks(is)%it
     IDW_rate = MODEL(m)%dy / MODEL(m)%dx
     IDW_rate_tmp = IDW_rate

     if( kxo /= kx .or. ito /= it ) then
        deallocate( tmp )
        kx = kxo
        it = ito
        allocate( tmp(ix,jx,kx,it) )
     end if
     tmp = 0.

     do j = 1, jx
        do i = 1, ix
           if( MODEL(mno)%cproj == 'LATLON' ) then
              lon = MODEL(mno)%slon + real(i-1)*MODEL(mno)%dx
              lat = MODEL(mno)%slat + real(j-1)*MODEL(mno)%dy
              IDW_rate_tmp = IDW_rate * cos( lat * d2r )

           else
              call xy2ll( real(i),                &
                   &      real(j),                &
                   &      lon,                    &
                   &      lat,                    &
                   &      MODEL(mno)%cproj,       &
                   &      MODEL(mno)%dx,          &
                   &      MODEL(mno)%dy,          &
                   &      MODEL(mno)%clon,        &
                   &      MODEL(mno)%clat,        &
                   &      MODEL(mno)%xc,          &
                   &      MODEL(mno)%yc,          &
                   &      tlat1=MODEL(mno)%tlat1, &
                   &      tlat2=MODEL(mno)%tlat2, &
                   &      earth=MODEL(mno)%er     )
           end if

           if( MODEL(m)%cproj == 'LATLON' ) then
              x = ( lon - MODEL(m)%slon ) / MODEL(m)%dx + 1.
              y = ( lat - MODEL(m)%slat ) / MODEL(m)%dy + 1.
           else
              call ll2xy( lon,                  &
                   &      lat,                  &
                   &      x,                    &
                   &      y,                    &
                   &      MODEL(m)%cproj,       &
                   &      MODEL(m)%dx,          &
                   &      MODEL(m)%dy,          &
                   &      MODEL(m)%clon,        &
                   &      MODEL(m)%clat,        &
                   &      MODEL(m)%xc,          &
                   &      MODEL(m)%yc,          &
                   &      tlat1=MODEL(m)%tlat1, &
                   &      tlat2=MODEL(m)%tlat2, &
                   &      earth=MODEL(m)%er     )
           end if

           if(  x < 1. .or. x > real(ixo) .or. &
                y < 1. .or. y > real(jxo) ) then
              do t = 1, it
                 do k = 1, kx
                    tmp(i,j,k,t) = stacks(is)%undef
                 end do
              end do

           else
              do t = 1, it
                 do k = 1, kx
                    xo = x
                    yo = y

                    if( int(y) == jxo ) then
                       write(0,'(a,2i,2e,2i)') '**TODO**', i, j, x, y, ixo, jxo
                       tmp(i,j,k,t) = stacks(is)%undef

                    else if( int(x) == ixo ) then
                       write(0,'(a,2i,2e,2i)') '**TODO**', i, j, x, y, ixo, jxo
                       tmp(i,j,k,t) = stacks(is)%undef

                    else
                       tmp(i,j,k,t) = IDW( xo, yo, n, is, IDW_rate_tmp )
                    end if
                 end do
              end do
           end if

        end do
     end do

!     if( associated( stacks(is)%val ) ) nullify( stacks(is)%val )
     if( associated( stacks(is)%val ) ) deallocate( stacks(is)%val )
     allocate( stacks(is)%val(ix,jx,kx,it) )
     stacks(is)%ix    = ix
     stacks(is)%jx    = jx
     stacks(is)%kx    = kx
     stacks(is)%it    = it
     stacks(is)%model = mno
     stacks(is)%val   = tmp

     write(6,*) '***DEBUG***', is, m, itpmethod
  end do
  
  deallocate( tmp )
  itpmethod = itpm
  FuncHconv = RPN_NOERROR
  return
contains
  include "func_inner_idw.f90"
end function FuncHconv
