  real function IDW( x, y, n, v, rate )
    real   , intent(INOUT) :: x, y
    real   , intent(IN)    :: n
    integer, intent(IN)    :: v
    real   , intent(IN)    :: rate

    integer :: i1, i2, j1, j2, num, nundef
    real    :: d1, d2, d3, d4, dmin, up, down, r2
    real    :: rx, ry

    num  = 0
    up   = 0.
    down = 0.
    r2   = rate * rate

    i1  = int(x+0.0001)
    i2  = i1 + 1
    j1  = int(y+0.0001)
    j2  = j1 + 1
    rx  = x - real(i1)
    ry  = y - real(j1)

    if( itpmethod == 3 ) then
       if(  stacks(v)%val(i1,j1,k,1) /= stacks(v)%undef .and. &
            stacks(v)%val(i1,j2,k,1) /= stacks(v)%undef .and. &
            stacks(v)%val(i2,j2,k,1) /= stacks(v)%undef .and. &
            stacks(v)%val(i2,j1,k,1) /= stacks(v)%undef ) then
          IDW = ( 1. - ry ) * ( ( 1. - rx ) * stacks(v)%val(i1,j1,k,1)   &
               &                     + rx   * stacks(v)%val(i2,j1,k,1) ) &
               &     + ry   * ( ( 1. - rx ) * stacks(v)%val(i1,j2,k,1)   &
               &                     + rx   * stacks(v)%val(i2,j2,k,1) )
       else
          IDW = stacks(v)%undef
       end if
       return
    end if

    d1 = sqrt( (x-real(i1))*(x-real(i1))+(y-real(j1))*(y-real(j1))*r2 )
    if( d1 < 1.e-5 ) then
       IDW = stacks(v)%val(i1,j1,k,1)
       return
    end if
    d2 = sqrt( (x-real(i2))*(x-real(i2))+(y-real(j1))*(y-real(j1))*r2 )
    if( d2 < 1.e-5 ) then
       IDW = stacks(v)%val(i2,j1,k,1)
       return
    end if
    d3 = sqrt( (x-real(i2))*(x-real(i2))+(y-real(j2))*(y-real(j2))*r2 )
    if( d3 < 1.e-5 ) then
       IDW = stacks(v)%val(i2,j2,k,1)
       return
    end if
    d4 = sqrt( (x-real(i1))*(x-real(i1))+(y-real(j2))*(y-real(j2))*r2 )
    if( d4 < 1.e-5 ) then
       IDW = stacks(v)%val(i1,j2,k,1)
       return
    end if

    ! nearest neighbour
    if( itpmethod == 1 ) then
       dmin   = 10.
       nundef = 0
       IDW    = stacks(v)%undef
       if( stacks(v)%val(i1,j1,k,1) == stacks(v)%undef ) nundef = nundef + 1
       if( stacks(v)%val(i2,j1,k,1) == stacks(v)%undef ) nundef = nundef + 1
       if( stacks(v)%val(i2,j2,k,1) == stacks(v)%undef ) nundef = nundef + 1
       if( stacks(v)%val(i1,j2,k,1) == stacks(v)%undef ) nundef = nundef + 1
       if( nundef > 1 ) return

       if( d1 < dmin .and. stacks(v)%val(i1,j1,k,1) /= stacks(v)%undef ) then
          dmin = d1
          IDW  = stacks(v)%val(i1,j1,k,1)
          x = i1
          y = j1
       end if
       if( d2 < dmin .and. stacks(v)%val(i2,j1,k,1) /= stacks(v)%undef ) then
          dmin = d2
          IDW  = stacks(v)%val(i2,j1,k,1)
          x = i2
          y = j1
       end if
       if( d3 < dmin .and. stacks(v)%val(i2,j2,k,1) /= stacks(v)%undef ) then
          dmin = d3
          IDW  = stacks(v)%val(i2,j2,k,1)
          x = i2
          y = j2
       end if
       if( d4 < dmin .and. stacks(v)%val(i1,j2,k,1) /= stacks(v)%undef ) then
          dmin = d4
          IDW  = stacks(v)%val(i1,j2,k,1)
          x = i1
          y = j2
       end if
       return
    end if

    if( stacks(v)%val(i1,j1,k,1) /= stacks(v)%undef ) then
       up   = up + stacks(v)%val(i1,j1,k,1) / (d1**n)
       down = down + 1. / (d1**n)
       num  = num + 1
    end if

    if( stacks(v)%val(i2,j1,k,1) /= stacks(v)%undef ) then
       up   = up + stacks(v)%val(i2,j1,k,1) / (d2**n)
       down = down + 1. / (d2**n)
       num  = num + 1
    end if

    if( stacks(v)%val(i2,j2,k,1) /= stacks(v)%undef ) then
       up   = up + stacks(v)%val(i2,j2,k,1) / (d3**n)
       down = down + 1. / (d3**n)
       num  = num + 1
    end if

    if( stacks(v)%val(i1,j2,k,1) /= stacks(v)%undef ) then
       up   = up + stacks(v)%val(i1,j2,k,1) / (d4**n)
       down = down + 1. / (d4**n)
       num  = num + 1
    end if

    if( num >= 3 .and. down > 1.e-6 ) then
       IDW = up / down
    else
       IDW = stacks(v)%undef
    end if

    return
  end function IDW
