integer function FuncLatLon( stacks, MaxStacks, mno, mode )
  use RPN
  use ModelInfo
  use Geographic
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mno
  integer        , intent(IN)    :: mode

  integer :: ix
  integer :: jx
  integer :: kx
  integer :: it
  integer :: i
  integer :: j
  

  real, dimension(:,:,:,:), allocatable :: lat
  real, dimension(:,:,:,:), allocatable :: lon

  FuncLatLon = RPN_FUNC_OPE_ERROR

  ix = MODEL(mno)%ix
  jx = MODEL(mno)%jx
  kx = 1
  it = 1

  if( MODEL(mno)%mproj > 3 .or. MODEL(mno)%mproj < 0 ) return

  ! memory allocation
  allocate( lat(ix,jx,kx,it) )
  allocate( lon(ix,jx,kx,it) )
  
  ! lat-lon grid
  if( MODEL(mno)%mproj == 0 ) then
     do j = 1, jx
        do i = 1, ix
           lat(i,j,1,1) = MODEL(mno)%slat + real(j-1) * MODEL(mno)%dy
           lon(i,j,1,1) = MODEL(mno)%slon + real(i-1) * MODEL(mno)%dx
        end do
     end do

  ! map projection
  else
     do j = 1, jx
        do i = 1, ix
           call xy2ll( real(i),                                              &
                &      real(j),                                              &
                &      lon(i,j,1,1),                                         &
                &      lat(i,j,1,1),                                         &
                &      MODEL(mno)%cproj,                                     &
                &      MODEL(mno)%dx,                                        &
                &      MODEL(mno)%dy,                                        &
                &      MODEL(mno)%clon,                                      &
                &      MODEL(mno)%clat,                                      &
                &      MODEL(mno)%xc,                                        &
                &      MODEL(mno)%yc,                                        &
                &      tlat1=MODEL(mno)%tlat1,                               &
                &      tlat2=MODEL(mno)%tlat2,                               &
                &      earth=MODEL(mno)%er                                    )
        end do
     end do
  end if

  FuncLatLon = RPN_NOERROR

  if( mode == 1 ) then
     FuncLatLon = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      lat,                                       &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )
  else if( mode == 2 ) then
     FuncLatLon = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      lon,                                       &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )
  else if( mode == 12 ) then
     FuncLatLon = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      lon,                                       &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )

     FuncLatLon = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      lat,                                       &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )
  else if( mode == 21 ) then
     FuncLatLon = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      lon,                                       &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )

     FuncLatLon = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      lat,                                       &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )
  end if

  deallocate( lat )
  deallocate( lon )

  return
end function FuncLatLon
