integer function FuncParcel( stacks, MaxStacks, mode, zunit, lev1, lev2 )
  use RPN
  use ModelInfo
  use namelist
  use constant
  implicit none

  integer         , intent(IN)    :: MaxStacks
  type(RPN_STACK) , intent(INOUT) :: stacks(MaxStacks)
  character(len=*), intent(IN)    :: mode
  integer         , intent(IN)    :: zunit
  real            , intent(IN)    :: lev1
  real            , intent(IN)    :: lev2

  integer :: ic
  integer :: m
  integer :: ix
  integer :: jx
  integer :: kx
  integer :: it
  integer :: i
  integer :: j
  integer :: k
  integer :: t
  integer :: k1
  integer :: k2

  real :: esm
  real :: esp
  real :: qs
  real :: qe
  real :: ept
  real :: th
  real :: tm
  real :: tp
  real :: pm
  real :: pp
  real :: qvm
  real :: qvp
  real :: dtm
  real :: dtp
  real :: tlcl
  real :: plcl
  real :: tlfc
  real :: plfc
  real :: tlnb
  real :: plnb
  real :: tlift
  real :: plift
  real :: cape
  real :: cin
  real :: tmp
  real :: Rdlnp
  real :: rate1
  real :: rate2
  real :: dp = 100

  real, dimension(:,:,:,:), allocatable :: vtmp

  FuncParcel = RPN_FUNC_OPE_ERROR

  ic = stacks(1)%current
  m  = stacks(ic)%model

  ! require 3 stacks
  if( ic < 4 ) return

  ix = stacks(ic)%ix
  jx = stacks(ic)%jx
  kx = stacks(ic)%kx
  it = stacks(ic)%it

  ! grid check
  do i = ic-2, ic-1
     if(         stacks(i)%ix /= ix .or. stacks(i)%jx /= jx                        &
          & .or. stacks(i)%kx /= kx .or. stacks(i)%it /= it ) return
  end do

  allocate( vtmp(ix,jx,1,it) )
  
  ! the unit of lev1 and lev2 is Pa
  if( zunit == 1 ) then
     do t = 1, it
        do j = 1, jx
           do i = 1, ix
              vtmp(i,j,1,t) = stacks(ic-2)%undef
           end do
        end do
     end do

  ! the unit of lev1 and lev2 is grid indices
  else if( zunit == 2 ) then
     k1 = int( lev1 + 0.0001 )
     k2 = int( lev2 + 0.0001 )
     if( lev1 < 0. ) k1 = 1
     if( k1 < 1 .or. k1 > kx ) return
     if( k2 < 1 .or. k1 > kx ) k2 = kx

     do t = 1, it

        MAIN2J: do j = 1, jx
           MAIN2I: do i = 1, ix

              if( lev1 < 0. ) then
                 k1 = 1
                 do k = 1, kx
                    if(    stacks(ic)%val(i,j,k,t)   /= stacks(ic)%undef   .and. &
                         & stacks(ic-1)%val(i,j,k,t) /= stacks(ic-1)%undef .and. &
                         & stacks(ic-2)%val(i,j,k,t) /= stacks(ic-2)%undef ) then
                       k1 = k
                       exit
                    end if
                 end do
              end if

              do k = k1, k2
                 if(    stacks(ic)%val(i,j,k1,t)   == stacks(ic)%undef   .or. &
                      & stacks(ic-1)%val(i,j,k1,t) == stacks(ic-1)%undef .or. &
                      & stacks(ic-2)%val(i,j,k1,t) == stacks(ic-2)%undef ) then
                    vtmp(i,j,1,t) = stacks(ic-2)%undef
                    cycle MAIN2I
                 end if
              end do

              qe = stacks(ic-1)%val(i,j,k1,t)
              th = stacks(ic-2)%val(i,j,k1,t)                   &
                   & * ( p0 / stacks(ic)%val(i,j,k1,t) )**kappa

              pm = stacks(ic)%val(i,j,k1,t)
              pp = pm - dp

              ! LCL
              do
                 tm = th / ( ( p0 / pm )**kappa )
                 tp = th / ( ( p0 / pp )**kappa )

                 esm = esat( tm )
                 esp = esat( tp )

                 qvm = eps * esm / ( pm - esm )
                 qvp = eps * esp / ( pp - esp )

                 tmp = ( qvm - qe ) * ( qvp - qe )
                 if( tmp <= 0. ) then
                    rate1 = ( qvm - qe ) / ( qvm - qvp )
                    rate2 = ( qe - qvp ) / ( qvm - qvp )

                    tlcl = rate1 * tp + rate2 * tm
                    plcl = rate1 * pp + rate2 * pm
                    
                    exit
                 end if

                 pm = pp
                 pp = pp - dp

                 if( pp < 0. ) then
                    vtmp(i,j,1,t) = stacks(ic-2)%undef
                    cycle MAIN2I
                 end if
              end do

              if( mode(1:4) == 'TLCL' ) then
                 vtmp(i,j,1,t) = tlcl
                 cycle MAIN2I
              else if( mode(1:4) == 'PLCL' ) then
                 vtmp(i,j,1,t) = plcl
                 cycle MAIN2I
              end if

              ept = th * exp( xl * qe / ( cpd * tlcl ) )

              cape = 0.
              cin  = 0.
              plfc = stacks(ic-2)%undef
              plnb = stacks(ic-2)%undef

              pm = stacks(ic)%val(i,j,k1,t)
              if( pm >= plcl ) then
                 tm = th / ( ( p0 / pm )**kappa )
              else
                 tm = inq_wet( ept, pm )
              end if
              do k = k1, kx-1
                 pp = stacks(ic)%val(i,j,k+1,t)

                 if( pp >= plcl ) then
                    tp = th / ( ( p0 / pp )**kappa )
                 else
                    tp = inq_wet( ept, pp )
                 end if

                 Rdlnp = log( pm / pp ) * rd
                 dtm   = tm - stacks(ic-2)%val(i,j,k  ,t)
                 dtp   = tp - stacks(ic-2)%val(i,j,k+1,t)

                 rate1 = abs( dtm )
                 rate2 = abs( dtp )


                 ! LFC
                 if( plfc == stacks(ic-2)%undef ) then
                    if( stacks(ic-2)%val(i,j,k+1,t) == stacks(ic-2)%undef ) cycle MAIN2I
                    if( dtm * dtp < 0. ) then
                       tlfc =   tp * rate1 / ( rate1 + rate2 ) &
                            & + tm * rate2 / ( rate1 + rate2 )
                       plfc =   pp * rate1 / ( rate1 + rate2 ) &
                            & + pm * rate2 / ( rate1 + rate2 )

                       cin  = cin - 0.5 * rd * log( pm / plfc ) * rate1
                       cape = 0.5 * rd * log( plfc / pp ) * rate2
                    else
                       cin = cin - 0.5 * Rdlnp * ( rate1 + rate2 )
                    end if

                 else
                    ! LNB
                    if( plnb == stacks(ic-2)%undef ) then
                       if( stacks(ic-2)%val(i,j,k+1,t) == stacks(ic-2)%undef ) cycle MAIN2I
                       if( dtm == stacks(ic-2)%undef .or. dtp == stacks(ic-2)%undef ) cycle MAIN2I
                       if( dtm * dtp < 0. ) then
                          tlnb =   tp * rate1 / ( rate1 + rate2 ) &
                               & + tm * rate2 / ( rate1 + rate2 )
                          plnb =   pp * rate1 / ( rate1 + rate2 ) &
                               & + pm * rate2 / ( rate1 + rate2 )

                          cape = cape + 0.5 * rd * log( pm / plnb ) * rate1
                       else
                          cape = cape + 0.5 * Rdlnp * ( rate1 + rate2 )
                       end if
                    end if

                 end if

                 plift = pp
                 tlift = tp

                 pm = pp
                 tm = tp
              end do

              if( plnb == stacks(ic-2)%undef ) then
                 cape = stacks(ic-2)%undef
              end if

              if( mode(1:5) == 'PLIFT' ) then
                 vtmp(i,j,1,t) = plift
              else if( mode(1:4) == 'PLFC' ) then
                 vtmp(i,j,1,t) = plfc
              else if( mode(1:4) == 'PLNB' ) then
                 vtmp(i,j,1,t) = plnb
              else if( mode(1:5) == 'TLIFT' ) then
                 vtmp(i,j,1,t) = tlift
              else if( mode(1:4) == 'TLFC' ) then
                 vtmp(i,j,1,t) = tlfc
              else if( mode(1:4) == 'TLNB' ) then
                 vtmp(i,j,1,t) = tlnb
              else if( mode(1:4) == 'CAPE' ) then
                 vtmp(i,j,1,t) = cape
              else if( mode(1:4) == 'CIN' ) then
                 vtmp(i,j,1,t) = cin
              end if

           end do MAIN2I
        end do MAIN2J

     end do
  else
     return
  end if

  if( associated( stacks(ic-2)%val ) ) deallocate( stacks(ic-2)%val )
  stacks(ic-2)%kx = 1
  allocate( stacks(ic-2)%val(ix,jx,1,it) )
  stacks(ic-2)%val(1:ix,1:jx,1,1:it) = vtmp(1:ix,1:jx,1,1:it)

  call RPN_ClearStacks( stacks, MaxStacks )
  call RPN_ClearStacks( stacks, MaxStacks )


  deallocate( vtmp )
  
  FuncParcel = RPN_NOERROR

  return
contains
  real function esat( t )
    real, intent(IN) :: t

    esat = 611.2 * exp( 17.67 * ( t - 273.15 ) / ( t - 29.65 ) )

    return
  end function esat

  real function inq_wet( ept, p )
    real, intent(IN) :: ept
    real, intent(IN) :: p
    
    integer :: i
    
    real :: t2
    real :: d
    real :: w
    real :: x

    t2 = 253.16
    d  = 120.

    do i = 1, 20
       d = d / 2
       w = 0.622 * esat( t2 ) / ( p - esat( t2 ) )
       x = ept - t2 * ( p0 / p )**kappa * exp( xl * w / ( cpd * t2 ) )
       if( abs( x ) < 1e-4 ) exit

       if( x >= 0. ) then
          t2 = t2 + d
       else
          t2 = t2 - d
       end if
    end do

    inq_wet = t2
    
    return
  end function inq_wet
end function FuncParcel
