integer function FuncRtheta( stacks, MaxStacks, mno, mode, mode2, x, y )
  use RPN
  use ModelInfo
  use Geographic
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mno
  integer        , intent(IN)    :: mode
  integer        , intent(IN)    :: mode2
  real           , intent(IN)    :: x
  real           , intent(IN)    :: y

  integer :: ix
  integer :: jx
  integer :: kx
  integer :: it
  integer :: i
  integer :: j

  real :: lat0
  real :: lon0
  real :: lat1
  real :: lon1
  real :: x0
  real :: y0
  real :: x1
  real :: y1
  real :: xx
  real :: yy
  real :: r2d
  
  real, dimension(:,:,:,:), allocatable :: r
  real, dimension(:,:,:,:), allocatable :: theta

  r2d = 57.2957795130823e0

  FuncRtheta = RPN_FUNC_OPE_ERROR

  if( mno < 1 .or. mno > MODEL(1)%TotalInfo ) return

  ix = MODEL(mno)%ix
  jx = MODEL(mno)%jx
  kx = 1
  it = 1

  if( MODEL(mno)%mproj > 3 .or. MODEL(mno)%mproj < 0 ) return

  ! memory allocation
  allocate( r(ix,jx,kx,it)     )
  allocate( theta(ix,jx,kx,it) )
  
  ! lat-lon grid
  if( MODEL(mno)%mproj == 0 ) then
     if( mode == 1 ) then
        lat0 = MODEL(mno)%slat + ( y - 1.0 ) * MODEL(mno)%dy
        lon0 = MODEL(mno)%slon + ( x - 1.0 ) * MODEL(mno)%dx
     else
        lat0 = y
        lon0 = x
     end if

     do j = 1, jx
        do i = 1, ix
           lat1 = MODEL(mno)%slat + real(j-1) * MODEL(mno)%dy
           lon1 = MODEL(mno)%slon + real(i-1) * MODEL(mno)%dx

           call Dis2GPoint( r(i,j,1,1),                                     &
                &           lat0, lon0,                                     &
                &           lat1, lon1,                                     &
                &           earth=MODEL(mno)%er )
           call Azimuth2Point( theta(i,j,1,1),                              &
                &           lat0, lon0,                                     &
                &           lat1, lon1,                                     &
                &           earth=MODEL(mno)%er )
           theta(i,j,1,1) = 90.0 - theta(i,j,1,1)
           if( theta(i,j,1,1) > 180.0 ) theta(i,j,1,1) = theta(i,j,1,1) - 360.0
        end do
     end do

  ! map projection
  else
     if( mode == 1 ) then
        x0 = x
        y0 = y
     else
        call ll2xy( real(x),                                               &
             &      real(y),                                               &
             &      x0,                                                    &
             &      y0,                                                    &
             &      MODEL(mno)%cproj,                                      &
             &      MODEL(mno)%dx,                                         &
             &      MODEL(mno)%dy,                                         &
             &      MODEL(mno)%clon,                                       &
             &      MODEL(mno)%clat,                                       &
             &      MODEL(mno)%xc,                                         &
             &      MODEL(mno)%yc,                                         &
             &      tlat1=MODEL(mno)%tlat1,                                &
             &      tlat2=MODEL(mno)%tlat2,                                &
             &      earth=MODEL(mno)%er                                     )
     end if

     do j = 1, jx
        do i = 1, ix
           xx = ( real(i) - x0 ) * MODEL(mno)%dx
           yy = ( real(j) - y0 ) * MODEL(mno)%dy

           r(i,j,1,1)     = sqrt( xx * xx + yy * yy )
           theta(i,j,1,1) = atan2( yy, xx ) * r2d
        end do
     end do
  end if

  FuncRtheta = RPN_NOERROR

  if( mode2 == 1 ) then
     FuncRtheta = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      r,                                         &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )
  else if( mode2 == 2 ) then
     FuncRtheta = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      theta,                                     &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )
  else if( mode2 == 12 ) then
     FuncRtheta = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      theta,                                     &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )

     FuncRtheta = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      r,                                         &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )
  else if( mode2 == 21 ) then
     FuncRtheta = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      theta,                                     &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )

     FuncRtheta = RPN_PushStack( stacks,                                    &
          &                      MaxStacks,                                 &
          &                      r,                                         &
          &                      ix,                                        &
          &                      jx,                                        &
          &                      kx,                                        &
          &                      it,                                        &
          &                      undef=MODEL(mno)%undef,                    &
          &                      model=mno,                                 &
          &                      omodel=mno                                 )
  end if

  deallocate( r     )
  deallocate( theta )

  return
end function FuncRtheta
