integer function FuncTCpoint( stacks, MaxStacks,                             &
     &                        Pmax, Area, Rmin, Rmax, dth, mode )
  use RPN
  use ModelInfo
  use namelist
  use GeoGraphic
  implicit none

  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  real           , intent(IN)    :: Pmax
  real           , intent(IN)    :: Area
  real           , intent(IN)    :: Rmin
  real           , intent(IN)    :: Rmax
  real           , intent(IN)    :: dth
  integer        , intent(IN)    :: mode

  integer :: ic
  integer :: ix
  integer :: jx
  integer :: kx
  integer :: it
  integer :: ip
  integer :: is
  integer :: ie
  integer :: jp
  integer :: js
  integer :: je
  integer :: i
  integer :: j
  integer :: k
  integer :: t
  integer :: i2
  integer :: j2
  integer :: in
  integer :: jn
  integer :: mno
  integer :: r
  integer :: ir
  integer :: th
  integer :: ith
  integer :: itpm
  integer :: nc
  integer :: imin
  integer :: jmin
  integer :: nidx

  real :: dx
  real :: dy
  real :: dr
  real :: dis
  real :: er
  real :: lat
  real :: lon
  real :: rvar
  real :: radius
  real :: theta
  real :: x
  real :: y
  real :: IDW_rate
  real :: IDW_rate_tmp
  real :: aslp
  real :: aslp2
  real :: rtmp1
  real :: n
  real :: rvar_min

  real :: d2r
  real :: r2d

  integer, dimension(:,:), allocatable :: idx
  real   , dimension(:,:), allocatable :: rtmp2d1
  real   , dimension(:,:), allocatable :: rtmp2d2
  real   , dimension(:,:), allocatable :: rtmp2d3

  d2r = 0.01745329252e0
  r2d = 57.29577951e0

  FuncTCpoint = RPN_FUNC_OPE_ERROR

  ic = stacks(1)%current
  if( ic < 2 ) return

  dr = Rmin
  ir = ( Rmax - Rmin ) / dr + 1

  ith = 360.0 / dth - 1

  ix = stacks(ic)%ix
  jx = stacks(ic)%jx
  kx = stacks(ic)%kx
  it = stacks(ic)%it
  mno = stacks(ic)%model

  n    = itparg1
  itpm = itpmethod
  if( itpmethod == 1 ) itpmethod = 3

  IDW_rate = MODEL(mno)%dy / MODEL(mno)%dx
  IDW_rate_tmp = IDW_rate

  if( abs(MODEL(mno)%er) < 1e-3 .or. abs(MODEL(mno)%er) > 1e+20 ) then
     er = 6.371e+6
  else
     er = MODEL(mno)%er
  end if

  ! memory allocation
  allocate( rtmp2d1(ix,jx) )
  allocate( rtmp2d2(ix,jx) )
  allocate( rtmp2d3(ix,jx) )

  do t = 1, it
     do k = 1, kx
        do j = 1, jx
        do i = 1, ix
           rtmp2d2(i,j) = stacks(ic)%undef
           rtmp2d3(i,j) = stacks(ic)%undef
        end do
        end do

        nidx = 0

        ! search pressure minimum point
        do j = 2, jx-1
        do i = 2, ix-1
           if(    stacks(ic)%val(i-1,j+1,k,t) /= stacks(ic)%undef .and.      &
                & stacks(ic)%val(i  ,j+1,k,t) /= stacks(ic)%undef .and.      &
                & stacks(ic)%val(i+1,j+1,k,t) /= stacks(ic)%undef .and.      &
                & stacks(ic)%val(i-1,j  ,k,t) /= stacks(ic)%undef .and.      &
                & stacks(ic)%val(i  ,j  ,k,t) /= stacks(ic)%undef .and.      &
                & stacks(ic)%val(i+1,j  ,k,t) /= stacks(ic)%undef .and.      &
                & stacks(ic)%val(i-1,j-1,k,t) /= stacks(ic)%undef .and.      &
                & stacks(ic)%val(i  ,j-1,k,t) /= stacks(ic)%undef .and.      &
                & stacks(ic)%val(i+1,j-1,k,t) /= stacks(ic)%undef ) then
              if( stacks(ic)%val(i,j,k,t) > Pmax ) then
                 rtmp2d1(i,j) = 0.0
                 cycle
              end if

              if(    stacks(ic)%val(i,j,k,t)                                 &
                   &            <= stacks(ic)%val(i-1,j+1,k,t) .and.         &
                   & stacks(ic)%val(i,j,k,t)                                 &
                   &            <= stacks(ic)%val(i  ,j+1,k,t) .and.         &
                   & stacks(ic)%val(i,j,k,t)                                 &
                   &            <= stacks(ic)%val(i+1,j+1,k,t) .and.         &
                   & stacks(ic)%val(i,j,k,t)                                 &
                   &            <= stacks(ic)%val(i-1,j  ,k,t) .and.         &
                   & stacks(ic)%val(i,j,k,t)                                 &
                   &            <= stacks(ic)%val(i+1,j  ,k,t) .and.         &
                   & stacks(ic)%val(i,j,k,t)                                 &
                   &            <= stacks(ic)%val(i-1,j-1,k,t) .and.         &
                   & stacks(ic)%val(i,j,k,t)                                 &
                   &            <= stacks(ic)%val(i  ,j-1,k,t) .and.         &
                   & stacks(ic)%val(i,j,k,t)                                 &
                   &            <= stacks(ic)%val(i+1,j-1,k,t) ) then
                 nidx = nidx + 1
                 rtmp2d1(i,j) = real(nidx)
              else
                 rtmp2d1(i,j) = 0.0
              end if

           else
              rtmp2d1(i,j) = 0.0
           end if
        end do
        end do

        if( nidx > 0 ) then
           if( allocated( idx ) ) deallocate( idx )
           allocate( idx(nidx,3) )
           do j = 1, jx
           do i = 1, ix
              if( rtmp2d1(i,j) < 0.5 ) cycle
              idx(int(rtmp2d1(i,j)+1e-3),1) = i
              idx(int(rtmp2d1(i,j)+1e-3),2) = j
              idx(int(rtmp2d1(i,j)+1e-3),3) = int(rtmp2d1(i,j)+1e-3)
           end do
           end do

           if( MODEL(mno)%mproj == 0 ) then
              ! currently, not supported
           else
              do i = 1, nidx-1
              do j = i+1, nidx
                 dx = real(idx(i,1)-idx(j,1)) * MODEL(mno)%dx
                 dy = real(idx(i,2)-idx(j,2)) * MODEL(mno)%dy
                 dis = sqrt( dx*dx + dy*dy )
                 if( dis < 2*Area ) then
                    if( idx(i,3) < idx(j,3) ) then
                       idx(j,3) = idx(i,3)
                    else
                       idx(i,3) = idx(j,3)
                    end if
                 end if
              end do
              end do
           end if
           
        else
           do j = 1, jx
           do i = 1, ix
              stacks(ic)%val(i,j,k,t) = rtmp2d3(i,j)
           end do
           end do
           cycle
        end if

!!$do in = 1, nidx
!!$   write(6,*) '***DEBUG***', in, idx(in,1:3)
!!$end do
!!$stop

        do in = 1, nidx
!!$           if( mode == 1 ) then
!!$              do j = 1, jx
!!$              do i = 1, ix
!!$                 rtmp2d2(i,j) = stacks(ic)%undef
!!$              end do
!!$              end do
!!$           end if

           do jn = 1, nidx
              if( idx(jn,3) /= in ) cycle
              i = idx(jn,1)
              j = idx(jn,2)

              if( MODEL(mno)%mproj == 0 ) then
                 lat = MODEL(mno)%slat + real(j-1) * MODEL(mno)%dy
                 lon = MODEL(mno)%slon + real(i-1) * MODEL(mno)%dx
                 IDW_rate_tmp = IDW_rate * cos(lat*d2r)

                 jp = r2d * Area / er / MODEL(mno)%dy + 1.0e0
                 ip = r2d * Area / er / MODEL(mno)%dy * cos(lat*d2r) + 1.0e0
                 is = max(  1, i-ip )
                 ie = min( ix, i+ip )
                 js = max(  1, j-jp )
                 je = min( jx, j+jp )
                 !...
                 !... currently, not supported
                 !...

              else
                 ip = Area / MODEL(mno)%dx + 1.0e0
                 jp = Area / MODEL(mno)%dy + 1.0e0
                 is = max(    2, i-ip )
                 ie = min( ix-1, i+ip )
                 js = max(    2, j-jp )
                 je = min( jx-1, j+jp )

                 do j2 = js, je
                 do i2 = is, ie
                    if( rtmp2d2(i2,j2) /= stacks(ic)%undef ) cycle

                    dx = real(i2-i) * MODEL(mno)%dx
                    dy = real(j2-j) * MODEL(mno)%dy
                    dis = sqrt( dx*dx + dy*dy )
                    if( dis > Area ) cycle
                    if( stacks(ic)%val(i2,j2,k,t) == stacks(ic)%undef ) cycle

                    rvar = -10.0
                    do r = 1, ir
                       radius = real(r) * dr
                       nc    = 0
                       aslp  = 0.0
                       do th = 0, ith
                          theta = th * dth
                          x = real(i2) + radius*cos(theta*d2r) / MODEL(mno)%dx
                          y = real(j2) + radius*sin(theta*d2r) / MODEL(mno)%dx

                          if(    x < 2.0 .or. x > real(ix-1) .or.            &
                               & y < 2.0 .or. y > real(jx-1) ) cycle

                          rtmp1 = IDW( x, y, n, ic, IDW_rate_tmp )
                          if( rtmp1 == stacks(ic)%undef ) cycle

                          aslp  = aslp + rtmp1
                          nc    = nc + 1
                       end do

                       if( nc < 3 ) cycle

                       aslp  = aslp  / real(nc)
                       aslp2 = 0.0
                       do th = 0, ith
                          theta = th * dth
                          x = real(i2) + radius*cos(theta*d2r) / MODEL(mno)%dx
                          y = real(j2) + radius*sin(theta*d2r) / MODEL(mno)%dx

                          if(    x < 2.0 .or. x > real(ix-1) .or.            &
                               & y < 2.0 .or. y > real(jx-1) ) cycle

                          rtmp1 = IDW( x, y, n, ic, IDW_rate_tmp )
                          if( rtmp1 == stacks(ic)%undef ) cycle

                          aslp2 = aslp2 + ( aslp - rtmp1 )*( aslp - rtmp1 )
                       end do

                       aslp2 = aslp2 / real(nc)
                       if( rvar > 0.0 ) then
                          rvar  = rvar + aslp2
                       else
                          rvar  = aslp2
                       end if

                       rtmp2d2(i2,j2) = rvar
                       if( mode == 2 ) rtmp2d3(i2,j2) = rvar

                    end do
                 end do
                 end do

              end if

           end do

           if( mode == 1 ) then
              imin = 0
              jmin = 0
              rvar_min = stacks(ic)%undef
              do j = 1, jx
              do i = 1, ix
                 if( rtmp2d2(i,j) == stacks(ic)%undef ) cycle
                 if( stacks(ic)%val(i,j,k,t) > Pmax ) cycle
                 if( imin /= 0 ) then
                    if( rvar_min > rtmp2d2(i,j) ) then
                       imin = i
                       jmin = j
                       rvar_min = rtmp2d2(i,j)
                    end if
                 else
                    imin = i
                    jmin = j
                    rvar_min = rtmp2d2(i,j)
                 end if
                 rtmp2d2(i,j) = stacks(ic)%undef
              end do
              end do

              if( imin > 0 ) then
                 rtmp2d3(imin,jmin) = stacks(ic)%val(imin,jmin,k,t)
              end if

           end if

        end do

        do j = 1, jx
        do i = 1, ix
           stacks(ic)%val(i,j,k,t) = rtmp2d3(i,j)
        end do
        end do

        if( allocated( idx ) ) deallocate( idx )
     end do
  end do

  ! memory deallocation
  deallocate( rtmp2d1 )
  deallocate( rtmp2d2 )
  deallocate( rtmp2d3 )

  FuncTCpoint = RPN_NOERROR

  return
contains
  include "func_inner_idw.f90"
end function FuncTCpoint
