!
!  ic-4  ic-3 ic-2 ic-1 ic
! [SST] [PSLV] [P] [T] [QV] FUNC(TCWINSLP,mode)
!
!   ( Original program was written in Fortran77 by Kerry A. Emanuel )
!   converted source code to Fortran90 and expanded 3-D array
!   data by KATO Masaya
!
integer function FuncTCWinSlp( stacks, MaxStacks, mode )
  use RPN
  use ModelInfo
  use namelist
  use constant
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mode

  integer :: i
  integer :: j
  integer :: k
  integer :: t
  integer :: ic
  integer :: ix
  integer :: jx
  integer :: kx
  integer :: it
  integer :: ip
  integer :: nk
  integer :: np
  integer :: idiss
  integer :: status

  real :: ckcd
  real :: sig
  real :: b
  real :: vreduc
  real :: es0
  real :: rs0
  real :: tv1
  real :: tvav
  real :: pm
  real :: capea
  real :: capem
  real :: capems
  real :: toa
  real :: tom
  real :: toms
  real :: tp
  real :: pp
  real :: rp
  real :: rat
  real :: cat
  real :: catfac
  real :: fac
  real :: pnew

  real, dimension(:,:,:,:), allocatable :: vmax
  real, dimension(:,:,:,:), allocatable :: pmin

  real, dimension(:), allocatable :: tvrdif

  ckcd   = 0.9
  sig    = 0.0
  idiss  = 1
  b      = 2.0
  nk     = 1
  vreduc = 0.8

  FuncTCWinSlp = RPN_FUNC_OPE_ERROR

  if( mode /= 1 .and. mode /= 2 .and. mode /= 12 ) return

  ic = stacks(1)%current
  if( ic < 6 ) return

  ix = stacks(ic)%ix
  jx = stacks(ic)%jx
  kx = stacks(ic)%kx
  it = stacks(ic)%it

  do ip = ic-4,ic-3
     if( stacks(ip)%ix /= ix .or. stacks(ip)%jx /= jx ) return
  end do

  do ip = ic-2,ic-1
     if(    stacks(ip)%ix /= ix .or.                                         &
          & stacks(ip)%jx /= jx .or.                                         &
          & stacks(ip)%kx /= kx .or.                                         &
          & stacks(ip)%it /= it ) return
  end do

  allocate( vmax(ix,jx,1,1) )
  allocate( pmin(ix,jx,1,1) )
  allocate( tvrdif(kx)      )

  do t = 1, it
     do j = 1, jx
        LOOPI: do i = 1, ix
           vmax(i,j,1,1) = stacks(ic)%undef
           pmin(i,j,1,1) = stacks(ic)%undef
           if( stacks(ic-4)%val(i,j,1,1) == stacks(ic-4)%undef ) cycle
           if( stacks(ic-3)%val(i,j,1,1) == stacks(ic-3)%undef ) cycle

           es0 = 6.112 * exp( 17.67 * ( stacks(ic-4)%val(i,j,1,1) - 273.15 ) &
                &             / ( stacks(ic-4)%val(i,j,1,1) - 29.65 ) )
           vmax(i,j,1,1) = 0.0
           pmin(i,j,1,1) = stacks(ic-3)%val(i,j,1,1)

           np = 0
           pm = 95000.0

           call cape_( ix,                                                   &
                &      jx,                                                   &
                &      kx,                                                   &
                &      it,                                                   &
                &      i,                                                    &
                &      j,                                                    &
                &      stacks(ic-1)%val(i,j,nk,1),                           &
                &      stacks(ic)%val(i,j,nk,1),                             &
                &      stacks(ic-2)%val(i,j,nk,1),                           &
                &      stacks(ic-1)%val,                                     &
                &      stacks(ic)%val,                                       &
                &      stacks(ic-2)%val,                                     &
                &      stacks(ic-1)%undef,                                   &
                &      stacks(ic)%undef,                                     &
                &      stacks(ic-2)%undef,                                   &
                &      sig,                                                  &
                &      capea,                                                &
                &      toa,                                                  &
                &      status )
           if( status < 0 ) then
              vmax(i,j,1,1) = stacks(ic)%undef
              pmin(i,j,1,1) = stacks(ic)%undef
              cycle LOOPI
           end if

           ITR: do
              tp = stacks(ic-1)%val(i,j,nk,1)
              pp = min( pm, 100000.0 )
              rp = 0.622 * stacks(ic)%val(i,j,nk,1)                          &
                   &     * stacks(ic-3)%val(i,j,1,1)                         &
                   &     / ( pp * ( 0.622 + stacks(ic)%val(i,j,nk,1))        &
                   &            - stacks(ic)%val(i,j,nk,1)                   &
                   &            * stacks(ic-3)%val(i,j,1,1) )
              call cape_( ix,                                                &
                   &      jx,                                                &
                   &      kx,                                                &
                   &      it,                                                &
                   &      i,                                                 &
                   &      j,                                                 &
                   &      tp,                                                &
                   &      rp,                                                &
                   &      pp,                                                &
                   &      stacks(ic-1)%val,                                  &
                   &      stacks(ic)%val,                                    &
                   &      stacks(ic-2)%val,                                  &
                   &      stacks(ic-1)%undef,                                &
                   &      stacks(ic)%undef,                                  &
                   &      stacks(ic-2)%undef,                                &
                   &      sig,                                               &
                   &      capem,                                             &
                   &      tom,                                               &
                   &      status )
              if( status < 0 ) then
                 vmax(i,j,1,1) = stacks(ic)%undef
                 pmin(i,j,1,1) = stacks(ic)%undef
                 cycle LOOPI
              end if

              rat = stacks(ic-4)%val(i,j,1,1) / tom
              if( idiss == 0 ) rat = 1.0

              tp = stacks(ic-4)%val(i,j,1,1)
              pp = min( pm, 100000.0 )
              rp = 0.622 * es0 / ( pp*0.01 - es0 )

              call cape_( ix,                                                &
                   &      jx,                                                &
                   &      kx,                                                &
                   &      it,                                                &
                   &      i,                                                 &
                   &      j,                                                 &
                   &      tp,                                                &
                   &      rp,                                                &
                   &      pp,                                                &
                   &      stacks(ic-1)%val,                                  &
                   &      stacks(ic)%val,                                    &
                   &      stacks(ic-2)%val,                                  &
                   &      stacks(ic-1)%undef,                                &
                   &      stacks(ic)%undef,                                  &
                   &      stacks(ic-2)%undef,                                &
                   &      sig,                                               &
                   &      capems,                                            &
                   &      toms,                                              &
                   &      status )
              if( status < 0 ) then
                 vmax(i,j,1,1) = stacks(ic)%undef
                 pmin(i,j,1,1) = stacks(ic)%undef
                 cycle LOOPI
              end if

              rs0 = rp
              tv1 = stacks(ic-1)%val(i,j,1,1) * ( 1.0                        &
                   &             + stacks(ic)%val(i,j,1,1) / 0.622 )         &
                   &             / ( 1.0 + stacks(ic)%val(i,j,1,1) )
              tvav = 0.5 * ( tv1 + stacks(ic-4)%val(i,j,1,1)                 &
                   &             * ( 1.0 + rs0/0.622 ) / ( 1.0 + rs0 ) )
              cat = capem - capea + 0.5 * ckcd * rat * ( capems - capem )
              cat = max( cat, 0.0 )
              pnew = stacks(ic-3)%val(i,j,1,1) * exp(-cat / ( 287.04 * tvav ))

              if( abs( pnew - pm ) > 20. ) then
                 pm = pnew
                 np = np + 1
                 if( np > 1000 .or. pm < 40000.0 ) then
                    pmin(i,j,1,1) = stacks(ic-3)%val(i,j,1,1)
                    exit ITR
                 else
                    cycle ITR
                 end if
              else
                 catfac = 0.5 * ( 1.0 + 1.0 / b )
                 cat = capem - capea + ckcd * rat * catfac * ( capems - capem )
                 cat = max( cat, 0.0 )
                 pmin(i,j,1,1) = stacks(ic-3)%val(i,j,1,1)                    &
                      &                * exp(-cat/(287.04*tvav))
                 exit ITR
              end if
           end do ITR

           fac = max( 0.0, (capems-capem) )
           vmax(i,j,1,1) = vreduc * sqrt( ckcd * rat * fac )

        end do LOOPI
     end do
  end do

  if( mode == 1 ) then
     stacks(ic-4)%undef = stacks(ic)%undef
     do j = 1, jx
        do i = 1, ix
           stacks(ic-4)%val(i,j,1,1) = pmin(i,j,1,1)
        end do
     end do
     call RPN_ClearStacks( stacks, MaxStacks )
  else if( mode == 2 ) then
     stacks(ic-4)%undef = stacks(ic)%undef
     do j = 1, jx
        do i = 1, ix
           stacks(ic-4)%val(i,j,1,1) = vmax(i,j,1,1)
        end do
     end do
     call RPN_ClearStacks( stacks, MaxStacks )
  else if( mode == 12 ) then
     stacks(ic-4)%undef = stacks(ic)%undef
     stacks(ic-3)%undef = stacks(ic)%undef
     do j = 1, jx
        do i = 1, ix
           stacks(ic-4)%val(i,j,1,1) = vmax(i,j,1,1)
           stacks(ic-3)%val(i,j,1,1) = pmin(i,j,1,1)
        end do
     end do
  end if
  call RPN_ClearStacks( stacks, MaxStacks )
  call RPN_ClearStacks( stacks, MaxStacks )
  call RPN_ClearStacks( stacks, MaxStacks )

  deallocate( vmax )
  deallocate( pmin )
  deallocate( tvrdif )

  FuncTCWinSlp = RPN_NOERROR
  return
contains
  subroutine cape_( ix, jx, kx, it, i, j,                                    &
       &            tp, rp, pp, t, r, p,                                     &
       &            t_undef, r_undef, p_undef,                               &
       &            sig, caped, tob, iflag )
    integer                        , intent(IN)  :: ix
    integer                        , intent(IN)  :: jx
    integer                        , intent(IN)  :: kx
    integer                        , intent(IN)  :: it
    integer                        , intent(IN)  :: i
    integer                        , intent(IN)  :: j
    real                           , intent(IN)  :: tp
    real                           , intent(IN)  :: rp
    real                           , intent(IN)  :: pp
    real   , dimension(ix,jx,kx,it), intent(IN)  :: t
    real   , dimension(ix,jx,kx,it), intent(IN)  :: r
    real   , dimension(ix,jx,kx,it), intent(IN)  :: p
    real                           , intent(IN)  :: t_undef
    real                           , intent(IN)  :: r_undef
    real                           , intent(IN)  :: p_undef
    real                           , intent(IN)  :: sig
    real                           , intent(OUT) :: caped
    real                           , intent(OUT) :: tob
    integer                        , intent(OUT) :: iflag

    integer :: k
    integer :: ncmax
    integer :: jmin
    integer :: nc
    integer :: inb

    real :: Cpd
    real :: Cpv
    real :: Cl
    real :: CpvmCl
    real :: Rv
    real :: Rd
    real :: eps
    real :: alv0
    real :: alv
    real :: esp
    real :: evp
    real :: rh
    real :: S
    real :: chi
    real :: plcl
    real :: tg
    real :: rg
    real :: tlvr
    real :: es
    real :: sl
    real :: em
    real :: sg
    real :: ap
    real :: tgnew
    real :: enew
    real :: rmean
    real :: na
    real :: pa
    real :: pfac
    real :: pma
    real :: pat
    real :: pinb

    Cpd    = 1005.7
    Cpv    = 1870.0
    Cl     = 2500.0
    CpvmCl = Cpv - Cl
    Rv     = 461.5
    Rd     = 287.04
    eps    = Rd / Rv
    alv0   = 2.501e6

    caped = 0.0
    iflag = -1

    if( rp < 1.0e-6 .or. tp < 200.0 ) return
    if( rp == r_undef .or. tp == t_undef .or. pp == p_undef ) return
    if( t(i,j,1,1) == t_undef ) return

    tob   = t(i,j,1,1)

    esp = 6.112 * exp( 17.67 * ( tp - 273.15 ) / ( tp - 29.65 ) )
    evp = rp * pp * 0.01 / ( eps + rp )   ! hPa
    rh  = evp / esp
    rh  = min( rh, 1.0 )
    alv = alv0 + CpvmCl * ( tp - 273.15 )
    S   = ( Cpd + rp * Cl ) * log(tp) - Rd * log(pp-evp*100.0)               &
         &  + alv * rp / tp - rp * Rv * log(rh)

    chi  = tp / ( 1669.0 - 122.0 * rh - tp )
    plcl = pp * ( rh**chi )

    ncmax = 0
    do k = 1, kx
       tvrdif(k) = 0.0
    end do

    jmin = 1e6
    OUTLOOP: do k = 1, kx
       if(    p(i,j,k,1) == p_undef .or.                                     &
            & r(i,j,k,1) == r_undef .or.                                     &
            & t(i,j,k,1) == t_undef ) cycle OUTLOOP
       if( p(i,j,k,1) < 5900.0 .or. p(i,j,k,1) >= pp ) cycle OUTLOOP

       jmin = min( jmin, k )

       if( p(i,j,k,1) >= plcl ) then
          tg = tp * ( p(i,j,k,1) / pp )**(Rd/Cpd)
          rg = rp

          tlvr = tg * ( 1.0 + rg/eps ) / ( 1.0 + rg )
          tvrdif(k) = tlvr - t(i,j,k,1) * ( 1.0 + r(i,j,k,1)/eps )           &
               &                        / ( 1.0 + r(i,j,k,1) )
       else
          tg = t(i,j,k,1)
          es = 6.112 * exp( 17.67 * ( t(i,j,k,1) - 273.15 )                  &
               &                  / ( t(i,j,k,1) - 29.65 ) )
          rg = eps * es / ( p(i,j,k,1)*0.01 - es )

          nc = 0
          INLOOP: do
             nc = nc + 1

             alv = alv0 + CpvmCl * ( tg - 273.15 )
             sl  = ( Cpd + rp*Cl + alv*alv*rg / ( Rv*tg*tg ) ) / tg
             em  = rg * p(i,j,k,1)*0.01 / ( eps + rg )
             sg  = ( Cpd + rp*Cl ) * log(tg) - Rd*log(p(i,j,k,1)-em*100.0)   &
                  &  + alv*rg/tg
             if( nc < 3 ) then
                ap = 0.3
             else
                ap = 1.0
             end if
             tgnew = tg + ap * ( S - sg ) / sl

             if( abs(tgnew-tg) > 0.001 ) then
                tg = tgnew
                enew = 6.112 * exp( 17.67 * ( tg - 273.15 ) / ( tg - 29.65 ) )

                if( nc > 500 .or. enew > ( p(i,j,k,1) - 100.0 ) ) then
                   iflag = 2
                   return
                end if
                rg = eps * enew / ( p(i,j,k,1)*0.01 - enew )
                cycle INLOOP
             end if
             exit INLOOP
          end do INLOOP

          ncmax = max(nc,ncmax)

          rmean = sig*rg + ( 1.0-sig )*rp
          tlvr  = tg * ( 1.0 + rg/eps ) / ( 1.0 + rmean )
          tvrdif(k) = tlvr - t(i,j,k,1) * ( 1.0 + r(i,j,k,1)/eps )           &
               &                        / ( 1.0 + r(i,j,k,1) )
       end if
    end do OUTLOOP

    iflag = 0
    na = 0.0
    pa = 0.0

    inb = 1
    do k = kx,jmin,-1
       if( tvrdif(k) > 0.0 ) inb = max(inb,k)
    end do

    if( inb == 1 ) return

    if( inb > 1 ) then
       do k = jmin+1,inb
          pfac = Rd * ( tvrdif(k)+tvrdif(k-1) )                              &
              &     * ( p(i,j,k-1,1) - p(i,j,k,1) )                          &
              &     / ( p(i,j,k,1) + p(i,j,k-1,1) )
          pa = pa + max(pfac,0.0)
          na = na + min(pfac,0.0)
       end do

       pma = ( pp + p(i,j,jmin,1) )
       pfac = Rd * ( pp - p(i,j,jmin,1) ) / pma
       pa = pa + pfac*max(tvrdif(jmin),0.0)
       na = na - pfac*min(tvrdif(jmin),0.0)

       pat = 0.0
       tob = t(i,j,inb,1)
       if( inb < kx ) then
          pinb = ( p(i,j,inb+1,1)*tvrdif(inb) - p(i,j,inb,1)*tvrdif(inb+1) ) &
               & / ( tvrdif(inb) - tvrdif(inb+1) )
          pat = Rd*tvrdif(inb) * (p(i,j,inb,1) - pinb) / (p(i,j,inb,1) + pinb)
          tob = ( t(i,j,inb,1) * ( pinb - p(i,j,inb+1,1) )                   &
               & + t(i,j,inb+1,1) * ( p(i,j,inb,1) - pinb ) )                &
               & / ( p(i,j,inb,1) - p(i,j,inb+1,1) )
       end if

       caped = pa + pat - na
       caped = max(caped,0.0)
    end if

    return
  end subroutine cape_
end function FuncTCWinSlp
