integer function FuncVint( stacks, MaxStacks, low, high, mode, no )
  use RPN
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  real           , intent(INOUT) :: low
  real           , intent(INOUT) :: high
  integer        , intent(IN)    :: mode
  integer        , intent(IN)    :: no

  integer :: ic, i, j, k, t, iv
  integer :: ix, jx, kx, it
  integer :: kdn, kup
  real :: ks, ke, delta
  real :: rlow, rhigh, tmp, r1d, r2d, r1u, r2u

  real, dimension(:,:,:), allocatable :: vint
  logical :: pls

  FuncVint = RPN_FUNC_OPE_ERROR

  ic = stacks(1)%current
  ks = -1.
  ke = -1.

  if( no < 0 ) return
  if( ic < no+1 ) return
  if( stacks(ic)%kx == 1 ) then
     FuncVint = RPN_NOERROR
     return
  end if

  if( .not. associated( stacks(ic)%olev ) ) then
     FuncVint = RPN_NOT_OUT_LEVELS
     return
  end if

  ix = stacks(ic)%ix
  jx = stacks(ic)%jx
  kx = stacks(ic)%kx
  it = stacks(ic)%it

  ! dimension check
  do i = 2, no
     if(    stacks(ic-i+1)%ix /= ix .or. &
          & stacks(ic-i+1)%jx /= jx .or. &
          & stacks(ic-i+1)%kx /= kx .or. &
          & stacks(ic-i+1)%it /= it    ) return
  end do

  allocate( vint(ix,jx,it) )

  do iv = ic-no+1,ic
     vint = 0.
     pls = .true.
     if( stacks(iv)%olev(2) - stacks(iv)%olev(1) < 0. ) pls = .false.

     ! 1st. level
     if( low == RPN_Undefined ) then
        ks  = 1.0
        low = stacks(iv)%olev(1)
     else
        if( pls ) then
           if( low <= stacks(iv)%olev(1) ) then
              low = stacks(iv)%olev(1)
              ks = 1.0
           end if
        else
           if( low >= stacks(iv)%olev(1) ) then
              low = stacks(iv)%olev(1)
              ks = 1.0
           end if
        end if
     end if

     ! middle levels
     do k = 1, kx-1
        rlow = ( low - stacks(iv)%olev(k) ) * ( low - stacks(iv)%olev(k+1) )
        if( ks < 0 .and. rlow <= 0. ) then
           ks =     ( low                  - stacks(iv)%olev(k) )  &
                & / ( stacks(iv)%olev(k+1) - stacks(iv)%olev(k) ) + k
        end if

        rhigh = ( high - stacks(iv)%olev(k) ) * ( high - stacks(iv)%olev(k+1) )
        if( ke < 0. .and. rhigh <= 0. ) then
           ke =     ( high                 - stacks(iv)%olev(k) )  &
                & / ( stacks(iv)%olev(k+1) - stacks(iv)%olev(k) ) + k
        end if
     end do

     ! top level
     if( high == RPN_Undefined ) then
        high = stacks(iv)%olev(kx)
        ke   = real( kx )
     else
        if( pls ) then
           if( high >= stacks(iv)%olev(stacks(iv)%kx) ) then
              ke   = real( kx )
              high = stacks(iv)%olev(kx)
           end if
        else
           if( high <= stacks(iv)%olev(stacks(iv)%kx) ) then
              ke   = real( kx )
              high = stacks(iv)%olev(kx)
           end if
        end if
     end if

     kdn = int( ks ) + 1
     kup = int( ke )
     kup = min( kx-1, kup )

     r1d = ks - real( int(ks) )
     r2d = real(kdn) - ks
     r1u = ke - real( kup )
     r2u = real( kup+1 ) - ke

     do t = 1, it
        do j = 1, jx
           do i = 1, ix
              if(    stacks(iv)%val(i,j,kdn  ,t) /= stacks(iv)%undef .and. &
                   & stacks(iv)%val(i,j,kdn-1,t) /= stacks(iv)%undef ) then
                 delta = stacks(iv)%olev(kdn) - low
                 tmp    =   r1d * stacks(iv)%val(i,j,kdn  ,t) &
                      &   + r2d * stacks(iv)%val(i,j,kdn-1,t)
                 vint(i,j,t) = vint(i,j,t) &
                      &     + 0.5 * ( tmp + stacks(iv)%val(i,j,kdn,t) ) * delta
              end if

              if(    stacks(iv)%val(i,j,kup+1,t) /= stacks(iv)%undef .and. &
                   & stacks(iv)%val(i,j,kup  ,t) /= stacks(iv)%undef ) then
                 delta = high - stacks(iv)%olev(kup)
                 tmp   =  r1u * stacks(iv)%val(i,j,kup+1,t) &
                      & + r2u * stacks(iv)%val(i,j,kup  ,t)
                 vint(i,j,t) = vint(i,j,t) &
                      &     + 0.5 * ( tmp + stacks(iv)%val(i,j,kup,t) ) * delta
              end if

              do k = kdn, kup-1
                 if(    stacks(iv)%val(i,j,k  ,t) /= stacks(iv)%undef .and. &
                      & stacks(iv)%val(i,j,k+1,t) /= stacks(iv)%undef ) then
                    delta = stacks(iv)%olev(k+1) - stacks(iv)%olev(k)
                    vint(i,j,t) = vint(i,j,t) &
                         &      + 0.5 * ( stacks(iv)%val(i,j,k  ,t)           &
                         &              + stacks(iv)%val(i,j,k+1,t) ) * delta
                 end if
              end do

           end do
        end do
     end do

     stacks(iv)%kx = 1
!     if( associated( stacks(iv)%val ) ) nullify( stacks(iv)%val )
     if( associated( stacks(iv)%val ) ) deallocate( stacks(iv)%val )
     allocate( stacks(iv)%val(ix,jx,1,it) )

     do t = 1, it
        do j = 1, jx
           do i = 1, ix
              stacks(iv)%val(i,j,1,t) = vint(i,j,t)
           end do
        end do
     end do

  end do

  deallocate( vint )

  FuncVint = RPN_NOERROR
  return
end function FuncVint
