!
!  mode:  1 X/Y
!         2 LON/LAT (defined by map data)
!         3 LON/LAT (defined by last 2-stacks)
!
!  mode2:     1 meridional
!             2 zonal
!             3 great circle
!         other simply linear division
!
integer function FuncZcone( stacks, MaxStacks, mode,  &
     &                      x0, y0, azimuth, radius, mode2, mode3, zlev )
  use RPN
  use GeoGraphic
  use ModelInfo
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mode
  real           , intent(IN)    :: x0
  real           , intent(IN)    :: y0
  real           , intent(IN)    :: azimuth
  real           , intent(IN)    :: radius
  integer        , intent(IN)    :: mode2
  integer        , intent(IN)    :: mode3
  real           , intent(IN)    :: zlev

  integer :: ic
  integer :: m
  integer :: ix, jx, kx, it
  integer :: i
  integer :: j
  integer :: i1, i2
  integer :: j1, j2
  integer :: iprj

  real :: leva, levc
  real :: rx, ry
  real :: x, y
  real :: xa, ya
  real :: dis
  real :: tazimuth

  real, dimension(:,:,:,:), allocatable :: zcone
  
  FuncZcone = RPN_FUNC_OPE_ERROR

  tazimuth = tan( azimuth * 3.14159265 / 180. )

  ic = stacks(1)%current
  m  = stacks(ic)%model
  if( ic < 2 ) return

  ix = stacks(ic)%ix
  jx = stacks(ic)%jx
  kx = stacks(ic)%kx
  it = stacks(ic)%it

  if( MODEL(m)%cproj == 'LATLON' ) then
     iprj = 0
  else
     iprj = 1
  end if

  if( mode /= 2 ) then
     x = x0
     y = y0
  else
     ! currently lat/lon grid is unsupported
     if( MODEL(m)%cproj == 'LATLON' ) then
        return
        x = MODEL(m)%slon + ( x0 - 1. ) * MODEL(m)%dx
        y = MODEL(m)%slat + ( y0 - 1. ) * MODEL(m)%dy
     else
        call ll2xy( x0,                                          &
             &      y0,                                          &
             &      x,                                           &
             &      y,                                           &
             &      MODEL(m)%cproj,                              &
             &      MODEL(m)%dx,                                 &
             &      MODEL(m)%dy,                                 &
             &      MODEL(m)%clon,                               &
             &      MODEL(m)%clat,                               &
             &      MODEL(m)%xc,                                 &
             &      MODEL(m)%yc,                                 &
             &      tlat1=MODEL(m)%tlat1,                        &
             &      tlat2=MODEL(m)%tlat2,                        &
             &      earth=MODEL(m)%er                            )
     end if
  end if

  ! point check
  if( x < 1. .or. y < 1. .or. x > ix-1 .or. y > jx-1 ) return

  ! point level ( levc )
  i1 = int( x + 0.0001 )
  i2 = i1 + 1
  j1 = int( y + 0.0001 )
  j2 = j1 + 1
  rx = x - real(i1)
  ry = y - real(j1)
  if(  stacks(ic)%val(i1,j1,1,1) /= stacks(ic)%undef .and. &
       stacks(ic)%val(i1,j2,1,1) /= stacks(ic)%undef .and. &
       stacks(ic)%val(i2,j2,1,1) /= stacks(ic)%undef .and. &
       stacks(ic)%val(i2,j1,1,1) /= stacks(ic)%undef ) then
     levc = ( 1. - ry ) * ( ( 1. - rx ) * stacks(ic)%val(i1,j1,1,1)   &
          &                      + rx   * stacks(ic)%val(i2,j1,1,1) ) &
          &      + ry   * ( ( 1. - rx ) * stacks(ic)%val(i1,j2,1,1)   &
          &                      + rx   * stacks(ic)%val(i2,j2,1,1) )
  else
     return
  end if

  ! memory allocation
  allocate( zcone(ix,jx,1,1) )
  
  ! X/Y grid
  if( iprj == 1 ) then
     do j = 1, jx
        ya = ( real(j) - y ) * MODEL(m)%dy
        do i = 1, ix
           xa = ( real(i) - x ) * MODEL(m)%dx
           dis = sqrt( xa*xa + ya*ya )

           if( dis <= radius ) then
              leva = dis * tazimuth
              if( stacks(ic)%val(i,j,1,1) /= stacks(ic)%undef ) then
                 zcone(i,j,1,1) = leva
              else if( leva > stacks(ic)%val(i,j,1,1) ) then
                 zcone(i,j,1,1) = leva
              else
                 zcone(i,j,1,1) = stacks(ic)%undef
              end if
           else
              zcone(i,j,1,1) = stacks(ic)%undef
           end if
        end do
     end do

  else
     if( allocated( zcone ) ) deallocate( zcone )
     return
  end if

  if( associated( stacks(ic)%val ) ) deallocate( stacks(ic)%val )
  allocate( stacks(ic)%val(ix,jx,1,1) )

  stacks(ic)%kx = 1
  stacks(ic)%it = 1
  do j = 1, jx
     do i = 1, ix
        stacks(ic)%val(i,j,1,1) = zcone(i,j,1,1)
     end do
  end do

  if( allocated( zcone ) ) deallocate( zcone )
  FuncZcone = RPN_NOERROR
  return
end function FuncZcone
