integer function FuncZcoordinate( stacks, MaxStacks, mode, mno, no )
  use string
  use RPN
  use ModelInfo
  use namelist
  implicit none
  integer        , intent(IN)    :: MaxStacks
  type(RPN_STACK), intent(INOUT) :: stacks(MaxStacks)
  integer        , intent(IN)    :: mode
  integer        , intent(IN)    :: mno
  integer        , intent(IN)    :: no

  integer, parameter :: io = 13

  integer :: i, j, k, k2, t, status
  integer :: ix, jx, kx, it, iv
  integer :: Model_No, kxo
  integer :: ic, ip, Type, nstr

  real :: z1st, zinc, zdn, zmd, zup

  logical :: FileExist, skip

  character(len=20)  :: title
  character(len=128) :: buf

  real, dimension(:)      , allocatable :: olevel
  real, dimension(:,:,:,:), allocatable :: oval

  FuncZcoordinate = RPN_FUNC_OPE_ERROR

  ic = stacks(1)%current
  if( no < 0 ) return
  if( ic < no+mode ) return
!  if( mode == 1 .and. ic < no+1 ) return
!  if( mode == 2 .and. ic < no+2 ) return

  ! read level information files
  DEF: do i = 1, MaxDefineFiles
     if( LevelInfoFile(i)(1:1) == '' ) exit

     inquire( file=trim(LevelInfoFile(i)), EXIST=FileExist )
     if( .not. FileExist ) return

     open( io, file=trim(LevelInfoFile(i)), IOSTAT=status )
     if( status /= 0 ) return

     skip = .true.
     OLOOP: do
        ip    = 0
        buf   = ''
        title = ''
        read( io, '(a)', IOSTAT=status ) buf
        if( status /= 0 ) exit

        call ToUpper( buf )
        if( buf(1:1) == '#' ) cycle

        Type = InquireStringType( buf, ip )
        if( Type /= TypeChar ) cycle
        status = InquireCharacter( buf, ip, nstr, title )

        ! Model No
        if( title(1:8) == 'MODEL_NO' ) then
           Type = InquireStringType( buf, ip )
           if( Type /= TypeNumber ) return

           status = InquireNumber( buf, ip, Model_No )
           if( status /= STRING_NOERROR ) return

           if( Model_No == mno ) skip = .false.
           cycle
        end if

        if( skip ) cycle

        if( title(1:6) == 'LEVELS' ) then
           Type = InquireStringType( buf, ip )
           if( Type /= TypeNumber ) return

           status = InquireNumber( buf, ip, kxo )
           if( status /= STRING_NOERROR ) return
           if( kxo < 1 ) return

           allocate( olevel(kxo) )
           cycle
        end if

        if( title(1:4) == 'DATA' ) then
           zinc = 0.
           Type = InquireStringType( buf, ip )
           if( Type == TypeNumber ) then
              status = InquireNumber( buf, ip, z1st )
              Type = InquireStringType( buf, ip )
              if( Type == TypeNumber ) then
                 status = InquireNumber( buf, ip, zinc )
                 do k = 1, kxo
                    olevel(k) = z1st + real(k-1)*zinc
                 end do
                 exit DEF
              end if
           end if

           k = 0
           do
              ip  = 0
              buf = ''
              read( IO, '(a)', IOSTAT=status ) buf
              if( status /= 0 ) return
              if( buf(1:1) == '/' ) then
                 if( k == kxo ) then
                    exit DEF
                 else
                    return
                 end if
              end if

              do
                 Type = InquireStringType( buf, ip )
                 if( Type == TypeNumber ) then
                    k = k + 1
                    if( k <= kxo ) then
                       status = InquireNumber( buf, ip, olevel(k) )
                    end if
                 else if( Type == TypeChar ) then
                    cycle
                 else
                    exit
                 end if
              end do

              if( k == kxo ) exit
           end do
        end if

     end do OLOOP
  end do DEF
  close( io )
  if( .not. allocated( olevel ) ) return

  ix = stacks(ic)%ix
  jx = stacks(ic)%jx
  kx = stacks(ic)%kx
  it = stacks(ic)%it
  do iv = ic-no-mode+2, ic-1
     if(    stacks(iv)%ix /= ix .or. &
          & stacks(iv)%jx /= jx .or. &
          & stacks(iv)%kx /= kx .or. &
          & stacks(iv)%it /= it ) return
  end do

  allocate( oval(ix,jx,kxo,it) )

  if( mode == 1 ) then
     do iv = ic-no+1, ic
        oval = stacks(iv)%undef
        do t = 1, it
           do k = 1, kxo
              zmd = olevel(k)
              do k2 = 1,kx-1
                 zup = stacks(iv)%olev(k2+1)
                 zdn = stacks(iv)%olev(k2  )
                 if( ( zmd - zup ) * ( zmd - zdn ) <= 0. ) then
                    do j = 1, jx
                       do i = 1, ix
                          if(    stacks(iv)%val(i,j,k2,t)                &
                               &    /= stacks(iv)%undef .and.            &
                               & stacks(iv)%val(i,j,k2+1,t)              &
                               &    /= stacks(iv)%undef      ) then
                             oval(i,j,k,t) = ( stacks(iv)%val(i,j,k2+1,t)   &
                                  &              * ( zdn - zmd )            &
                                  &            + stacks(iv)%val(i,j,k2,t)   &
                                  &              * ( zmd - zup ) )          &
                                  &          / ( zdn - zup )
                          end if
                       end do
                    end do
                    exit
                 end if
              end do
           end do
        end do

!        if( associated( stacks(iv)%olev) ) nullify( stacks(iv)%olev )
        if( associated( stacks(iv)%olev) ) deallocate( stacks(iv)%olev )
        allocate( stacks(iv)%olev(kxo) )
        do k = 1, kxo
           stacks(iv)%olev(k) = olevel(k)
        end do

!        if( associated( stacks(iv)%val ) ) nullify( stacks(iv)%val  )
        if( associated( stacks(iv)%val ) ) deallocate( stacks(iv)%val  )
        stacks(iv)%kx = kxo
        allocate( stacks(iv)%val(ix,jx,kxo,it) )
        do t = 1, it
           do k = 1, kxo
              do j = 1, jx
                 do i = 1, ix
                    stacks(iv)%val(i,j,k,t) = oval(i,j,k,t)
                 end do
              end do
           end do
        end do
     end do

  else if( mode == 2 ) then
     do iv = ic-no, ic-1
        oval = stacks(iv)%undef
        do t = 1, it
           do k = 1, kxo
              zmd = olevel(k)
              do j = 1, jx
                 do i = 1, ix
                    do k2 = 1, kx-1
                       if(    stacks(ic)%val(i,j,k2+1,t)          &
                            &    == stacks(ic)%undef .or.         &
                            & stacks(ic)%val(i,j,k2,t)            &
                            &    == stacks(ic)%undef        ) cycle
                       zup = stacks(ic)%val(i,j,k2+1,t)
                       zdn = stacks(ic)%val(i,j,k2  ,t)

                       if( ( zmd - zup ) * ( zmd - zdn ) <= 0. ) then
                          if(    stacks(iv)%val(i,j,k2+1,t)       &
                               &    /= stacks(iv)%undef .and.     &
                               & stacks(iv)%val(i,j,k2,t)         &
                               &    /= stacks(iv)%undef      ) then
                             oval(i,j,k,t) = ( stacks(iv)%val(i,j,k2+1,t)   &
                                  &              * ( zdn - zmd )            &
                                  &            + stacks(iv)%val(i,j,k2,t)   &
                                  &              * ( zmd - zup ) )          &
                                  &          / ( zdn - zup )
                             exit
                          end if
                       end if
                    end do
                 end do
              end do
           end do
        end do

!        if( associated( stacks(iv)%olev ) ) nullify( stacks(iv)%olev )
        if( associated( stacks(iv)%olev ) ) deallocate( stacks(iv)%olev )
        allocate( stacks(iv)%olev(kxo) )
        do k = 1, kxo
           stacks(iv)%olev(k) = olevel(k)
        end do

!        if( associated( stacks(iv)%val ) ) nullify( stacks(iv)%val  )
        if( associated( stacks(iv)%val ) ) deallocate( stacks(iv)%val  )
        stacks(iv)%kx = kxo
        allocate( stacks(iv)%val(ix,jx,kxo,it) )
        do t = 1, it
           do k = 1, kxo
              do j = 1, jx
                 do i = 1, ix
                    stacks(iv)%val(i,j,k,t) = oval(i,j,k,t)
                 end do
              end do
           end do
        end do
     end do
     call RPN_ClearStacks( stacks, MaxStacks )
     
  end if

  deallocate( olevel )
  deallocate( oval   )
  FuncZcoordinate = RPN_NOERROR
  return
end function FuncZcoordinate
