program test
  use GRIB2
  use time_handle

  ! BufferSize
  integer, parameter :: BufferSize = 102400

  ! GRIB2 informations
  type(GRIB2DAT), dimension(:), allocatable :: RAW

  ! GRIB2 file
  character(len=128) :: grib2_file

  ! convert endian
  logical :: ByteSwap = .true.

  integer :: i
  integer :: IO
  integer :: nval
  integer :: status
  integer :: syear
  integer :: smonth
  integer :: sday
  integer :: shour
  integer :: sminute
  integer :: ssecond
  integer :: cyear
  integer :: cmonth
  integer :: cday
  integer :: chour
  integer :: cminute
  integer :: csecond

  real    :: tcoeff
  real    :: csecond_r4
  real(8) :: ctime

  character(len=19) :: cdate

  ! fortran IO unit number
  IO  = 11

  ! file names
  write(6,'(a,$)') 'GRIB2 FILE NAME: '
  read(5,'(a)') grib2_file
  !grib2_file = 'data/Z__C_RJTD_20090115000000_RDR_JMAGPV_Ggis1km_Prr10lv_ANAL_grib2.bin'

  ! inquire 3-D data
  GRIB2_BufferSize = BufferSize
  status = GRIB2_Open( IO, trim(grib2_file), ByteSwap, nval )
  allocate( RAW(nval) )
  status = GRIB2_Scan( IO, RAW, nval, ByteSwap )
  status = GRIB2_Close( io )

  ! inquire time
  syear   = RAW(1)%Sec1%Year
  smonth  = RAW(1)%Sec1%Month
  sday    = RAW(1)%Sec1%Day
  shour   = RAW(1)%Sec1%Hour
  sminute = RAW(1)%Sec1%Minute
  ssecond = RAW(1)%Sec1%Second

  write(6,'(a)') '============================================================================'
  write(6,'(a)') ' NO   NX   NY   DATE                  PC   PN  L1T  L1L  L2T  L2L  FTM  ATM'
  write(6,'(a)') '============================================================================'
  do i = 1, nval
     if( RAW(i)%Sec4%TemplateNumber == 0 ) then
        ctime = get_total_date( syear,                   &
             &                  smonth,                  &
             &                  sday,                    &
             &                  shour,                   &
             &                  sminute,                 &
             &                  real(ssecond),           &
             &                  RAW(i)%Sec1%Year,        &
             &                  RAW(i)%Sec1%Month,       &
             &                  RAW(i)%Sec1%Day,         &
             &                  RAW(i)%Sec1%Hour,        &
             &                  RAW(i)%Sec1%Minute,      &
             &                  real(RAW(i)%Sec1%Second) )
        
        if( RAW(i)%Sec4%GenerationProcess == 2 ) then
           ! minute
           if( RAW(i)%Sec4%TimeUnit == 0 ) then
              tcoeff = 60.
           ! hour
           else if( RAW(i)%Sec4%TimeUnit == 1 ) then
              tcoeff = 3600.
           ! day
           else if( RAW(i)%Sec4%TimeUnit == 2 ) then
              tcoeff = 86400.
!          ! month
!          else if( RAW(i)%Sec4%TimeUnit == 3 ) then
!             tcoeff = 2592000.
!          ! year
!          else if( RAW(i)%Sec4%TimeUnit == 4 ) then
!             tcoeff = 31536000.
           ! 3 hours
           else if( RAW(i)%Sec4%TimeUnit == 10 ) then
              tcoeff = 10800.
           ! 6 hours
           else if( RAW(i)%Sec4%TimeUnit == 11 ) then
              tcoeff = 21600.
           ! 12 hours
           else if( RAW(i)%Sec4%TimeUnit == 12 ) then
              tcoeff = 43200.
           ! second
           else if( RAW(i)%Sec4%TimeUnit == 13 ) then
              tcoeff = 1
           else
              write(6,*) 'currently unsupported'
              stop
           end if
        end if
        ctime = ctime + tcoeff * RAW(i)%Sec4%ForcastTime

     else if( RAW(i)%Sec4%TemplateNumber == 8 .or. &
          &   RAW(i)%Sec4%TemplateNumber == 50008 ) then
        ctime = get_total_date( syear,                       &
             &                  smonth,                      &
             &                  sday,                        &
             &                  shour,                       &
             &                  sminute,                     &
             &                  real(ssecond),               &
             &                  RAW(i)%Sec4%LastYear,        &
             &                  RAW(i)%Sec4%LastMonth,       &
             &                  RAW(i)%Sec4%LastDay,         &
             &                  RAW(i)%Sec4%LastHour,        &
             &                  RAW(i)%Sec4%LastMinute,      &
             &                  real(RAW(i)%Sec4%LastSecond) )

     else
        write(6,*) 'currently unsupported'
        stop
     end if

     call get_now_date( syear,         &
          &             smonth,        &
          &             sday,          &
          &             shour,         &
          &             sminute,       &
          &             real(ssecond), &
          &             ctime,         &
          &             cyear,         &
          &             cmonth,        &
          &             cday,          &
          &             chour,         &
          &             cminute,       &
          &             csecond_r4     )

     csecond = int( csecond_r4 )
     write(cdate,'(i4.4,a,i2.2,a,i2.2,a,i2.2,a,i2.2,a,i2.2)') &
          & cyear, '-', cmonth, '-', cday, '_', &
          & chour, ':', cminute, ':', csecond

     write(6,'(i4,2i5,a21,8i5)') i, &
          &     RAW(i)%Sec3%Nx, &
          &     RAW(i)%Sec3%Ny, &
          &     cdate,                         &
          &     RAW(i)%Sec4%ParameterCategory, &
          &     RAW(i)%Sec4%ParameterNumber, &
          &     RAW(i)%Sec4%FirstSurfaceType, &
          &     RAW(i)%Sec4%FirstSurfaceValue, &
          &     RAW(i)%Sec4%SecondSurfaceType, &
          &     RAW(i)%Sec4%SecondSurfaceValue, &
          &     RAW(i)%Sec4%ForcastTime,       &
          &     RAW(i)%Sec4%TimeRangeLength
  end do
  write(6,'(a)') '============================================================================'
  write(6,'(a)') ' NO   NX   NY   DATE                  PC   PN  L1T  L1L  L2T  L2L  FTM  ATM'
  write(6,'(a)') '============================================================================'

  stop
end program test

