/*
  hopen.c : Open file.

  void hopen(int *unit,int *nunit,const char *fname,int *mode,
             int *err,int *oflag,int len);


    nunit = UNIX file descriptor associated with 'unit'
    fname = UNIX file name 
    mode  = 0 : write only - file will be created if it doesn't exist, 
                           - otherwise will be rewritten
          = 1 : read only
          = 2 : read/write
    err   = 0 : no error opening file.
         != 0 : Error opening file
    oflag =  0 : no notification if file opened OK (errors are printed)
          =  1 : file name and unit number printed (and errors)
          = -1 : no print at all (not even errors)

  */

#include "hlib.h"

void hopen(int *nunit,const char *fname,int *mode,
	   int *err,int *oflag,int len)
{
  /* definition */
  char *name;
  int i,file_len;

  /* initialize */
  file_len = len;

  if((name = (char *)malloc(len + 1)) == NULL){
    printf("Not be allocated\n");
    exit(-1);
  }

  if (*oflag >= 1)
    printf("hopen: File = %s\n\n", fname);

  /* strip trailing blanks and add null character to name */
  for (i = 0; fname[i] != ' ' && fname[i] != '\0' && i < len; ++i)
    name[i] = fname[i];
  name[i] = '\0';

  /* open the file */
  if (*mode == 0)             /* WRITE ONLY */
    *nunit = creat(name, PERMS);
  else if (*mode == 1)        /* READ ONLY */
    *nunit = open(name, O_RDONLY, 0);
  else{                        /* READ/WRITE */
    /* *nunit = open(name, O_RDWR); */
    *nunit = creat(name, PERMS);
  }


  if (*oflag >= 1)
    printf("UNIX File descriptor: %d\n\n", *nunit);
  
  *err = 0;
  if (*nunit == -1) {             /* error opening file */
    if (*oflag >= 0){
      printf("Error opening %s  Error status: %d\n", fname, -2);
      perror("hopen.c");
    };
    *err = -2;
  };
  
}
