!
! Fortran90 module handling MM5 (Ver.1-3) output data
!
!         AUTHOR: KATO Masaya <kato@ep.sci.hokudai.ac.jp>
!  FIRST RELEASE: 2003/04/21
!    LAST UPDATE: 2007/04/24
!
module MM5
  use time_handle
  implicit none
  private

  ! definition of structures
  include "module_mm5_structure.f90"

  ! error code
  include "module_mm5_error_code.f90"

  ! internal variables
  logical :: FileExist
  logical :: debug = .false.
  integer :: status

  ! module procedures
  interface MM5_GetData
     module procedure MM5_GetData_2D
     module procedure MM5_GetData_3D
  end interface

  ! public variables
  public :: MM5Data, MM5_Variables, MM5_V2HEAD, MM5_V3BIG, MM5_Times
  public :: MM5_open, MM5_close
  public :: MM5_Inquire_Variable, MM5_Inquire_Time
  public :: MM5_GetData

contains
  ! I/O
  include "module_mm5_io.f90"

  ! Initialize
  include "module_mm5_initialize.f90"

  ! inquire MM5 header information
  include "module_mm5_info.f90"

  ! inquire MM5 Version 1 or 2 header information
  include "module_mm5_info_v2.f90"

  ! inquire MM5 Version 3 header information
  include "module_mm5_info_v3.f90"

  ! get data from MM5 output file
  include "module_mm5_getdata.f90"

  ! data handling
  include "module_mm5_handle_v2.f90"
  include "module_mm5_handle_v3.f90"

  ! inquire data
  include "module_mm5_inquire.f90"

  subroutine D2C( vdat, ix, jx, kx )
    integer, intent(IN)    :: ix, jx, kx
    real   , intent(INOUT) :: vdat(ix,jx,kx)

    integer :: i, j

    do j = 1, jx-1
       do i = 1, ix-1
          vdat(i,j,:) = ( vdat(i,j,:) + vdat(i+1,j,:) &
               & + vdat(i,j+1,:) + vdat(i+1,j+1,:) ) * .25
       end do
    end do

    return
  end subroutine D2C

  subroutine C2D( vdat, ix, jx, kx)
    integer, intent(IN)    :: ix, jx, kx
    real   , intent(INOUT) :: vdat(ix,jx,kx)

    real, allocatable :: fcrs(:,:,:)

    integer           :: i, j , k
    allocate( fcrs( ix, jx, kx ) )

    fcrs = vdat
    do k=1,kx
       do i=2,ix-1
          do j=2,jx-1
             vdat(i,j,k) = (fcrs(i,j,k) + fcrs(i-1,j,k) &
                  & + fcrs(i,j-1,k) + fcrs(i-1,j-1,k)) * 0.25
          end do
       end do

       do j=2,jx-1
          vdat(1,j,k) = ((fcrs(1,j,k) + fcrs(1,j-1,k)) * 1.5 &
               & - (fcrs(2,j,k) + fcrs(2,j-1,k)) * 0.5) * 0.5
          vdat(ix,j,k) = ((fcrs(ix-1,j,k) + fcrs(ix-1,j-1,k)) * 1.5 &
               & - (fcrs(ix-2,j,k) + fcrs(ix-2,j-1,k)) * 0.5) * 0.5
       end do

       do i=2,ix-1
          vdat(i,1,k) = ((fcrs(i,1,k) + fcrs(i-1,1,k)) * 1.5 &
               & - (fcrs(i,2,k) + fcrs(i-1,2,k)) * 0.5) * 0.5
          vdat(i,jx,k) = ((fcrs(i,jx-1,k) + fcrs(i-1,jx-1,k)) * 1.5 &
               & - (fcrs(i,jx-2,k) + fcrs(i-1,jx-2,k)) * 0.5) * 0.5
       end do

       vdat(1,1,k) = 4. * fcrs(1,1,k) - &
            & ( vdat(1,2,k) + vdat(2,1,k) + vdat(2,2,k) )
       vdat(ix,1,k) = 4. * fcrs(ix-1,1,k) - &
            ( vdat(ix-1,2,k) + vdat(ix-1,1,k) + vdat(ix,2,k) )
       vdat(1,jx,k) = 4. * fcrs(1,jx-1,k) - &
            & ( vdat(1,jx-1,k) + vdat(2,jx,k) + vdat(2,jx-1,k) )
       vdat(ix,jx,k) = 4. * fcrs(ix-1,jx-1,k) - &
            & ( vdat(ix,jx-1,k)+ vdat(ix-1,jx-1,k) + vdat(ix-1,jx,k) )
    end do

    deallocate( fcrs )
    return
  end subroutine C2D
end module MM5
