  ! read 2-D data
  integer function MM5_GetData_2D( io, M5, name, vdat, staggering, ordering, &
       & t, xtime, ctime, mode, ForceCross, Decouple )
    integer                          , intent(IN)    :: io
    type(MM5Data)                    , intent(INOUT) :: M5
    character(len=*)                 , intent(IN)    :: name
    real             , dimension(:,:), intent(INOUT) :: vdat
    character(len=4)    , optional   , intent(INOUT) :: staggering, ordering
    integer             , optional   , intent(IN)    :: t, mode
    real                , optional   , intent(IN)    :: xtime
    character(len=24)   , optional   , intent(IN)    :: ctime
    logical             , optional   , intent(IN)    :: ForceCross, Decouple

    integer           :: it, imode, ix, jx, i, j, status, vid, vid2, flag
    real              :: ixtime, rtmp
    character(len=4)  :: st, or
    logical           :: FC, DC
    real, allocatable :: pstar(:,:)

    imode  = 1
    it     = 1
    ixtime = 0.
    vdat   = 0.
    FC     = .false.
    DC     = .false.

    if( present( ForceCross ) ) FC    = ForceCross
    if( present( Decouple   ) ) DC    = Decouple
    if( present( mode       ) ) imode = mode

    if( imode /= 2 ) rewind( io )
    
    if( present( xtime ) ) then
       ixtime = xtime
       it     = -1
    end if
    if( present( t     ) ) then
       it     = t
       ixtime = -999.
    end if

    status = MM5_Inquire_Variable( M5, name, vid=vid )
    if( status < 0 ) then
       MM5_GetData_2D = MM5_no_such_variable
       if( MM5_GetData_2D /= MM5_no_error ) vdat = 0.
       return
    end if

    if( M5%version == 2 ) then

       MM5_GetData_2D = MM5_SearchVar_V2( io, M5, vid, it, ixtime )
       if( size( vdat, dim=1 ) /= M5%vinfo(vid)%ix .or. &
            & size( vdat, dim=2 ) /= M5%vinfo(vid)%jx ) &
            & MM5_GetData_2D = MM5_invalid_dimensions

       if( MM5_GetData_2D == MM5_no_error ) then
          read( io, IOSTAT=status ) vdat
          if( status /= 0 ) then
             MM5_GetData_2D = MM5_read_error
          end if

          ! seek the head of this time
          do i = 1, vid+1
             backspace io
          end do

          if( FC .and. M5%vinfo(vid)%dot ) call D2C( vdat, M5%ix, M5%jx, 1 )

          if( imode /= 2 ) then
             rewind( io )
             M5%cur_t     = 1
             M5%cur_xtime = M5%tinfo(1)%xtime
          end if

          if( name(1:8) == 'PSEALVLC' .or. name(1:8) == 'PSEALVLD' .or. &
               & name(1:4) == 'PSFC' ) then
             vdat = 100. * vdat          ! hPa -> Pa

          else if( name(1:8) .eq. 'PSTARCRS' ) then
             vdat = 1000. * vdat         ! kPa -> Pa
          end if

       else
          rewind( io )
          M5%cur_t     = 1
          M5%cur_xtime = M5%tinfo(1)%xtime
       end if

       if( MM5_GetData_2D /= MM5_no_error ) vdat = 0.
       return

    else if( M5%version == 3 ) then
       MM5_GetData_2D = MM5_SearchVar_V3( io, M5, name, it, ixtime, 0, &
            & st, or, ix=ix, jx=jx)
       if( present(staggering) ) staggering = st
       if( present(ordering)   ) ordering   = or
       if( size( vdat, dim=1 ) /= ix .or. size( vdat, dim=2 ) /= jx ) &
            & MM5_GetData_2D = MM5_invalid_dimensions
       if( MM5_GetData_2D == MM5_no_error ) then
!          read( io, IOSTAT=status ) ((vdat(i,j),j=1,jx),i=1,ix)
          read( io, IOSTAT=status ) vdat
          if( status /= 0 ) then
             MM5_GetData_2D = MM5_read_error
          end if

          if( FC .and. M5%vinfo(vid)%staggering(1:1) == 'D' ) &
               & call D2C( vdat, M5%ix, M5%jx, 1 )

          if( imode == 2 ) then
             if( M5%cur_t == 1 ) then
                rewind( io )
                M5%cur_xtime = M5%tinfo(1)%xtime
             else
                do
                   read( io ) flag
                   if( flag == 1 ) then
                      read( io ) rtmp
                      read( io ) rtmp
                   else if( flag == 2 ) then
                      exit
                   end if
                end do

                do i = 1, M5%tinfo(M5%cur_t)%skip
                   backspace io
                end do
             end if
          else
             rewind( io )
             M5%cur_t     = 1
             M5%cur_xtime = M5%tinfo(1)%xtime
          end if
       else
          rewind( io )
          M5%cur_t     = 1
          M5%cur_xtime = M5%tinfo(1)%xtime
          if( MM5_GetData_2D /= MM5_no_error ) vdat = 0.
          return
       end if
    else
       if( debug ) write(6,*) 'Unit ', io, ' is not an MM5 data?'
       MM5_GetData_2D = MM5_invalid_format
       if( MM5_GetData_2D /= MM5_no_error ) vdat = 0.
       return
    end if

    if( MM5_GetData_2D /= MM5_no_error ) vdat = 0.
    return
  end function MM5_GetData_2D

  ! read 3-D data
  integer function MM5_GetData_3D( io, M5, name, vdat, staggering, ordering, &
       & t, xtime, ctime, mode, ForceCross, Decouple )
    integer                         , intent(IN)    :: io
    type(MM5Data)                   , intent(INOUT) :: M5
    character(len=*)                , intent(IN)    :: name
    real          , dimension(:,:,:), intent(INOUT) :: vdat
    character(len=4)      , optional, intent(INOUT) :: staggering, ordering
    integer               , optional, intent(IN)    :: t, mode
    real                  , optional, intent(IN)    :: xtime
    character(len=24)     , optional, intent(IN)    :: ctime
    logical               , optional, intent(IN)    :: ForceCross, Decouple

    integer           :: it, imode, ix, jx, kx, i, j, k, vid, vid2, flag
    integer           :: icd, ick
    real              :: ixtime, rtmp
    character(len=4)  :: st, or
    logical           :: FC, DC
    real, allocatable :: pstar(:,:)

    real, dimension(:,:,:), allocatable :: dum3d
    real, dimension(:,:)  , allocatable :: dum2d

    imode  = 1
    it     = 1
    ixtime = 0.
    vdat   = 0.
    FC     = .false.
    DC     = .false.
    vdat   = 0.

    if( present( ForceCross ) ) FC    = ForceCross
    if( present( Decouple   ) ) DC    = Decouple
    if( present( mode       ) ) imode = mode
    
    if( imode /= 2 ) rewind( io )
    if( present( xtime ) ) then
       ixtime = xtime
       it     = -1
    end if
    if( present( t     ) ) then
       it     = t
       ixtime = -999.
    end if

    status = MM5_Inquire_Variable( M5, name, vid=vid )
    if( status < 0 ) then
       MM5_GetData_3D = MM5_no_such_variable
       if( MM5_GetData_3D /= 0 ) vdat = 0.
       return
    end if

    if( M5%version == 2 ) then
       MM5_GetData_3D = MM5_SearchVar_V2( io, M5, vid, it, ixtime )
       if( size( vdat, dim=1 ) /= M5%vinfo(vid)%ix .or. &
            & size( vdat, dim=2 ) /= M5%vinfo(vid)%jx ) &
            & MM5_GetData_3D = MM5_invalid_dimensions

       if( MM5_GetData_3D == MM5_no_error ) then
          if( ( M5%id == 5 .or. M5%id == 6 ) .and. &
               & M5%vinfo(vid)%name(1:8) == 'W       ' ) then
             allocate( dum3d(M5%vinfo(vid)%ix,M5%vinfo(vid)%jx,M5%vinfo(vid)%kx-1) )
             allocate( dum2d(M5%vinfo(vid)%ix,M5%vinfo(vid)%jx) )
             read( io, IOSTAT=status ) dum2d, dum3d
             if( status /= 0 ) then
                MM5_GetData_3D = MM5_read_error
             else
                if( size( vdat, dim=3 ) == M5%kx ) then
                   ick = 0
                   call MM5_Full2Half( vdat, dum2d, dum3d )
                else if( size( vdat, dim=3 ) == M5%vinfo(vid)%kx ) then
                   ick = 1
                   vdat(:,:,1) = dum2d(:,:)
                   do k = 1, M5%kx
                      vdat(:,:,k+1) = dum3d(:,:,k)
                   end do
                else
                   MM5_GetData_3D = MM5_invalid_dimensions
                end if
          
             end if
             deallocate( dum2d )
             deallocate( dum3d )
          else
             if( size( vdat, dim=3 ) == M5%vinfo(vid)%kx ) then
                ick = 0
                read( io, IOSTAT=status ) vdat
                if( status /= 0 ) then
                   MM5_GetData_3D = MM5_read_error
                end if
             else
                MM5_GetData_3D = MM5_invalid_dimensions
             end if
          end if

          ! seek the head of this time
          do i = 1, vid+1
             backspace io
          end do

          if( FC .and. M5%vinfo(vid)%dot ) &
               & call D2C( vdat, M5%ix, M5%jx, size(vdat,dim=3) )
          if( DC .and. M5%vinfo(vid)%coupled ) then
             allocate( pstar( M5%ix, M5%jx ) )
             status = MM5_Inquire_Variable( M5, 'PSTARCRS', vid=vid2 )
             do i =1, vid2
                read( io, IOSTAT=status ) rtmp
             end do
             read( io, IOSTAT=status ) pstar
             if( status /= 0 ) then
                MM5_GetData_3D = MM5_read_error
             end if

             if( M5%vinfo(vid)%dot ) then
                if( FC ) then
                   icd = 1
                else
                   icd = 0
                   call C2D( pstar, M5%ix, M5%jx, 1 )
                end if
             else
                icd = 1
             end if

             do k = 1, M5%kx+ick
                do j = 1,M5%jx-icd
                   do i = 1,M5%ix-icd
                      vdat(i,j,k) = vdat(i,j,k) / pstar(i,j)
                   end do
                end do
             end do

             do i = 1, vid2+1
                backspace io
             end do
             deallocate( pstar )
          end if

          if( imode /= 2 ) then
             rewind( io )
             M5%cur_t     = 1
             M5%cur_xtime = M5%tinfo(1)%xtime
          end if

       else
          rewind( io )
          M5%cur_t     = 1
          M5%cur_xtime = M5%tinfo(1)%xtime
       end if
          
       if( MM5_GetData_3D /= 0 ) vdat = 0.
       return

       if( debug ) write(6,*) 'Sorry, Now under construction.'
       MM5_GetData_3D = MM5_now_under_construction
       if( MM5_GetData_3D /= 0 ) vdat = 0.
       return

    else if( M5%version == 3 ) then
       MM5_GetData_3D = MM5_SearchVar_V3( io, M5, name, it, ixtime, 0, &
            & st, or, ix=ix, jx=jx, kx=kx)
       if( present( staggering ) ) staggering = st
       if( present( ordering   ) ) ordering   = or

       if( MM5_GetData_3D == MM5_no_error .and. ( size( vdat, dim=1 ) /= ix &
            & .or. size( vdat, dim=2 ) /= jx ) ) &
            & MM5_GetData_3D = MM5_invalid_dimensions

       if( MM5_GetData_3D == MM5_no_error ) then
          if( ( M5%id == 5 .or. M5%id == 11 ) .and. &
               & M5%vinfo(vid)%name(1:8) == 'W       ' ) then
             if( size( vdat, dim=3 ) == M5%kx ) then
                allocate( dum3d( M5%ix, M5%jx, M5%kx+1 ) )
                read( io, IOSTAT=status ) dum3d
                if( status /= 0 ) then
                   MM5_GetData_3D = MM5_read_error
                else
                   call MM5_Full2Half( vdat, dum3d(:,:,1), &
                        & dum3d(:,:,2:M5%kx+1) )
                end if
                deallocate( dum3d )

             else if( size( vdat, dim=3 ) == M5%vinfo(vid)%kx ) then
                read( io, IOSTAT=status ) vdat
                if( status /= 0 ) MM5_GetData_3D = MM5_read_error

             else
                MM5_GetData_3D = MM5_invalid_dimensions
             end if

          else

             read( io, IOSTAT=status ) vdat
             if( status /= 0 ) then
                MM5_GetData_3D = MM5_read_error
             end if
          end if

          if( FC .and. M5%vinfo(vid)%staggering(1:1) == 'D' ) &
               & call D2C( vdat, ix, jx, kx )

          if( imode == 2 ) then
             if( M5%cur_t == 1 ) then
                rewind( io )
                M5%cur_xtime = M5%tinfo(1)%xtime

             else
                do
                   read( io, IOSTAT=status ) flag
                   if( flag == 1 ) then
                      read( io ) rtmp
                      read( io ) rtmp
                   else if( flag == 2 ) then
                      exit
                   end if
                end do

                do i = 1, M5%tinfo(M5%cur_t)%skip
                   backspace io
                end do
             end if
          else
             rewind( io )
             M5%cur_t     = 1
             M5%cur_xtime = M5%tinfo(1)%xtime
          end if
       else
          rewind( io )
          M5%cur_t     = 1
          M5%cur_xtime = M5%tinfo(1)%xtime
          if( MM5_GetData_3D /= 0 ) vdat = 0.
          return
       end if
    else
       if( debug ) write(6,*) 'Unit ', io, ' is not an MM5 data?'
       MM5_GetData_3D = MM5_invalid_format
       if( MM5_GetData_3D /= 0 ) vdat = 0.
       return
    end if

    if( MM5_GetData_3D /= 0 ) vdat = 0.
    return
  end function MM5_GetData_3D

  subroutine MM5_Full2Half( vdat, dum2d, dum3d )
    real, dimension(:,:,:), intent(OUT) :: vdat
    real, dimension(:,:)  , intent(IN)  :: dum2d
    real, dimension(:,:,:), intent(IN)  :: dum3d

    integer :: k
    integer :: ix, jx, kx

    ix = size( dum3d, dim=1 )
    jx = size( dum3d, dim=2 )
    kx = size( dum3d, dim=3 )

    vdat(:,:,1) = ( dum2d(:,:) + dum3d(:,:,1) ) * .5
    do k = 2, kx
       vdat(:,:,k) = ( dum3d(:,:,k-1) + dum3d(:,:,k) ) * .5
    end do

    return
  end subroutine MM5_Full2Half
