  integer function MM5_SearchVar_V3( io, M5, name, t, xtime, mode, &
       & staggering, ordering, ix, jx, kx, it )
    integer                   , intent(IN)    :: io
    type(MM5Data)             , intent(INOUT) :: M5
    character(len=*)          , intent(IN)    :: name
    integer                   , intent(IN)    :: t
    real                      , intent(IN)    :: xtime
    integer                   , intent(IN)    :: mode
    character(len=4)          , intent(INOUT) :: staggering, ordering
    integer         , optional, intent(INOUT) :: ix, jx, kx, it

    integer :: i, t1, t2, itmp, clen, flag
    integer           :: ndim
    integer           :: start_index(4), end_index(4)
    real              :: ctime
    character(len=24) :: current_date
    character(len=9)  :: cname
    character(len=25) :: unit
    character(len=46) :: description

    clen = len_trim( name )

    if( t > 0 ) then
       if( t > M5%it ) then
          MM5_SearchVar_V3 = MM5_out_of_t_range
          return
       end if
       MM5_SearchVar_V3 = MM5_SkipTime_V3( io, M5, M5%cur_t, t - M5%cur_t )
       M5%cur_t     = t
       M5%cur_xtime = M5%tinfo(t)%xtime
    else
       if( xtime < M5%tvalid_min .and. xtime > M5%tvalid_max ) then
          MM5_SearchVar_V3 = MM5_out_of_t_range
          return
       end if
       t1 = M5%cur_t
       t2 = -1
       do i = 1, M5%it
          if( abs( M5%tinfo(i)%xtime - xtime ) < 0.001) then
             t2 = i
             exit
          end if
       end do
       if( t2 < 0 ) then
          MM5_SearchVar_V3 = MM5_no_such_xtime
          return
       end if

       MM5_SearchVar_V3 = MM5_SkipTime_V3( io, M5, t1, t2 - t1 )
       M5%cur_t     = t2
       M5%cur_xtime = M5%tinfo(t2)%xtime
    end if

    do
       read( io, IOSTAT=status ) flag
       if( flag == 0 ) then
          read( io ) itmp
       else if( flag == 1 ) then
          read( io, IOSTAT=status ) ndim, start_index, end_index, ctime, &
               & staggering, ordering, current_date, cname, unit, description

          if( name(1:clen) == cname(1:clen) ) then
             MM5_SearchVar_V3 = MM5_no_error
             if( present(ix) ) ix = end_index(1) - start_index(1) + 1
             if( present(jx) ) jx = end_index(2) - start_index(2) + 1
             if( present(kx) ) kx = end_index(3) - start_index(3) + 1
             if( present(it) ) it = end_index(4) - start_index(4) + 1
             return
          else
             read( io ) itmp
          end if
       else
          MM5_SearchVar_V3 = MM5_no_such_variable
          return
       end if
    end do

    return
  end function MM5_SearchVar_V3

  integer function MM5_SkipTime_V3( io, M5, t, skip )
    integer          , intent(IN)    :: io
    type(MM5Data)    , intent(INOUT) :: M5
    integer          , intent(IN)    :: t, skip

    integer :: i, total, itmp

    if( skip > 0 ) then         ! forward skip
       total = 0
       do i = t, t + skip - 1
          total = total + M5%tinfo(i)%skip
       end do
       do i = 1, total
          read( io, IOSTAT=status ) itmp
          if( status /= 0 ) then
             MM5_SkipTime_V3 = MM5_read_error
             return
          end if
       end do
       
    else if( skip < 0 ) then    ! backward skip
       total = 0
       do i = t-1, t+skip, -1
          total = total + M5%tinfo(i)%skip
       end do
       do i = 1, total
          backspace io
       end do
    end if

    MM5_SkipTime_V3 = MM5_no_error
    return
  end function MM5_SkipTime_V3
