  integer function MM5_GetInfo_V2( io, M5 )
    integer      , intent(IN)    :: io
    type(MM5Data), intent(INOUT) :: M5

    type(MM5_V2HEAD) :: v2h

    integer           :: i, j, itmp
    integer           :: yr, mt, dy, hr
    character(len=80) :: fmt1, fmt2

    fmt1 = "(i2.2,i2.2,'-',i2.2,'-',i2.2,'_',i2.2,':',i2.2,':',i2.2,'.',i4.4)"
    fmt2 = "(i4.4,'-',i2.2,'-',i2.2,'_',i2.2,':',i2.2,':',i2.2,'.',i4.4)"

    rewind( io )

!    if( allocated( M5%v2head ) ) then
!       MM5_GetInfo_V2 = MM5_already_associated_v2head
!       return
!    end if

!    ! allocate V2 header
!    allocate( M5%v2head )

    read( io, IOSTAT=status ) M5%v2head%mif, M5%v2head%mrf, &
         & M5%v2head%mifc, M5%v2head%mrfc
    if( status /= 0 ) then
       if( debug ) write(6,*) 'Unit ', io, ' is not a MM5 data?.'
       MM5_GetInfo_V2 = MM5_invalid_format
       return
    end if

    M5%id = M5%v2head%mif(  1,1)
    M5%ix = M5%v2head%mif(104,1)
    M5%jx = M5%v2head%mif(105,1)
    M5%ds = M5%v2head%mrf(101,1) * 1000.

    select case ( M5%id )
    case ( 1 )
       M5%cid = 'TERRAIN   '
    case ( 2 )
       M5%cid = 'DATAGRID  '
    case ( 3 )
       M5%cid = 'RAWINS    '
    case ( 5 )
       M5%cid = 'MMINPUT   '
    case ( 6 )
       M5%cid = 'MMOUTPUT  '
    case ( 7 )
       M5%cid = 'INTERPB   '
    case default
       M5%cid = 'MISC      '
    end select

    if( M5%id > 1 ) then
       M5%ptop = M5%v2head%mrf(1,2) * 100.
       if( M5%id >= 5 ) then
          M5%ps0 = M5%v2head%mrf(2,M5%id)
          M5%ts0 = M5%v2head%mrf(3,M5%id)
          M5%tlp = M5%v2head%mrf(4,M5%id)
       end if
    end if

    if( M5%id >= 5 .and. M5%id /= 7 ) then
       M5%kx = int( M5%v2head%mrf(101,M5%id) )
    else if( M5%id == 7 ) then
       M5%kx = M5%v2head%mif(101,M5%id)
    else
       if( M5%v2head%mif(101,1) == 1 ) then
          if( M5%v2head%mifc(1,1)(1:28) /= &
               & 'Output from Program little_r' ) then
             if( M5%v2head%mif(5,1) == 1 ) then
                M5%ix = M5%v2head%mif(6,1)
                M5%jx = M5%v2head%mif(7,1)
             end if
          end if
       end if
       if( M5%id == 1 ) then
          M5%kx = 1
       else
          M5%kx = M5%v2head%mif(101,M5%id)
       end if
    end if

    ! vertical levels
    if( associated( M5%level ) ) then
       MM5_GetInfo_V2 = MM5_already_associated_level
       return
    else
       allocate( M5%level( M5%kx ) )
    end if

    if( M5%id == 5 .or. M5%id == 6 ) then
       do i = 1, M5%kx
          M5%level(i) = M5%v2head%mrf(101+i,M5%id)
       end do
    else if( M5%id == 2 .or. M5%id == 3 .or. M5%id == 7 ) then
       do i = 1, M5%kx
          M5%level(i) = real( M5%v2head%mif(101+i,M5%id) ) * 100.   ! Pa
       end do
    end if

    ! variables informations
    M5%nv = M5%v2head%mif(201,M5%id) + M5%v2head%mif(202,M5%id) &
         & + M5%v2head%mif(203,M5%id) + M5%v2head%mif(204,M5%id)

    if( associated( M5%vinfo ) ) then
       MM5_GetInfo_V2 = MM5_already_associated_varinfo
       return
    else
       allocate( M5%vinfo( M5%nv ) )
    end if

    ! 3-D variables
    do i = 1, M5%V2head%mif(201,M5%id)
       M5%vinfo(i)%ndim = 3
       M5%vinfo(i)%ix   = M5%ix
       M5%vinfo(i)%jx   = M5%jx
       M5%vinfo(i)%kx   = M5%kx
       M5%vinfo(i)%name = M5%v2head%mifc(204+i,M5%id)(1:8)
       if( ( M5%id == 5 .or. M5%id == 6 ) .and. &
            & M5%vinfo(i)%name(1:8) == 'W       ' ) M5%vinfo(i)%kx = M5%kx + 1
       if( mod( M5%v2head%mif(204+i,M5%id), 10 ) == 1 ) then
          M5%vinfo(i)%coupled = .true.
       else
          M5%vinfo(i)%coupled = .false.
       end if
       if( M5%v2head%mif(204+i,M5%id) / 10 == 1 ) then
          M5%vinfo(i)%dot = .true.
       else
          M5%vinfo(i)%dot = .false.
       end if
    end do

    ! 2-D variables
    do i = M5%v2head%mif(201,M5%id)+1, &
         & M5%v2head%mif(201,M5%id)+M5%v2head%mif(202,M5%id)
       M5%vinfo(i)%ndim = 2
       M5%vinfo(i)%ix   = M5%ix
       M5%vinfo(i)%jx   = M5%jx
       M5%vinfo(i)%kx   = -1
       M5%vinfo(i)%name = M5%v2head%mifc(204+i,M5%id)(1:8)
       if( mod( M5%v2head%mif(204+i,M5%id), 10 ) == 1 ) then
          M5%vinfo(i)%coupled = .true.
       else
          M5%vinfo(i)%coupled = .false.
       end if
       if( M5%v2head%mif(204+i,M5%id) / 10 == 1 ) then
          M5%vinfo(i)%dot = .true.
       else
          M5%vinfo(i)%dot = .false.
       end if
    end do

    backspace io

    M5%it = 0
    do
       read( io, IOSTAT=status ) v2h%mif, v2h%mrf, v2h%mifc, v2h%mrfc
       if( status /= 0 ) exit
       do i = 1, M5%nv
          read( io, IOSTAT=status ) itmp
       end do
       M5%it = M5%it + 1
    end do

    rewind( io )

    if( associated( M5%tinfo ) ) then
       MM5_GetInfo_V2 = MM5_already_associated_tinfo
       return
    else
       allocate( M5%tinfo(M5%it) )
    end if

    j = 0
    do
       j = j + 1
       read( io, IOSTAT=status ) v2h%mif, v2h%mrf, v2h%mifc, v2h%mrfc
       if( status /= 0 ) exit
       do i = 1, M5%nv
          read( io, IOSTAT=status ) itmp
       end do
       M5%tinfo(j)%skip  = M5%nv + 1
       if( M5%id == 1 ) cycle
       if( M5%id < 5 ) then
          M5%tinfo(j)%xtime = real( get_total_date( v2h%mif(31,M5%id), &
               & v2h%mif(32,M5%id), v2h%mif(33,M5%id), v2h%mif(34,M5%id), &
               & v2h%mif(35,M5%id), v2h%mif(21,M5%id), v2h%mif(22,M5%id), &
               & v2h%mif(23,M5%id), v2h%mif(24,M5%id), v2h%mif(25,M5%id) ) )
       else
          if( M5%id == 5 ) then
             M5%tinfo(j)%xtime = v2h%mrf(1,5)
          else
             M5%tinfo(j)%xtime = v2h%mrf(1,5)
          end if
       end if

       yr = v2h%mif(1,6) / 1000000
       mt = mod( v2h%mif(1,6), 1000000 ) / 10000
       dy = mod( v2h%mif(1,6), 10000   ) / 100
       hr = mod( v2h%mif(1,6), 100     )
       if( M5%id == 6 .or. M5%id == 7 ) then
          if( yr == v2h%mif(15,M5%id) .and. mt == v2h%mif(14,M5%id) .and. &
               & dy == v2h%mif(13,M5%id) .and. hr == v2h%mif(12,M5%id) ) then
             write( M5%tinfo(j)%date, fmt=fmt1 ) &
                  & v2h%mif(16,M5%id), v2h%mif(15,M5%id), &
                  & v2h%mif(14,M5%id), v2h%mif(13,M5%id), &
                  & v2h%mif(12,M5%id), v2h%mif(11,M5%id), 0, 0
          else
             write( M5%tinfo(j)%date, fmt=fmt1 ) &
                  & v2h%mif(16,M5%id), yr, mt, &
                  & dy, hr, v2h%mif(11,M5%id), 0, 0
          end if
       else if( M5%id == 5 ) then
          write( M5%tinfo(j)%date, fmt=fmt2 ) &
               & v2h%mif(21,2), v2h%mif(22,2), v2h%mif(23,2), &
               & v2h%mif(24,2), v2h%mif(25,2), 0, 0
       else
          write( M5%tinfo(j)%date, fmt=fmt2 ) &
               & v2h%mif(21,M5%id), v2h%mif(22,M5%id), v2h%mif(23,M5%id), &
               & v2h%mif(24,M5%id), v2h%mif(25,M5%id), 0, 0
       end if
    end do

    MM5_GetInfo_V2 = MM5_no_error

    return
  end function MM5_GetInfo_V2
