  integer function MM5_Inquire_Variable( M5, name, vid )
    type(MM5Data)             , intent(IN)  :: M5
    character(len=*)          , intent(IN)  :: name
    integer         , optional, intent(OUT) :: vid

    integer          :: i, clen
    character(len=9) :: cname

    cname(1:9)    = '         '
    clen          = len( name )
    cname(1:clen) = name(1:clen)

    MM5_Inquire_Variable = MM5_no_such_variable

    do i = 1, M5%nv
       if( cname == M5%vinfo(i)%name ) then
          MM5_Inquire_Variable = i
          exit
       end if
    end do

    if( present( vid ) ) vid = MM5_Inquire_Variable

    return
  end function MM5_Inquire_Variable

  integer function MM5_Inquire_Time( M5, ctime, xtime, tid, ForceHour )
    use time_handle
    type(MM5Data)              , intent(IN)  :: M5
    character(len=24), optional, intent(IN)  :: ctime
    real             , optional, intent(IN)  :: xtime
    integer          , optional, intent(OUT) :: tid
    logical          , optional, intent(IN)  :: ForceHour

    integer :: i
    integer :: cyr, cmt, cdy, chr, cmn
    integer :: yr, mt, dy, hr, mn
    real    :: csc, sc, dt
    logical :: FH

    real(8) :: ctotal, mtotal

    FH = .true.
    if( present( ForceHour ) ) FH = ForceHour
    if( M5%id < 6 ) FH = .false.

    MM5_Inquire_Time = MM5_no_such_time
    if( present( ctime ) ) then
       do i = 1, M5%it
          if( FH ) then
             call MM5_SeparateCtime( ctime, cyr, cmt, cdy, chr, cmn, csc )
             call MM5_SeparateCtime( M5%tinfo(i)%date, yr, mt, dy, hr, mn, sc )
             if( M5%version == 2 ) then
                dt = M5%v2head%mrf(309,6)
             else if( M5%version == 3 ) then
                dt = M5%v3big%bhr(2,12)
             end if
             hr = hr + int( ( mn + sc / 60. + dt / 60. ) / 60. )
             ctotal = get_total_date( yr, mt, dy, hr, cyr, cmt, cdy, chr )
             if( abs( ctotal ) < 0.01 ) then
                MM5_Inquire_Time = i
                exit
             end if

          else
             if( ctime == M5%tinfo(i)%date ) then
                MM5_Inquire_Time = i
                exit
             end if
          end if
       end do

    else if( present( xtime ) ) then
       do i = 1, M5%it
          if( abs( xtime - M5%tinfo(i)%xtime ) < 0.001 ) then
             MM5_Inquire_Time = i
             exit
          end if
       end do
    else
       MM5_Inquire_Time = 1
    end if

    if( present( tid ) ) tid = MM5_Inquire_Time

    return
  end function MM5_Inquire_Time

  subroutine MM5_SeparateCtime( date, year, month, day, hour, minute, second )
    character(len=24), intent(IN)  :: date
    integer          , intent(OUT) :: year, month, day, hour, minute
    real             , intent(OUT) :: second

    integer, parameter :: Izero = ichar('0')

    !                1234 67 90 23 56 89 1234
    ! date format is YYYY-MM-DD_hh:mm:ss.ssss
    year =   1000 * ( ichar( date(1:1) ) - Izero ) &
         & +  100 * ( ichar( date(2:2) ) - Izero ) &
         & +   10 * ( ichar( date(3:3) ) - Izero ) &
         & +          ichar( date(4:4) ) - Izero

    month = 10 * ( ichar( date(6:6) ) - Izero ) + ichar( date(7:7) ) - Izero

    day   = 10 * ( ichar( date(9:9) ) - Izero ) + ichar( date(10:10) ) - Izero

    hour  = 10 * ( ichar( date(12:12) ) - Izero ) &
         & +       ichar( date(13:13) ) - Izero

    minute = 10 * ( ichar( date(15:15) ) - Izero ) &
         & +        ichar( date(16:16) ) - Izero

    second =    10. * real( ichar( date(18:18) ) - Izero ) &
         & +          real( ichar( date(19:19) ) - Izero ) &
         & +    0.1 * real( ichar( date(21:21) ) - Izero ) &
         & +   0.01 * real( ichar( date(22:22) ) - Izero ) &
         & +  0.001 * real( ichar( date(23:23) ) - Izero ) &
         & + 0.0001 * real( ichar( date(24:24) ) - Izero )

    return
  end subroutine MM5_SeparateCtime
