  ! open MM5 data
  integer function MM5_open( io, M5, FileName )
    integer                   , intent(IN)    :: io
    type(MM5Data)             , intent(INOUT) :: M5
    character(len=*), optional, intent(IN)    :: FileName

    integer :: itmp

    ! initial value
    MM5_open = MM5_no_error

    ! file check
    if( present( FileName ) ) then
       inquire( file=FileName, EXIST=FileExist )
       if( .not. FileExist ) then
          if( debug ) write(6,*) 'File ', FileName, ' not found.'
          MM5_open = MM5_file_not_found
       end if
    end if

    inquire( unit=io, OPENED=FileExist )
    if( FileExist ) then
       if( debug ) write(6,*) 'Unit ', io, ' has already used.'
       MM5_open = MM5_unit_already_used
    end if
    if( MM5_open < 0 ) return

    if( present( FileName ) ) then
       open( io, file=FileName, form='unformatted' )
    end if

    ! initializing structure
    status = MM5_Initialize( M5 )

    ! version check
    read( io ) itmp
    if( itmp == 0 ) then
       M5%version = 3
    else if( itmp > 0 .and. itmp < 8 ) then
       M5%version = 2
    else
       if( debug ) write(6,*) 'File ', FileName, ' may be not MM5 format.'
       MM5_open = MM5_invalid_format
       return
    end if

    M5%nv  = 0
    status = MM5_GetInfo( io, M5 )

    if( M5%id == 1 ) then
       M5%cur_t     = 1
       M5%cur_xtime = 0.
    else
       M5%cur_t     = 1
       M5%cur_xtime = M5%tinfo(1)%xtime
    end if
    rewind( io )
  end function MM5_open

  ! close MM5 data
  integer function MM5_close( io, M5 )
    integer      , intent(IN)    :: io
    type(MM5Data), intent(INOUT) :: M5

    MM5_close = MM5_FreeMemory( M5 )

    if( M5%version /= 2 .and. M5%version /= 3 ) then
       if( debug ) write(6,*) 'Unit ', io, ' has already closed.'
       MM5_close = MM5_already_closed
       return
    end if

    MM5_close = MM5_no_error
    close( io )
  end function MM5_close

  ! free allocated data
  integer function MM5_FreeMemory( M5 )
    type(MM5Data), intent(INOUT) :: M5

!    if( associated( M5%v2head ) ) nullify( M5%v2head )
!    if( associated( M5%v3big  ) ) nullify( M5%v3big  )
    if( associated( M5%level  ) ) nullify( M5%level  )
    if( associated( M5%vinfo  ) ) nullify( M5%vinfo  )
    if( associated( M5%tinfo  ) ) nullify( M5%tinfo  )

    MM5_FreeMemory = MM5_no_error
    return
  end function MM5_FreeMemory
