program info
  use MM5
  implicit none

  type(MM5Data) :: M5

  integer            :: io, status
  character(len=128) :: infile

  integer                             :: i, vid, k, j
  real, dimension(:,:,:), allocatable :: vdat3
  real, dimension(:,:)  , allocatable :: vdat2

  character(len=9) :: cvar

  io = 11
  infile = 'mmout_v2'

  write(6,*) 'INPUT MM5 DATA:'
  read(5,'(a128)') infile

  status = MM5_open( io, M5, FileName=infile )
!  status = MM5_open( io, M5 )

  write(6,*) 'FILE NAME = ', trim( infile )
  write(6,*) status

  if( M5%version == 2 ) then
     call WriteHeaderV2( io, M5 )
  else if( M5%version == 3 ) then
     call WriteHeaderV3( io, M5 )
  end if

  status = MM5_close( io, M5 )
  stop
end program info

subroutine WriteHeaderV3( io, M5 )
  use MM5
  implicit none

  integer      , intent(IN)    :: io
  type(MM5Data), intent(INOUT) :: M5

  integer :: i, status, flag, itmp
  integer :: yr, mt, dy, hr, mn
  real    :: sc
  
  integer           :: ndim
  integer           :: start_index(4), end_index(4)
  real              :: xtime
  character(len=4)  :: staggering, ordering
  character(len=24) :: current_date
  character(len=9)  :: name
  character(len=25) :: unit
  character(len=46) :: description


  write(6,*) 'MM5V2 FORMAT DATA'
  select case ( M5%id )
  case ( 1 )
     write(6,*) 'TERRAIN OUTPUT DATA'
  case ( 2 )
     write(6,*) 'REGRID OUTPUT DATA'
  case ( 3 )
     write(6,*) 'RAWINS/Little_R OUTPUT DATA'
  case ( 4 )
     write(6,*) "RAWINS' SURFACE ANALYSIS OUTPUT DATA"
  case ( 5 )
     write(6,*) 'MODEL INPUT DATA'
  case ( 6 )
     write(6,*) 'MODEL LOWER BOUNDARY CONDITIONAL DATA'
  case ( 7 )
     write(6,*) 'MODEL LATERAL BOUNDARY CONDITIONAL DATA'
  case ( 8 )
     write(6,*) 'INTERPORATED MODEL OUTPUT ON P LEVELS'
  case ( 11 )
     write(6,*) 'MODEL OUTPUT ON SIGMA LEVEL'
  end select

  write(6,*)
  write(6,*) 'IX = ', M5%ix
  write(6,*) 'JX = ', M5%jx
  write(6,*) 'KX = ', M5%kx
  write(6,*) 'DOMAIN GRID DISTANCE (KM) = ', M5%v3big%bhr(9,1) * .001
  write(6,*) 'NUMBER OF VARIABLES = ', M5%nv
  write(6,*)

  if( M5%v3big%bhi(7,1) == 1 ) then
     write(6,*) 'MAP PROJECTION IS LAMBERT CONFORMAL'
  else if( M5%v3big%bhi(7,1) == 2 ) then
     write(6,*) 'MAP PROJECTION IS POLAR STEREOGRAPHIC'
  else if( M5%v3big%bhi(7,1) == 2 ) then
     write(6,*) 'MAP PROJECTION IS MERCATOR'
  end if
  write(6,*) 'MAP PROJECTION = ', M5%v3big%bhi(7,1)
  write(6,*) 'COARSE DOMAIN I GRID = ', M5%v3big%bhi(5,1)
  write(6,*) 'COARSE DOMAIN J GRID = ', M5%v3big%bhi(6,1)
  write(6,*) 'EXPANDED DOMAIN I GRID = ', M5%v3big%bhi(9,1)
  write(6,*) 'EXPANDED DOMAIN J GRID = ', M5%v3big%bhi(10,1)
  write(6,*) 'OFFSET IN I DUE TO THE EXPANSION = ', M5%v3big%bhi(11,1)
  write(6,*) 'OFFSET IN J DUE TO THE EXPANSION = ', M5%v3big%bhi(12,1)
  write(6,*) 'IS COARSE DOMAIN EXPANDED = ', M5%v3big%bhi(8,1)
  write(6,*) 'COARSE DOMAIN GRID DISTANCE (KM) = ', M5%v3big%bhr(1,1) * .001
  write(6,*) 'COARSE DOMAIN CENTER LATITUDE = ', M5%v3big%bhr(2,1)
  write(6,*) 'COARSE DOMAIN CENTER LONGITUDE = ', M5%v3big%bhr(3,1)
  write(6,*) 'CONE FACTOR = ', M5%v3big%bhr(4,1)
  write(6,*) 'TRUE LATITUDE 1 = ', M5%v3big%bhr(5,1)
  write(6,*) 'TRUE LATITUDE 2 = ', M5%v3big%bhr(6,1)
  write(6,*) 'POLE POSITION = ', M5%v3big%bhr(7,1)
  write(6,*)
  write(6,*) 'DOMAIN ID = ', M5%v3big%bhi(13,1)
  write(6,*) 'MOTHER DOMAIN ID = ',M5%v3big%bhi(14,1)
  write(6,*) 'NEST LEVEL = ',M5%v3big%bhi(15,1)
  write(6,*) 'GRID RATIO WITH RESPECT TO COARSE = ', M5%v3big%bhi(20,1)
  write(6,*) 'GRID RATIO WITH RESPECT TO MOTHER = ', M5%v3big%bhi(21,1)
  write(6,*)
  write(6,*) 'I LOCATION IN MD OF DOMAIN POINT = ',M5%v3big%bhi(18,1)
  write(6,*) 'J LOCATION IN MD OF DOMAIN POINT = ',M5%v3big%bhi(19,1)
  write(6,*) 'I LOCATION IN CD OF SW POINT = ', M5%v3big%bhr(10,1)
  write(6,*) 'J LOCATION IN CD OF SW POINT = ', M5%v3big%bhr(11,1)
  write(6,*) 'I LOCATION IN CD OF NE POINT = ', M5%v3big%bhr(12,1)
  write(6,*) 'J LOCATION IN CD OF NE POINT = ', M5%v3big%bhr(13,1)
  write(6,*)

  if( M5%id == 5 .or. M5%id == 11 ) then
     do i = 1, M5%kx
        write(6,'(a,i4,a,f)') &
             & 'HALF SIGMA LEVEL', i, ' : ', M5%level(i)
     end do
  else if( M5%id == 2 .or. M5%id == 3 .or. M5%id == 8 ) then
     do i = 1, M5%kx
        write(6,'(a,i4,a,i5,a)') &
             & 'PRESSURE LEVEL', i, ' : ', M5%level(i), ' hPa'
     end do
     write(6,'(a)') '( 1001 MEANS SURFACE LEVEL )'
  end if

  write(6,*)
  write(6,*) 'STORED VARIABLE AT 1ST TIME', M5%tinfo(1)%nv, M5%tinfo(1)%skip
  do
     read( io, IOSTAT=status ) flag
     if( status /= 0 ) exit

     if( flag == 0 ) then
        read( io, IOSTAT=status ) itmp
        cycle

     else if( flag == 1 ) then
        read( io, IOSTAT=status ) ndim, start_index, end_index, xtime, &
             & staggering, ordering, current_date, name, unit, description
        if( status /= 0 ) exit
        write(6,'(i1,a3,a9,a25,a)') &
             & ndim, 'D: ', name, unit, trim( description )
        read( io ) itmp

     else if( flag == 2 ) then
        exit
     end if
  end do

  do i = 1, M5%tinfo(1)%skip
     backspace io
  end do

  write(6,*)
  do i = 1, M5%it
     call DateSeparation( M5%tinfo(i)%date, yr, mt, dy, hr, mn, sc )
     yr = mod( yr, 100 )
     write(6,'(a,i2.2,i2.2,i2.2,i2.2)') &
          & 'DATE(YYMMDDHH) = ', yr, mt, dy, hr
  end do

  return
end subroutine WriteHeaderV3

subroutine WriteHeaderV2( io, M5 )
  use MM5
  implicit none

  integer      , intent(IN)    :: io
  type(MM5Data), intent(INOUT) :: M5

  integer :: i
  integer :: yr, mt, dy, hr, mn
  real    :: sc

  write(6,*) 'MM5V2 FORMAT DATA'
  select case ( M5%id )
  case ( 1 )
     write(6,*) 'TERRAIN OUTPUT DATA'
  case ( 2 )
     write(6,*) 'DATAGRID OUTPUT DATA'
  case ( 3 )
     write(6,*) 'RAWINS OUTPUT DATA'
  case ( 5 )
     write(6,*) 'MODEL INPUT DATA'
  case ( 6 )
     write(6,*) 'MODEL OUTPUT ON SIGMA LEVEL'
  case ( 7 )
     write(6,*) 'INTERPORATED MODEL OUTPUT ON P LEVELS'
  end select
  write(6,*)
  write(6,*) 'IX = ', M5%ix
  write(6,*) 'JX = ', M5%jx
  write(6,*) 'KX = ', M5%kx
  write(6,*) 'DOMAIN GRID DISTANCE (KM) = ', M5%v2head%mrf(101,1)
  write(6,*) 'NUMBER OF VARIABLES = ', M5%nv
  write(6,*)
  if( M5%v2head%mif(4,1) == 1 ) then
     write(6,*) 'MAP PROJECTION IS LAMBERT CONFORMAL'
  else if( M5%v2head%mif(4,1) == 2 ) then
     write(6,*) 'MAP PROJECTION IS POLAR STEREOGRAPHIC'
  else if( M5%v2head%mif(4,1) == 2 ) then
     write(6,*) 'MAP PROJECTION IS MERCATOR'
  end if
  write(6,*) 'MAP PROJECTION = ', M5%v2head%mif(4,1)
  write(6,*) 'COARSE DOMAIN I GRID = ', M5%v2head%mif(2,1)
  write(6,*) 'COARSE DOMAIN J GRID = ', M5%v2head%mif(3,1)
  write(6,*) 'EXPANDED DOMAIN I GRID = ', M5%v2head%mif(6,1)
  write(6,*) 'EXPANDED DOMAIN J GRID = ', M5%v2head%mif(7,1)
  write(6,*) 'OFFSET IN I DUE TO THE EXPANSION = ', M5%v2head%mif(8,1)
  write(6,*) 'OFFSET IN J DUE TO THE EXPANSION = ', M5%v2head%mif(9,1)
  write(6,*) 'IS COARSE DOMAIN EXPANDED = ', M5%v2head%mif(5,1)
  write(6,*) 'COARSE DOMAIN GRID DISTANCE (KM) = ', M5%v2head%mrf(1,1)
  write(6,*) 'COARSE DOMAIN CENTER LATITUDE = ', M5%v2head%mrf(2,1)
  write(6,*) 'COARSE DOMAIN CENTER LONGITUDE = ', M5%v2head%mrf(3,1)
  write(6,*) 'CONE FACTOR = ', M5%v2head%mrf(4,1)
  write(6,*) 'TRUE LATITUDE 1 = ', M5%v2head%mrf(5,1)
  write(6,*) 'TRUE LATITUDE 2 = ', M5%v2head%mrf(6,1)
  write(6,*) 'POLE POSITION = ', M5%v2head%mrf(7,1)
  write(6,*)
  write(6,*) 'DOMAIN ID = ', M5%v2head%mif(101,1)
  write(6,*) 'MOTHER DOMAIN ID = ',M5%v2head%mif(102,1)
  write(6,*) 'NEST LEVEL = ',M5%v2head%mif(103,1)
  write(6,*) 'GRID RATIO WITH RESPECT TO COARSE = ', M5%v2head%mif(108,1)
  write(6,*) 'GRID RATIO WITH RESPECT TO MOTHER = ', M5%v2head%mif(109,1)
  write(6,*)
  write(6,*) 'I LOCATION IN MD OF DOMAIN POINT = ',M5%v2head%mif(106,1)
  write(6,*) 'J LOCATION IN MD OF DOMAIN POINT = ',M5%v2head%mif(107,1)
  write(6,*) 'I LOCATION IN CD OF SW POINT = ', M5%v2head%mrf(102,1)
  write(6,*) 'J LOCATION IN CD OF SW POINT = ', M5%v2head%mrf(103,1)
  write(6,*) 'I LOCATION IN CD OF NE POINT = ', M5%v2head%mrf(104,1)
  write(6,*) 'J LOCATION IN CD OF NE POINT = ', M5%v2head%mrf(105,1)
  write(6,*)

  if( M5%id == 5 .or. M5%id == 6 ) then
     do i = 1, M5%kx
        write(6,'(a,i4,a,f)') &
             & 'HALF SIGMA LEVEL', i, ' : ', M5%v2head%mrf(101+i,M5%id)
     end do
  else if( M5%id == 2 .or. M5%id == 3 .or. M5%id == 7 ) then
     do i = 1, M5%kx
        write(6,'(a,i4,a,i5,a)') &
             & 'PRESSURE LEVEL', i, ' : ', M5%v2head%mif(101+i,M5%id), ' hPa'
     end do
     write(6,'(a)') '( 1001 MEANS SURFACE LEVEL )'
  end if

  write(6,*)
  write(6,*) 'STORED VARIABLE AT 1ST TIME'
  do i = 1, M5%v2head%mif(201,M5%id)
     write(6,'(a5,a80)') '3D : ', M5%v2head%mifc(204+i,M5%id) 
  end do
  do i = 1, M5%v2head%mif(202,M5%id)
     write(6,'(a5,a80)') &
          & '2D : ', M5%v2head%mifc(204+M5%v2head%mif(201,M5%id)+i,M5%id)
  end do
  write(6,*)

  if( M5%id == 1 ) return
  do i = 1, M5%it
     call DateSeparation( M5%tinfo(i)%date, yr, mt, dy, hr, mn, sc )
     yr = mod( yr, 100 )
     write(6,'(a,i2.2,i2.2,i2.2,i2.2)') &
          & 'DATE(YYMMDDHH) = ', yr, mt, dy, hr
  end do


  return
end subroutine WriteHeaderV2

subroutine DateSeparation( date, year, month, day, hour, minute, second )
  character(len=24), intent(IN)  :: date
  integer          , intent(OUT) :: year, month, day, hour, minute
  real             , intent(OUT) :: second

  integer :: Izero 

  Izero = ichar('0')

  !                1234 67 90 23 56 89 1234
  ! date format is YYYY-MM-DD_hh:mm:ss.ssss
  year =   1000 * ( ichar( date(1:1) ) - Izero ) &
       & +  100 * ( ichar( date(2:2) ) - Izero ) &
       & +   10 * ( ichar( date(3:3) ) - Izero ) &
       & +          ichar( date(4:4) ) - Izero

  month = 10 * ( ichar( date(6:6) ) - Izero ) + ichar( date(7:7) ) - Izero

  day   = 10 * ( ichar( date(9:9) ) - Izero ) + ichar( date(10:10) ) - Izero

  hour  = 10 * ( ichar( date(12:12) ) - Izero ) &
       & +       ichar( date(13:13) ) - Izero

  minute = 10 * ( ichar( date(15:15) ) - Izero ) &
       & +        ichar( date(16:16) ) - Izero

  second =    10. * real( ichar( date(18:18) ) - Izero ) &
       & +          real( ichar( date(19:19) ) - Izero ) &
       & +    0.1 * real( ichar( date(21:21) ) - Izero ) &
       & +   0.01 * real( ichar( date(22:22) ) - Izero ) &
       & +  0.001 * real( ichar( date(23:23) ) - Izero ) &
       & + 0.0001 * real( ichar( date(24:24) ) - Izero )
  return
end subroutine DateSeparation
