  subroutine ReadModelInfoMM5( IO, m, ierr )
    integer, intent(IN)  :: IO, m
    integer, intent(OUT) :: ierr

    integer :: status, ip, nstr, DataType
    integer :: ifiles, mIO
    integer :: i, j, k, t, ct

    character(len=128) :: buf
    character(len=20)  :: ctitle
    character(len=100) :: cvalue

    character(len=128) :: Mfile(MaxInFiles)
   
    type(MM5Data) :: M5
    logical       :: TimeOnce

    ! defaults
    ierr     = MI_MM5_INFO_ERROR
    mIO      = DIO
    TimeOnce = .false.

    ! TotalFiles
    ip     = 0
    buf    = ''
    ctitle = ''
    read( IO, '(a)', IOSTAT=status ) buf
    if( status /= 0 ) return
    DataType = InquireStringType( buf, ip )
    status   = InquireCharacter( buf, ip, nstr, ctitle )
    call ToUpper( ctitle )

    if( ctitle(1:10 ) == 'TOTALDATES' ) then
       DataType = InquireStringType( buf, ip )
       if( DataType /= TypeNumber ) return
       status = InquireNumber( buf, ip, MODEL(m)%it )

       if( MODEL(m)%it < 0 ) then
          MODEL(m)%it = 0
       else
          allocate( MODEL(m)%Tinfo( MODEL(m)%it ), STAT=status )
          if( status /= 0 ) return
          TimeOnce = .true.
       end if

       ip = 0
       read( IO, '(a)', IOSTAT=status ) buf
       if( status /= 0 ) return
       DataType = InquireStringType( buf, ip )
       status   = InquireCharacter( buf, ip, nstr, ctitle )
       call ToUpper( ctitle )

    else
       MODEL(m)%it = 0
    end if

       
    if( ctitle(1:10) /= 'TOTALFILES' ) return
    DataType = InquireStringType( buf, ip )
    if( DataType /= TypeNumber ) return
    status = InquireNumber( buf, ip, ifiles )

    ! defaults
    MODEL(m)%TotalVariables = 0
    MODEL(m)%DataType       = 'MM5 '
    MODEL(m)%vunit          = 'SIGMA '
    MODEL(m)%hunit          = 'M '
    MODEL(m)%Undef          = -1.e+35

    ! check total dates
    ct = 0
    do i = 1, ifiles
       ip     = 0
       buf    = ''
       cvalue = ''
       read( IO, '(a)', IOSTAT=status ) buf
       if( status /= 0 ) return
       Mfile(i) = buf

       status = MM5_open( mIO, M5, trim( buf ) ) 
       if( status /= 0 ) return

       if( i == 1 ) then
          MODEL(m)%ix = M5%jx
          MODEL(m)%jx = M5%ix
          MODEL(m)%kx = M5%kx
          MODEL(m)%TotalVariables = M5%nv

          if( M5%version == 2 ) then
             MODEL(m)%clat  = M5%v2head%mrf(2,1)
             MODEL(m)%clon  = M5%v2head%mrf(3,1)
             MODEL(m)%tlon  = M5%v2head%mrf(3,1)
             MODEL(m)%cone  = M5%v2head%mrf(4,1)
             MODEL(m)%tlat1 = M5%v2head%mrf(5,1)
             MODEL(m)%tlat2 = M5%v2head%mrf(6,1)
             MODEL(m)%mproj = M5%v2head%mif(4,1)

             MODEL(m)%xc = ( ( 1 + M5%v2head%mif(3,1) ) * 0.5           &
                  &              - M5%v2head%mrf(103,1) )               &
                  &      * M5%v2head%mrf(1,1) / M5%v2head%mrf(101,1) + 1
             MODEL(m)%yc = ( ( 1 + M5%v2head%mif(2,1) ) * 0.5           &
                  &              - M5%v2head%mrf(102,1) )               &
                  &      * M5%v2head%mrf(1,1) / M5%v2head%mrf(101,1) + 1

          else if( M5%version == 3 ) then
             MODEL(m)%clat  = M5%v3big%bhr(2,1)
             MODEL(m)%clon  = M5%v3big%bhr(3,1)
             MODEL(m)%tlon  = M5%v3big%bhr(3,1)
             MODEL(m)%cone  = M5%v3big%bhr(4,1)
             MODEL(m)%tlat1 = M5%v3big%bhr(5,1)
             MODEL(m)%tlat2 = M5%v3big%bhr(6,1)
             MODEL(m)%mproj = M5%v3big%bhi(7,1)

             MODEL(m)%xc = ( ( 1 + M5%v3big%bhi(6,1) ) * 0.5         &
                  &              - M5%v3big%bhr(11,1) )              &
                  &      * M5%v3big%bhr(1,1) / M5%v3big%bhr(9,1) + 1
             MODEL(m)%yc = ( ( 1 + M5%v3big%bhi(5,1) ) * 0.5         &
                  &              - M5%v3big%bhr(10,1) )              &
                  &      * M5%v3big%bhr(1,1) / M5%v3big%bhr(9,1) + 1
          end if
          select case( MODEL(m)%mproj )
          case ( 0 )
             MODEL(m)%cproj = 'LATLON'
          case ( 1 )
             MODEL(m)%cproj = 'LAMCON'
          case ( 2 )
             MODEL(m)%cproj = 'PSTEREO'
          case ( 3 )
             MODEL(m)%cproj = 'MERCAT'
          case default
             MODEL(m)%cproj = 'NOPROJ'
          end select

          allocate( MODEL(m)%Vinfo( M5%nv ), STAT=status )
          if( status /= 0 ) return

          allocate( MODEL(m)%lev( MODEL(m)%kx ) )
          if( M5%id == 1 ) then             ! TERRAIN DATA
             MODEL(m)%lev(1) = 0.
          else
             MODEL(m)%lev    = M5%level
             if(    M5%id == 2 .or. M5%id == 3 .or. &
                  & M5%id == 7 .or. M5%id == 8) then
                MODEL(m)%vunit = 'Pa'
             end if
          end if

          if( M5%version == 2 ) then
             MODEL(m)%dx = M5%v2head%mrf(101,1) * 1000.
             MODEL(m)%dy = M5%v2head%mrf(101,1) * 1000.
!             MODEL(m)%dt = M5%v2head%mrf(309,6)
          else if( M5%version == 3 ) then
             MODEL(m)%dx = M5%v3big%bhr(9,1)
             MODEL(m)%dy = M5%v3big%bhr(9,1)
!             MODEL(m)%dt = M5%v3big%bhr(2,12)
          else
             return
          end if
          call xy2ll( 1.,             &
               &      1.,             &
               &      MODEL(m)%slon,  &
               &      MODEL(m)%slat,  &
               &      MODEL(m)%cproj, &
               &      MODEL(m)%dx,    &
               &      MODEL(m)%dy,    &
               &      MODEL(m)%clon,  &
               &      MODEL(m)%clat,  &
               &      MODEL(m)%xc,    &
               &      MODEL(m)%yc,    &
               &      tlat1=MODEL(m)%tlat1, &
               &      tlat2=MODEL(m)%tlat2  )
          MODEL(m)%er = 6.37e+6

       else
          if(    MODEL(m)%ix /= M5%jx .or. &
               & MODEL(m)%jx /= M5%ix .or. &
               & MODEL(m)%kx /= M5%kx .or. &
               & MODEL(m)%TotalVariables /= M5%nv ) return
       end if
       if( .not. TimeOnce ) MODEL(m)%it = MODEL(m)%it + M5%it

       do t = 1, M5%it
          if( TimeOnce ) then
             ct = ct + 1
             MODEL(m)%Tinfo(ct)%cdate = M5%tinfo(t)%date
             MODEL(m)%Tinfo(ct)%xtime = M5%tinfo(t)%xtime
             MODEL(m)%Tinfo(ct)%File  = trim( buf )
          end if

          do j = 1, M5%nv
             if( t == 1 ) then
                MODEL(m)%Vinfo(j)%name = M5%vinfo(j)%name
                MODEL(m)%Vinfo(j)%ix   = M5%vinfo(j)%jx
                MODEL(m)%Vinfo(j)%jx   = M5%vinfo(j)%ix
                MODEL(m)%Vinfo(j)%kx   = M5%vinfo(j)%kx
                if( M5%vinfo(j)%kx == -1 ) MODEL(m)%Vinfo(j)%kx = 1
                MODEL(m)%Vinfo(j)%it   = 1
                MODEL(m)%Vinfo(j)%desc = ''
                MODEL(m)%Vinfo(j)%unit = ''
                if( M5%vinfo(j)%staggering(1:1) == 'C' ) then
                   MODEL(m)%Vinfo(j)%staggering = .false.
                else
                   MODEL(m)%Vinfo(j)%staggering = .true.
                end if
                
             else
                if( MODEL(m)%Vinfo(j)%name /= M5%vinfo(j)%name ) return

             end if
          end do
       end do

       status = MM5_close( mIO, M5 )
       if( status /= 0 ) return
    end do

    if( .not. TimeOnce ) then
       allocate( MODEL(m)%Tinfo( MODEL(m)%it ), STAT=status )
       if( status /= 0 ) return
       ct = 0
       do i = 1, ifiles
          status = MM5_open( mIO, M5, trim( Mfile(i) ) ) 
          if( status /= 0 ) return

          do t = 1, M5%it
             ct = ct + 1
             MODEL(m)%Tinfo(ct)%cdate = M5%tinfo(t)%date
             MODEL(m)%Tinfo(ct)%xtime = M5%tinfo(t)%xtime
             MODEL(m)%Tinfo(ct)%File  = Mfile(i)
          end do
       
          status = MM5_close( mIO, M5 )
          if( status /= 0 ) return
       end do
    end if

    if( m == 1 ) then
       Model_ix = MODEL(m)%ix
       Model_jx = MODEL(m)%jx
       Model_kx = MODEL(m)%kx
       Model_it = MODEL(m)%it
    end if

    ! other data
    do 
       ip     = 0
       buf    = ''
       ctitle = ''
       cvalue = ''

       read( IO, '(a)', IOSTAT=status ) buf
       if( status /= 0 ) exit
       if( buf(1:1) == '/' ) cycle

       DataType = InquireStringType( buf, ip )
       if( DataType /= TypeChar ) cycle
       status = InquireCharacter( buf, ip, nstr, ctitle )
       if( ctitle(1:1) == '#' ) cycle
       DataType = InquireStringType( buf, ip )

       call ToUpper( ctitle )
       if( ctitle(1:10) == 'ENDIANTYPE' ) then
          status = InquireCharacter( buf, ip, nstr, MODEL(m)%Endian )

       else if( ctitle(1:8) == 'BYTESIZE' ) then
          status = InquireNumber( buf, ip, MODEL(m)%ByteSize )

       else if( ctitle(1:6) == 'DELTAT' ) then
          status = InquireNumber( buf, ip, MODEL(m)%dt )

       end if

    end do

    ierr = MI_NOERROR
    return
  end subroutine ReadModelInfoMM5
