module namelist
  implicit none
  private

  integer, parameter :: MaxLineBuffer  = 512
  integer, parameter :: MaxDefineFiles =  10
  integer, parameter :: MaxPoints      = 2000
  integer, parameter :: MaxOrders      =   5
  integer, parameter :: MaxKeys        =  10
  integer, parameter :: MaxKeyArray    =  20

  integer, parameter :: ios = 42

  character(len=MaxLineBuffer) :: ConstantFile(MaxDefineFiles)
  character(len=MaxLineBuffer) :: EquationFile(MaxDefineFiles)
  character(len=MaxLineBuffer) :: ModelInfoFile(MaxDefineFiles)
  character(len=MaxLineBuffer) :: LevelInfoFile(MaxDefineFiles)

  logical :: ForceUndef
  integer :: NoData
  integer :: CacheMax

  character(len=MaxLineBuffer) :: OutFile
  character(len=MaxLineBuffer) :: MinMaxFile
  character(len=MaxLineBuffer) :: OutForm
  character(len=MaxKeys)       :: OutKey(MaxKeyArray)
  real, dimension(MaxPoints)   :: xp, yp
  real, dimension(MaxPoints)   :: lonp, latp
  integer                      :: itpmethod
  real                         :: itparg1
  logical                      :: RPNLog
  real                         :: OutRadius
  real                         :: OutSdeg
  real                         :: OutEdeg
  real, dimension(MaxKeys)     :: OutMinVal
  real, dimension(MaxKeys)     :: OutMaxVal
  integer, dimension(MaxKeys)  :: OutUndef
  logical                      :: DF3Swap


  ! for debug
  integer                      :: olev
  integer                      :: modelno
  integer                      :: stime, etime, tskip
  character(len=MaxLineBuffer) :: str_test
  character(len=MaxLineBuffer) :: str_test2
  character(len=MaxLineBuffer) :: str_test3
  character(len=MaxLineBuffer) :: str_test4
  character(len=MaxLineBuffer) :: str_test5
  logical                      :: print_env
  integer                      :: test_sum
  integer                      :: test_sum_times
  integer                      :: test_sum_skip
  character(len=MaxLineBuffer) :: rpn_log
  integer                      :: similarity
  integer                      :: input_sum
  integer                      :: input_sum_times
  integer                      :: input_model
  character(len=24)            :: input_ctime
  character(len=MaxLineBuffer) :: input_rpn
  integer                      :: input_wind(4)
  integer                      :: wind_trans
  real                         :: wind_tmat(6,2)
  integer                      :: output_model
  character(len=24)            :: output_ctime
  character(len=MaxLineBuffer) :: output_rpn
  real                         :: count_ratio
  integer                      :: order(MaxDefineFiles,MaxOrders)

  namelist/test_str/order,          &
       &            str_test,       &
       &            str_test2,      &
       &            str_test3,      &
       &            str_test4,      &
       &            str_test5,      &
       &            olev,           &
       &            modelno,        &
       &            stime,          &
       &            etime,          &
       &            tskip,          &
       &            print_env,      &
       &            test_sum,       &
       &            test_sum_times, &
       &            test_sum_skip,  &
       &            rpn_log,        &
       &            similarity,     &
       &            count_ratio,    &
       &            input_sum,      &
       &            input_sum_times,&
       &            input_model,    &
       &            input_ctime,    &
       &            input_rpn,      &
       &            input_wind,     &
       &            wind_trans,     &
       &            wind_tmat,      &
       &            output_model,   &
       &            output_ctime,   &
       &            output_rpn

  namelist/rpn_config/ConstantFile,        &
       &              EquationFile,        &
       &              ModelInfoFile,       &
       &              ForceUndef,          &
       &              LevelInfoFile
  
  namelist/read_config/NoData,       &
       &               CacheMax

  namelist/output_config/OutFile,    &
       &                 OutForm,    &
       &                 OutKey,     &
       &                 OutMinval,  &
       &                 OutMaxVal,  &
       &                 OutUndef,   &
       &                 DF3Swap,    &
       &                 MinMaxFile, &
       &                 RPNLog,     &
       &                 xp,         &
       &                 yp,         &
       &                 lonp,       &
       &                 latp,       &
       &                 OutRadius,  &
       &                 OutSdeg,    &
       &                 OutEdeg,    &
       &                 itpmethod,  &
       &                 itparg1

  public :: ConstantFile
  public :: EquationFile
  public :: ModelInfoFile
  public :: ForceUndef
  public :: LevelInfoFile

  public :: ios

  public :: NoData
  public :: CacheMax

  public :: OutFile
  public :: OutForm
  public :: OutKey
  public :: RPNLog
  public :: MinMaxFile
  public :: xp, yp
  public :: lonp, latp
  public :: OutRadius
  public :: OutSdeg
  public :: OutEdeg
  public :: OutMinval
  public :: OutMaxval
  public :: OutUndef
  public :: DF3Swap
  public :: itpmethod
  public :: itparg1

  public :: ReadNamelist

  public :: order
  public :: olev, stime, etime, tskip
  public :: modelno
  public :: str_test
  public :: print_env
  public :: test_sum, test_sum_times, test_sum_skip
  public :: rpn_log
  public :: similarity, count_ratio
  public :: input_sum, input_sum_times
  public :: input_model,  input_ctime,  input_rpn, input_wind
  public ::  wind_trans, wind_tmat
  public :: output_model, output_ctime, output_rpn

  public :: MaxDefineFiles
  public :: MaxPoints
  public :: MaxOrders
  public :: MaxKeys
  public :: MaxKeyArray

contains
  subroutine ReadNamelist( io, filename )
    integer         , intent(IN) :: io
    character(len=*), intent(IN) :: filename

    integer :: i, status

    ! defaults
    ConstantFile(1)  = 'constant.def'
    EquationFile(1)  = 'equations.def'
    ModelInfoFile(1) = 'model_info.def'
    LevelInfoFile(1) = 'level.def'
    ForceUndef       = .false.

    ConstantFile(2:MaxDefineFiles)  = ''
    EquationFile(2:MaxDefineFiles)  = ''
    ModelInfoFile(2:MaxDefineFiles) = ''
    LevelInfoFile(2:MaxDefineFiles) = ''

    NoData   = 0
    CacheMax = 20

    OutFile    = 'z.txt'
    OutForm    = 'FUNC(OUTPUT,TEXT,H2D,1,1) ='
    OutKey     = 'OUT'
    RPNLog     = .false.
    MinMaxFile = 'mg_output_stat.txt'
    xp         = -999.
    yp         = -999.
    lonp       = -999.
    latp       = -999.
    OutRadius  = 300000.
    OutSdeg    = 0.
    OutEdeg    = 360.
    OutMinVal  = 0.
    OutMaxVal  = 100.
    OutUndef   = 255
    DF3Swap    = .true.
    itpmethod  = 1              ! 1: Inverse-Distance Weight
    itparg1    = 2.             ! weighting power

    do i = 1, MaxDefineFiles
       order(i,:) = i
    end do

    olev      = 1
    modelno   = 1
    stime     = 1
    etime     = 1
    tskip     = 1
    str_test  = '1 1 ADD ='
    str_test2 = ''
    str_test3 = ''
    str_test4 = ''
    str_test5 = ''
    print_env = .false.
    test_sum  = 0
    test_sum_times = 1
    test_sum_skip  = 1
    rpn_log   = 'rpn_log.txt'
    similarity = 0
    count_ratio = 1.
    input_sum   = 0
    input_sum_times = 1
    input_model = 1
    input_ctime = '2004-09-06_00:00:00.0000'
    input_rpn   = ''
    input_wind(1)  = 20
    input_wind(2)  = 20
    input_wind(3)  = 30
    input_wind(4)  = 30
    wind_trans     = 0
    wind_tmat      = 0
    output_model = 1
    output_ctime = '2004-09-06_00:00:00.0000'
    output_rpn   = ''

    open( io, file=trim( filename ), IOSTAT=status )
    if( status /= 0 ) then
       write(6,*) 'namelist not open (due to permission or not found)'
       stop
    end if

    read( io, rpn_config, IOSTAT=status )
    if( status /= 0 ) then
       write(6,*) 'namelist syntax error (in rpn_config)'
       stop
    end if

    read( io, read_config, IOSTAT=status )
    if( status /= 0 ) then
       write(6,*) 'namelist syntax error (in read_config)'
       stop
    end if

    read( io, output_config, IOSTAT=status )
    if( status /= 0 ) then
       write(6,*) 'namelist syntax error (in output_config)'
       stop
    end if

    read( io, test_str, IOSTAT=status )
    if( status /= 0 ) then
       write(6,*) 'namelist syntax error (in test_str)'
       stop
    end if

    if( len_trim( str_test2 ) > 0 ) then
       str_test = trim( str_test ) // ' ' // trim( str_test2 )
    end if

    close( io )
    return
  end subroutine ReadNamelist
end module namelist
