subroutine ReadFromData( operation, stacks, MaxStacks, status, odr, mn )
!subroutine ReadFromData( operation, stacks, MaxStacks, status, odr )
  use string
  use ModelInfo
  use namelist
  use Endian
  use RPN
  implicit none
  integer                              , intent(IN)    :: MaxStacks
  type(RPN_STACK), dimension(MaxStacks), intent(INOUT) :: stacks
  character(len=*)                     , intent(IN)    :: operation
  integer                              , intent(INOUT) :: status
  integer                              , intent(INOUT) :: odr
  integer                              , intent(IN)    :: mn

  character(len=RPN_MaxModelName) :: ModelName
  character(len=RPN_MaxVarName)   :: VarName
  character(len=RPN_MaxAxisInfo)  :: Tinfo

  logical :: DataRead
  integer :: clen, Ctime, modelno_tmp
  character(len=24) :: Cdate

  real      , dimension(:,:,:,:), allocatable :: vtmp
  integer(1), dimension(:,:,:,:), allocatable :: i1tmp
  integer(2), dimension(:,:,:,:), allocatable :: i2tmp

  status = AnalyzeReadComponent( operation,        &
       &                         Model=ModelName,  &
       &                         Variable=VarName, &
       &                         Tinfo=Tinfo       )

  DataRead = .false.
  clen     = len_trim( VarName )
  Ctime    = Model_t
  Cdate    = Model_date
  modelno_tmp = modelno
  if( mn > 0 ) modelno = mn
  
  if( Tinfo(1:3) == 'T-1' .and. Model_t > 1 ) then
     Ctime = Model_t - 1
  else if( Tinfo(1:3) == 'T+1' .and. Model_t < Model_it ) then
     Ctime = Model_t + 1
  end if

  if( odr > MaxOrders ) odr = 1
  if( Ctime > MODEL(modelno)%it ) Ctime = MODEL(modelno)%it

!  Cdate = MODEL(1)%Tinfo(Ctime)%cdate
  Cdate = MODEL(modelno)%Tinfo(Ctime)%cdate
  if( RPN_debug ) write(6,*) '***READ: ', trim( operation )

  if( ModelName(1:6) == 'BINARY' ) then
     call ReadBinaryData( status, odr )

  else if( ModelName(1:3) == 'MM5' ) then
     write(6,*) '****ARE***'
     call ReadMM5Data( status )
!     status = 555
!     return

  else
     modelno = modelno_tmp
     return
  end if

  modelno = modelno_tmp
  return
contains
  include "readfromdata_binary.f90"

  subroutine ReadMM5Data( status )
    use MM5
    integer, intent(INOUT) :: status

    integer :: im, imf, iv, IO
    integer :: i, j ,k, t
    integer :: ix, jx, kx, it
    type(MM5Data) :: M5
    logical :: DataRead

    real, dimension(:,:,:), allocatable :: mtmp

    DataRead = .false.

!    do im = 1, MODEL(1)%TotalInfo
    do imf = 1, MaxDefineFiles
       im = order(imf,odr)
       if( im > MODEL(1)%TotalInfo ) cycle

       do iv = 1, MODEL(im)%TotalVariables
          if( VarName(1:clen+1) == MODEL(im)%Vinfo(iv)%name(1:clen+1) ) then
             ix = MODEL(im)%Vinfo(iv)%ix
             jx = MODEL(im)%Vinfo(iv)%jx
             kx = MODEL(im)%Vinfo(iv)%kx
             it = MODEL(im)%Vinfo(iv)%it
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! debug
             if( VarName(1:2) == 'W ' ) then
                kx = kx - 1
             end if
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
             if( it > 1 ) return

             if( MODEL(im)%Tinfo(1)%cdate(1:1) == 's' ) then
                Ctime = 1
                Cdate = 'YYYY-MM-DD_hh:mm:ss.ssss'

             else
                do t = 1, MODEL(im)%it
                   if( MODEL(im)%Tinfo(t)%cdate == Cdate ) then
                      Ctime = t
                      exit
                   end if
                end do
             end if

             if( RPN_debug ) write(6,*) '***READ: ', Ctime, Cdate

             allocate( vtmp(ix,jx,kx,it) )
             allocate( mtmp(jx,ix,kx   ) )
             IO = DIO + im - 1
write(6,*) '***MM5: ', trim( MODEL(im)%Tinfo(Ctime)%File ), MODEL(im)%Tinfo(Ctime)%xtime
             status = MM5_open( IO, M5, trim( MODEL(im)%Tinfo(Ctime)%File ) )
             if( status /= 0 ) return

             status = MM5_GetData( IO,                                 &
                  &                M5,                                 &
                  &                VarName(1:8),                       &
                  &                mtmp(:,:,:),                        &
                  &                xtime=MODEL(im)%Tinfo(Ctime)%xtime, &
                  &                Decouple=.true.                     )
             if( status /= 0 ) return

             status = MM5_close( IO, M5 )
             if( status /= 0 ) return

             do k = 1, kx
                do j = 1, jx
                   do i = 1, ix
                      vtmp(i,j,k,1) = mtmp(j,i,k)
                   end do
                end do
             end do

             if( RPN_debug ) write(6,*) '***STAGGERING: ', &
                  & MODEL(im)%Vinfo(iv)%staggering
             if( .not. MODEL(im)%Vinfo(iv)%staggering ) then
                do t = 1, it
                   do k = 1, kx
                      do i = 1, ix
                         vtmp(i,jx,k,t) = MODEL(im)%Undef
                      end do

                      do j = 1, jx
                         vtmp(ix,j,k,t) = MODEL(im)%Undef
                      end do
                   end do
                end do
             end if

             status = RPN_PushStack( stacks,             &
                  &                  MaxStacks,          &
                  &                  vtmp,               &
                  &                  ix,                 &
                  &                  jx,                 &
                  &                  kx,                 &
                  &                  it,                 &
                  &                  MODEL(im)%Undef,    &
                  &                  model=im,           &
                  &                  omodel=im,          &
                  &                  olev=MODEL(im)%lev, &
                  &                  cdate=Cdate,        &
                  &             staggering=MODEL(im)%Vinfo(iv)%staggering )


             DataRead = .true.
             deallocate( vtmp )
             deallocate( mtmp )
          end if
       end do
    end do

    if( .not. DataRead ) then
       if( NoData == 0 .or. NoData == 1 ) then
          ix = MODEL(1)%Vinfo(iv)%ix
          jx = MODEL(1)%Vinfo(iv)%jx
          kx = MODEL(1)%Vinfo(iv)%kx
          it = MODEL(1)%Vinfo(iv)%it
          allocate( vtmp(ix,jx,kx,it) )

          if( NoData == 0 ) then
             vtmp = 0.
          else if( NoData == 1 ) then
             vtmp = RPN_Undefined
          end if
           
          status = RPN_PushStack( stacks,          &
               &                  MaxStacks,       &
               &                  vtmp,            &
               &                  ix,              &
               &                  jx,              &
               &                  kx,              &
               &                  it,              &
               &                  RPN_Undefined,   &
               &                  model=1          )
          deallocate( vtmp )

       else
          status = RPN_NO_READ_DATA
       end if
    end if

    return
  end subroutine ReadMM5Data
end subroutine ReadFromData
