subroutine ReadFuncData( operation, stacks, MaxStacks, status )
  use string
  use ModelInfo
  use namelist
  use Endian
  use RPN
  implicit none
  integer         , intent(IN)    :: MaxStacks
  type(RPN_STACK) , intent(INOUT) :: stacks(MaxStacks)
  character(len=*), intent(IN)    :: operation
  integer         , intent(INOUT) :: status

  integer            :: i, nc
  integer            :: ip, is, ipos, ierr, mode, mode2, mode3, mno, no
  real               :: low, high, val, val2, x1, x2, y1, y2
  real               :: azimuth
  real               :: radius
  character(len=20)  :: FuncName, FuncArg, OutFormat, OutMode

  ! function
  integer :: FuncVint
  integer :: FuncVint2
  integer :: FuncGetIsoSurface
  integer :: FuncGetIsoSurface0
  integer :: FuncGetIsoSurface2
  integer :: FuncGetIsoSurface3
  integer :: FuncGetIsoSurface4
  integer :: FuncConvH3D
  integer :: FuncConvZ3D
  integer :: FuncZcoordinate
  integer :: FuncHconv
  integer :: FuncMask
  integer :: FuncMask2
  integer :: FuncBpass
  integer :: FuncBMask
  integer :: FuncUMask
  integer :: FuncCount
  integer :: FuncCross
  integer :: FuncOutput
  integer :: FuncHistogram
  integer :: FuncAverageGrid
  integer :: FuncTCpoint
  integer :: FuncCenterPoint
  integer :: FuncSetTime
  integer :: FuncAreaStat
  integer :: FuncParcel
  integer :: FuncWindGust
  integer :: FuncTCWinSlp
  integer :: FuncLatLon
  integer :: FuncRtheta
  integer :: FuncGridMean
  integer :: FuncZcone
  integer :: FuncDDR

  FuncName = ''
  FuncArg  = ''

  ip = 6
  status = AnalyzeFuncComponent( trim( operation ), ',', FuncName, ip, ipos )

  ! vertical interpolation
  !     FUNC(VINT,low,high,no)
  if( FuncName(1:5) == 'VINT ' ) then
     is   = ip
     low  = RPN_Undefined
     high = RPN_Undefined
     no   = 1

     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),       &
             &                         ',', FuncArg, ip, ipos   )
        call ToUpper( FuncArg )
        if( FuncArg(1:5) == 'UNDEF' ) then
           low = RPN_Undefined
        else
           ierr = InquireNumber( trim( FuncArg ), is, low )
           if( ierr /= STRING_NOERROR ) low = RPN_Undefined
        end if
     end if

     if( ipos == 0 ) then
        is      = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),       &
             &                         ',', FuncArg, ip, ipos   )
        call ToUpper( FuncArg )
        if( FuncArg(1:5) == 'UNDEF' ) then
           high = RPN_Undefined
        else
           ierr = InquireNumber( trim( FuncArg ), is, high )
           if( ierr /= STRING_NOERROR ) high = RPN_Undefined
        end if

     end if

     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ')', FuncArg, ip, ipos )
        ierr   = InquireNumber( trim( FuncArg ), is, no )
     end if

     status = FuncVint( stacks, MaxStacks, low, high, mode, no )

  ! vertical interpolation
  !     FUNC(VINT2,low,high,no)
  else if( FuncName(1:6) == 'VINT2 ' ) then
     is   = ip
     low  = RPN_Undefined
     high = RPN_Undefined
     no   = 1

     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),       &
             &                         ',', FuncArg, ip, ipos   )
        call ToUpper( FuncArg )
        if( FuncArg(1:5) == 'UNDEF' ) then
           low = RPN_Undefined
        else
           ierr = InquireNumber( trim( FuncArg ), is, low )
           if( ierr /= STRING_NOERROR ) low = RPN_Undefined
        end if
     end if

     if( ipos == 0 ) then
        is      = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),       &
             &                         ',', FuncArg, ip, ipos   )
        call ToUpper( FuncArg )
        if( FuncArg(1:5) == 'UNDEF' ) then
           high = RPN_Undefined
        else
           ierr = InquireNumber( trim( FuncArg ), is, high )
           if( ierr /= STRING_NOERROR ) high = RPN_Undefined
        end if

     end if

     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ')', FuncArg, ip, ipos )
        ierr   = InquireNumber( trim( FuncArg ), is, no )
     end if

     status = FuncVint2( stacks, MaxStacks, low, high, mode, no )

  ! iso-surface
  !     [T] FUNC(GLEV0,level,no)
  !     [T] FUNC(GLEV,level,no)
  !     [T] [HGT] FUNC(GLEV2,level,no)
  !     [T] [CVAL] FUNC(GLEV3,no)
  !     [T] [HGT] [CVAL] FUNC(GLEV4,no)
  else if( FuncName(1:4) == 'GLEV' ) then
     no = 1
     if( ipos == 0 ) then
        if( FuncName(5:5) /= '3' .and. FuncName(5:5) /= '4' ) then
           is = 1
           FuncArg = ''
           status = AnalyzeFuncComponent( trim( operation ),       &
                &                         ',', FuncArg, ip, ipos   )
           ierr = InquireNumber( trim( FuncArg ), is, low )
           if( ierr /= STRING_NOERROR ) then
              status = RPN_FUNC_ARG_ERROR
              return
           end if
        end if

        if( ipos == 0 ) then
           is      = 1
           FuncArg = ''
           status = AnalyzeFuncComponent( trim( operation ),     &
                &                         ')', FuncArg, ip, ipos )
           ierr   = InquireNumber( trim( FuncArg ), is, no )
        end if
     else if( FuncName(5:5) /= '3' .and. FuncName(5:5) /= '4' ) then
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     if( FuncName(5:5) == '2' ) then
        status = FuncGetIsoSurface2( stacks, MaxStacks, low, no )
     else if( FuncName(5:5) == '0' ) then
        status = FuncGetIsoSurface0( stacks, MaxStacks, int(low), no )
     else if( FuncName(5:5) == '3' ) then
        status = FuncGetIsoSurface3( stacks, MaxStacks, no )
     else if( FuncName(5:5) == '4' ) then
        status = FuncGetIsoSurface4( stacks, MaxStacks, no )
     else if( FuncName(5:5) == '' ) then
        status = FuncGetIsoSurface( stacks, MaxStacks, low, no )
     end if

  ! expand 2d holizontal-plain to 3d domain
  !     FUNC(H3D[,varname])
  else if( FuncName(1:3) == 'H3D' ) then
     is      = 1
     FuncArg = ''
     if( ipos == 0 ) then
        status = AnalyzeFuncComponent( trim( operation ),       &
             &                         ')', FuncArg, ip, ipos   )
     end if
     status = FuncConvH3D( stacks, MaxStacks, FuncArg )

  ! expand 1d z-coordinate to 3d domain
  !     FUNC(Z3D[,varname])
  else if( FuncName(1:3) == 'Z3D' ) then
     is      = 1
     FuncArg = ''
     if( ipos == 0 ) then
        status = AnalyzeFuncComponent( trim( operation ),       &
             &                         ')', FuncArg, ip, ipos   )
     end if
     status = FuncConvZ3D( stacks, MaxStacks, FuncArg )

  ! convert vertical coordinate
  !     FUNC(ZCONV,mode,mno,no)
  else if( FuncName(1:5) == 'ZCONV' ) then
     mno = 1
     no  = 1

     ! mode
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     if( ipos == 0 ) then
        ierr = InquireNumber( trim( FuncArg ), is, mode )
        if( mode /= 1 .and. mode /= 2 ) return

        is      = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mno )
     end if

     if( ipos == 0 ) then
        is = 1
        FuncArg = ''
        status  = AnalyzeFuncComponent( trim( operation ),    &
             &                         ')', FuncArg, ip, ipos )
        ierr    = InquireNumber( trim( FuncArg ), is, no )
     end if

     status = FuncZcoordinate( stacks, MaxStacks, mode, mno, no )

  ! convert horizontal coordinate
  !     FUNC(HCONV,model,no)
  else if( FuncName(1:5) == 'HCONV' ) then
     no   = 1
     mode = 1

     ! model No.
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
        if( mode < 1 .or. mode > MaxModelData ) return
     end if

     ! no
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ')', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, no )
     end if

     status = FuncHconv( stacks, MaxStacks, mode, no )

  ! get latitude and logitude
  !     FUNC(LATLON,model,mode)
  !         mode:   1: latitude
  !                 2: longitude
  !                12: latitude  (ic) and longitude (ic-1)
  !                21: longitude (ic) and latitude  (ic-1)
  else if( FuncName(1:6) == 'LATLON' ) then
     mode  = 1
     mode2 = 1

     ! model No.
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
        if( mode < 1 .or. mode > MaxModelData ) return
     end if

     ! no
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ')', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode2 )
     end if

     status = FuncLatLon( stacks, MaxStacks, mode, mode2 )

  ! get latitude and logitude
  !     FUNC(RTHETA,model,mode,mode2,x,y)
  !        model: model ID number
  !         mode: 1: X/Y  2: LON/LAT
  !        mode2:   1: distance (R)
  !                 2: angle (THETA)
  !                12: R     (ic) and THETA (ic-1)
  !                21: THETA (ic) and R     (ic-1)
  else if( FuncName(1:6) == 'RTHETA' ) then
     mno   = 1
     mode  = 1
     mode2 = 1

     if( ipos /= 0 ) then
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     do i = 1, 5
        is = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),                   &
             &                         ',', FuncArg, ip, ipos )
        if( status /= STRING_NOERROR ) return
        if( i < 5 .and. ipos /= 0 ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        select case( i )
        case ( 1 )
           ierr = InquireNumber( trim( FuncArg ), is, mno )
        case ( 2 )
           ierr = InquireNumber( trim( FuncArg ), is, mode )
        case ( 3 )
           ierr = InquireNumber( trim( FuncArg ), is, mode2 )
        case ( 4 )
           ierr = InquireNumber( trim( FuncArg ), is, x1 )
        case ( 5 )
           ierr = InquireNumber( trim( FuncArg ), is, y1 )
        end select
     end do

     if( mode < 1 .or. mode > 2 ) mode = 1
     status = FuncRtheta( stacks, MaxStacks, mno, mode, mode2, x1, y1 )

  ! radial derivation dx/dr
  !     FUNC(DDR,mode,x,y,no)
  !         mode: 1: X/Y  2: LON/LAT
  else if( FuncName(1:3) == 'DDR' ) then
     mode = 1
     no   = 1

     if( ipos /= 0 ) then
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     do i = 1, 4
        is = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),                   &
             &                         ',', FuncArg, ip, ipos )
        if( status /= STRING_NOERROR ) return
        if( i < 3 .and. ipos /= 0 ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        select case( i )
        case ( 1 )
           ierr = InquireNumber( trim( FuncArg ), is, mode )
        case ( 2 )
           ierr = InquireNumber( trim( FuncArg ), is, x1 )
        case ( 3 )
           ierr = InquireNumber( trim( FuncArg ), is, y1 )
        case ( 4 )
           ierr = InquireNumber( trim( FuncArg ), is, no )
        end select
     end do

     if( mode < 1 .or. mode > 2 ) mode = 1
     status = FuncDDR( stacks, MaxStacks, mode, x1, y1, no )

  ! average
  !     FUNC(ASTAT,mode,no)
  !        mode:  1: area average
  !               2: area variance
  !               3: area weight average ( require 3 stacks )
  !               4: area minimum
  !               5: area maximum
  else if( FuncName(1:5) == 'ASTAT' ) then
     mode = 1
     no   = 1

     ! mode
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
     end if

     ! no
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, no )
     end if

     status = FuncAreaStat( stacks, MaxStacks, mode, no )

  ! average
  !     FUNC(AGRID,mode,mode2,no)
  !        mode:  1: 1-2-1 smoothing
  !               2: grid average
  !               3: maximum value
  !               4: minimum value
  !        mode2: the number of grids of average
  else if( FuncName(1:5) == 'AGRID' ) then
     mode  = 1
     mode2 = 1
     no    = 1
     ! mode
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
     end if

     ! mode2
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode2 )
     end if

     ! no
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, no )
     end if

     status = FuncAverageGrid( stacks, MaxStacks, mode, mode2, no )

  ! average
  !     FUNC(GMEAN,mode,start,last,no)
  !        mode:  1: meridional mean ( X )
  !               2: zonal mean      ( y )
  !              11: meridional mean ( LON )
  !              12: meridional mean ( LAT )
  !       start: start x, y, lon, or lat
  !         end: end x, y, lon, or lat
  !          no: the number of stacks
  !
  else if( FuncName(1:5) == 'GMEAN' ) then
     mode  = 1
     x1    = -1.0
     x2    = -1.0
     no    = 1
     ! mode
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
     end if

     ! x1
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, x1 )
     end if

     ! x1
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, x2 )
     end if

     ! no
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, no )
     end if

     status = FuncGridMean( stacks, MaxStacks, mode, x1, x2, no )

  ! masking
  !     FUNC(MASK,mode,from,to,value,no)
  else if( FuncName(1:5) == 'MASK ' ) then
     ! mode
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     if( ipos == 0 ) then
        ierr = InquireNumber( trim( FuncArg ), is, mode )
     else
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     ! FROM
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     if( ipos == 0 ) then
        call ToUpper( FuncArg )
        if( FuncArg(1:5) == 'UNDEF' ) then
           low = RPN_Undefined
        else
           ierr = InquireNumber( trim( FuncArg ), is, low )
           if( ierr /= STRING_NOERROR ) low = RPN_Undefined
        end if
     else
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     ! TO
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     if( ipos == 0 ) then
        call ToUpper( FuncArg )
        if( FuncArg(1:5) == 'UNDEF' ) then
           high = RPN_Undefined
        else
           ierr = InquireNumber( trim( FuncArg ), is, high )
           if( ierr /= STRING_NOERROR ) high = RPN_Undefined
        end if
     else
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     ! val
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     call ToUpper( FuncArg )
     if( FuncArg(1:5) == 'UNDEF' ) then
        val = RPN_Undefined
     else
        ierr = InquireNumber( trim( FuncArg ), is, val )
        if( ierr /= STRING_NOERROR ) val = RPN_Undefined
     end if

     ! no
     no = 1
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, no )
        if( ierr /= STRING_NOERROR ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if
     end if
     status = FuncMask( stacks, MaxStacks, mode, low, high, val, no )

  ! masking
  !     FUNC(MASK2,mode,from,to,value,value2,no)
  else if( FuncName(1:5) == 'MASK2' ) then
     if( ipos /= 0 ) then
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     no = 1
     do i = 1, 6
        is = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),                  &
             &                         ',', FuncArg, ip, ipos )

        if( status /= STRING_NOERROR ) return
        if( i < 5 .and. ipos /= 0 ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        ierr = STRING_NOERROR

        select case ( i )
        case ( 1 )
           ierr = InquireNumber( trim(FuncArg ), is, mode )
        case ( 2 )
           call ToUpper( FuncArg )
           if( FuncArg(1:5) == 'UNDEF' ) then
              low = RPN_Undefined
           else
              ierr = InquireNumber( trim(FuncArg ), is, low )
           end if
        case ( 3 )
           call ToUpper( FuncArg )
           if( FuncArg(1:5) == 'UNDEF' ) then
              high = RPN_Undefined
           else
              ierr = InquireNumber( trim(FuncArg ), is, high )
           end if
        case ( 4 )
           call ToUpper( FuncArg )
           if( FuncArg(1:5) == 'UNDEF' ) then
              val = RPN_Undefined
           else
              ierr = InquireNumber( trim(FuncArg ), is, val )
           end if
        case ( 5 )
           call ToUpper( FuncArg )
           if( FuncArg(1:5) == 'UNDEF' ) then
              val2 = RPN_Undefined
           else
              ierr = InquireNumber( trim(FuncArg ), is, val2 )
           end if
        case ( 6 )
           ierr = InquireNumber( trim(FuncArg ), is, no )
        end select

        if( ierr /= STRING_NOERROR ) low = RPN_Undefined
     end do

     status = FuncMask2( stacks, MaxStacks, mode, low, high, val, val2, no )

  ! band-pass masking
  !     FUNC(BPASS,low,high)
  else if( FuncName(1:5) == 'BPASS' ) then
     if( ipos /= 0 ) then
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     no = 1
     do i = 1, 2
        is = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),                  &
             &                         ',', FuncArg, ip, ipos )

        if( status /= STRING_NOERROR ) return
        if( i < 2 .and. ipos /= 0 ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        ierr = STRING_NOERROR

        select case ( i )
        case ( 1 )
           ierr = InquireNumber( trim(FuncArg ), is, mode )
           call ToUpper( FuncArg )
           if( FuncArg(1:5) == 'UNDEF' ) then
              low = RPN_Undefined
           else
              ierr = InquireNumber( trim(FuncArg ), is, low )
           end if
        case ( 2 )
           call ToUpper( FuncArg )
           if( FuncArg(1:5) == 'UNDEF' ) then
              high = RPN_Undefined
           else
              ierr = InquireNumber( trim(FuncArg ), is, high )
           end if
        end select

        if( ierr /= STRING_NOERROR ) low = RPN_Undefined
     end do

     status = FuncBpass( stacks, MaxStacks, low, high )

  ! counting
  !     FUNC(COUNT,mode,from,to,no)
  else if( FuncName(1:5) == 'COUNT' ) then
     ! mode
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     if( ipos == 0 ) then
        ierr = InquireNumber( trim( FuncArg ), is, mode )
     else
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     ! FROM
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     if( ipos == 0 ) then
        call ToUpper( FuncArg )
        if( FuncArg(1:5) == 'UNDEF' ) then
           low = RPN_Undefined
        else
           ierr = InquireNumber( trim( FuncArg ), is, low )
           if( ierr /= STRING_NOERROR ) low = RPN_Undefined
        end if
     else
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     ! TO
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     if( ipos == 0 ) then
        call ToUpper( FuncArg )
        if( FuncArg(1:5) == 'UNDEF' ) then
           high = RPN_Undefined
        else
           ierr = InquireNumber( trim( FuncArg ), is, high )
           if( ierr /= STRING_NOERROR ) high = RPN_Undefined
        end if
     else
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     ! area ( the number of grid )
     is = 1
     status = AnalyzeFuncComponent( trim( operation ),     &
          &                         ',', FuncArg, ip, ipos )
     call ToUpper( FuncArg )
     if( FuncArg(1:5) == 'UNDEF' ) then
        val = RPN_Undefined
     else
        ierr = InquireNumber( trim( FuncArg ), is, mode2 )
        if( ierr /= STRING_NOERROR ) val = RPN_Undefined
     end if

     ! no
     no = 1
     if( ipos == 0 ) then
        is = 1
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, no )
        if( ierr /= STRING_NOERROR ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if
     end if
     status = FuncCount( stacks, MaxStacks, mode, low, high, mode2, no )

  ! finding center point
  !     FUNC(CPOINT,mode)
  !         mode: 1 maximum                            use 1-stack
  !               2 minimum                            use 1-stack
  !               3 convergence point of stream line   use 2-stacks
  else if( FuncName(1:6) == 'CPOINT' ) then
     is      = 1
     FuncArg = ''
     if( ipos == 0 ) then
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
        if( ierr /= STRING_NOERROR ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if
        status = FuncCenterPoint( stacks, MaxStacks, mode )
     else
        status = RPN_FUNC_ARG_ERROR
        return
     end if

  ! finding tropical cyclone center point
  !     FUNC(TCPOINT,pmax,area,rmin,rmax,dtheta,mode)
  else if( FuncName(1:7) == 'TCPOINT' ) then
     status = RPN_FUNC_ARG_ERROR
     if( ipos /= 0 ) return

     mode = 1
     do i = 1, 6
        is = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),                   &
             &                         ',', FuncArg, ip, ipos )
        if( status /= STRING_NOERROR ) return
        if( i < 5 .and. ipos /= 0 ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        select case( i )
        case ( 1 )
           status = InquireNumber( trim( FuncArg ), is , val )
        case ( 2 )
           status = InquireNumber( trim( FuncArg ), is , val2 )
        case ( 3 )
           status = InquireNumber( trim( FuncArg ), is , low )
        case ( 4 )
           status = InquireNumber( trim( FuncArg ), is , high )
        case ( 5 )
           status = InquireNumber( trim( FuncArg ), is , azimuth )
        case ( 6 )
           status = InquireNumber( trim( FuncArg ), is , mode )
           if( mode < 1 .or. mode > 2 ) mode = 1
        end select
     end do

     status = FuncTCpoint( stacks, MaxStacks,                               &
          &                val, val2, low, high, azimuth, mode  )

  ! wind gust
  !     FUNC(GUST,mode)
  !         mode: 1   [PTV] [WSPD] [TKE]     FUNC(GUST,1)
  !               2   [PTV] [WSPD] [TKE] [Z] FUNC(GUST,2)
  else if( FuncName(1:4) == 'GUST' ) then
     is      = 1
     mode    = 1
     FuncArg = ''
     if( ipos == 0 ) then
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
        if( ierr /= STRING_NOERROR ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        if( mode /= 1 .and. mode /= 2 ) mode = 1
        status = FuncWindGust( stacks, MaxStacks, mode )
     end if

  ! maximum wind speed and minmum slp at TC center
  !     [SST] [PSLV] [P] [T] [QV] FUNC(TCWINSLP,mode)
  !         mode: 1   output -> SLPmin
  !               2   output -> WINDmax
  !              12   output -> SLPmin(stack ic) WINDmax(stack(ic-1)
  else if( FuncName(1:8) == 'TCWINSLP' ) then
     is      = 1
     mode    = 1
     FuncArg = ''
     if( ipos == 0 ) then
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
        if( ierr /= STRING_NOERROR ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        if( mode /= 1 .and. mode /= 2 .and. mode /= 12 ) mode = 1
        status = FuncTCWinSlp( stacks, MaxStacks, mode )
     end if

  ! set time data for the value undefying the date
  !     FUNC(SETTIME,n)
  !         n: use previous n stack to set time
  else if( FuncName(1:7) == 'SETTIME' ) then
     is      = 1
     FuncArg = ''
     if( ipos == 0 ) then
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode )
        if( ierr /= STRING_NOERROR ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if
        status = FuncSetTime( stacks, MaxStacks, mode )
     else
        status = RPN_FUNC_ARG_ERROR
        return
     end if

  ! vertical cross section
  !     FUNC(CROSS,mode,x1,y1,x2,y2,nc,mode2,no)
  else if( FuncName(1:5) == 'CROSS' ) then
     if( ipos /= 0 ) then
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     no = 1
     do i = 1, 8
        is      = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )

        if( status /= STRING_NOERROR ) return
        if( i < 7 .and. ipos /= 0 ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        select case ( i )
        case ( 1 )
           ierr = InquireNumber( trim( FuncArg ), is, mode )
        case ( 2 )
           ierr = InquireNumber( trim( FuncArg ), is, x1 )
        case ( 3 )
           ierr = InquireNumber( trim( FuncArg ), is, y1 )
        case ( 4 )
           ierr = InquireNumber( trim( FuncArg ), is, x2 )
        case ( 5 )
           ierr = InquireNumber( trim( FuncArg ), is, y2 )
        case ( 6 )
           ierr = InquireNumber( trim( FuncArg ), is, nc )
        case ( 7 )
           ierr = InquireNumber( trim( FuncArg ), is, mode2 )
        case ( 8 )
           ierr = InquireNumber( trim( FuncArg ), is, no )
        end select
     end do
     if( no < 1 ) no = 1
     status = FuncCross( stacks, MaxStacks, mode, x1, y1, x2, y2, nc, mode2, no )

  ! create box-mask
  !     FUNC(BMASK,mode,mno,x1,y1,x2,y2,inval,outval)
  else if( FuncName(1:5) == 'BMASK' ) then
     if( ipos /= 0 ) then
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     no = 1
     do i = 1, 8
        is      = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )

        if( status /= STRING_NOERROR ) return
        if( i < 7 .and. ipos /= 0 ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        select case ( i )
        case ( 1 )
           ierr = InquireNumber( trim( FuncArg ), is, mode )
        case ( 2 )
           ierr = InquireNumber( trim( FuncArg ), is, mno )
        case ( 3 )
           ierr = InquireNumber( trim( FuncArg ), is, x1 )
        case ( 4 )
           ierr = InquireNumber( trim( FuncArg ), is, y1 )
        case ( 5 )
           ierr = InquireNumber( trim( FuncArg ), is, x2 )
        case ( 6 )
           ierr = InquireNumber( trim( FuncArg ), is, y2 )
        case ( 7 )
           call ToUpper( FuncArg )
           if( FuncArg(1:5) == 'UNDEF' ) then
              low = RPN_Undefined
           else
              ierr = InquireNumber( trim( FuncArg ), is, low )
              if( ierr /= STRING_NOERROR ) low = RPN_Undefined
           end if
        case ( 8 )
           call ToUpper( FuncArg )
           if( FuncArg(1:5) == 'UNDEF' ) then
              high = RPN_Undefined
           else
              ierr = InquireNumber( trim( FuncArg ), is, high )
              if( ierr /= STRING_NOERROR ) high = RPN_Undefined
           end if
        end select
     end do
     if( no < 1 ) no = 1
     status = FuncBmask( stacks, MaxStacks, mode, mno,                      &
          &              x1, y1, x2, y2, low, high )

  ! get the level of cone
  !     [TER] FUNC(ZCONE,mode,x,y,azimuth,radius,mode2,mode3,level)
  !            mode: 1 x=X, y=Y , 2 x=lon, y=lat
  !            x,y : X,Y or LON,LAT
  !            azimuth: azimuth angle (degree)
  !            radius:  radius (m)
  !            mode2: 0 plane  1 taking earth radius into account
  !            mode3: 1 AGL    2 ASL
  !            level: level (m)
  else if( FuncName(1:5) == 'ZCONE' ) then
     if( ipos /= 0 ) then
        status = RPN_FUNC_ARG_ERROR
        return
     end if

     no    = 1
     mode2 = 0
     mode3 = 1
     low   = 0.
     do i = 1, 8
        is      = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )

        if( status /= STRING_NOERROR ) return
        if( i < 8 .and. ipos /= 0 ) then
           status = RPN_FUNC_ARG_ERROR
           return
        end if

        select case ( i )
        case ( 1 )
           ierr = InquireNumber( trim( FuncArg ), is, mode )
        case ( 2 )
           ierr = InquireNumber( trim( FuncArg ), is, x1 )
        case ( 3 )
           ierr = InquireNumber( trim( FuncArg ), is, y1 )
        case ( 4 )
           ierr = InquireNumber( trim( FuncArg ), is, azimuth )
        case ( 5 )
           ierr = InquireNumber( trim( FuncArg ), is, radius )
        case ( 6 )
           ierr = InquireNumber( trim( FuncArg ), is, mode2 )
        case ( 7 )
           ierr = InquireNumber( trim( FuncArg ), is, mode3 )
        case ( 8 )
           ierr = InquireNumber( trim( FuncArg ), is, low )
        end select
     end do
     if( no < 1 ) no = 1
     status = FuncZcone( stacks, MaxStacks, mode, x1, y1, azimuth, radius, mode2, mode3, low )

  ! output file
  !     FUNC(OUTPUT,format,mode,mode2,no)
  !        format: TEXT, BIN1, BIN2, BIN4, GRD, DF3, V5D, CDF
  !          mode: H2D[c], Z2D, 3D, Z1D, PNT, (1|2|4)BYTE
  !         mode2: 1 (ijkt)    2(lon,lat,lev,t) or (i,lev)
  else if( FuncName(1:6) == 'OUTPUT' ) then
     no    = 1
     mode2 = 1
     OutFormat = ''
     OutMode   = ''
     status    = RPN_FUNC_ARG_ERROR
     if( ipos /= 0 ) return

     mode = AnalyzeFuncComponent( trim( operation ),       &
          &                       ',', OutFormat, ip, ipos )
     if( ipos /= 0 ) return
     mode = AnalyzeFuncComponent( trim( operation ),       &
          &                       ',', OutMode, ip, ipos )

     if( ipos == 0 ) then
        is = 1
        mode = AnalyzeFuncComponent( trim( operation ),     &
             &                       ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode2 )
        if( ierr /= STRING_NOERROR ) return
     end if

     if( ipos == 0 ) then
        is = 1
        mode = AnalyzeFuncComponent( trim( operation ),     &
             &                       ')', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, no )
        if( ierr /= STRING_NOERROR ) return
     end if
     status = FuncOutput( stacks, MaxStacks, OutFormat, OutMode, mode2, no )

  ! histogram
  !     [REG] FUNC(HIST,mode,min,max,inc)
  !        mode: 1 whole data   2: separate region using previous stack
  !         min: minimum value
  !         max: maximum value
  !         inc: increment
  else if( FuncName(1:4) == 'HIST' ) then
     no    = 1
     mode2 = 1
     OutFormat = ''
     OutMode   = ''
     status    = RPN_FUNC_ARG_ERROR
     if( ipos /= 0 ) return

     do i = 1, 4
        is = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        if( status /= STRING_NOERROR ) return
        if( i < 4 .and. ipos /= 0 ) return

        select case( i )
        case ( 1 )
           ierr = InquireNumber( trim( FuncArg ), is, mode )
        case ( 2 )
           ierr = InquireNumber( trim( FuncArg ), is, low  )
        case ( 3 )
           ierr = InquireNumber( trim( FuncArg ), is, high )
        case ( 4 )
           ierr = InquireNumber( trim( FuncArg ), is, val  )
        end select

        if( ierr /= STRING_NOERROR ) return
     end do


     status = FuncHistogram( stacks, MaxStacks, mode, low, high, val )

  ! change data from val to val2
  !     FUNC(UMASK,mode,val,val2,no)
  else if( FuncName(1:5) == 'UMASK' ) then
     no   = 1
     mode = 1
     status    = RPN_FUNC_ARG_ERROR
     if( ipos /= 0 ) return

     do i = 1, 4
        is = 1
        FuncArg = ''
        status = AnalyzeFuncComponent( trim( operation ),     &
             &                         ',', FuncArg, ip, ipos )
        if( status /= STRING_NOERROR ) return
        if( i < 4 .and. ipos /= 0 ) return

        select case( i )
        case ( 1 )
           ierr = InquireNumber( trim( FuncArg ), is, mode )
        case ( 2 )
           ierr = InquireNumber( trim( FuncArg ), is, val  )
        case ( 3 )
           ierr = InquireNumber( trim( FuncArg ), is, val2 )
        case ( 4 )
           ierr = InquireNumber( trim( FuncArg ), is, no   )
        end select

        if( ierr /= STRING_NOERROR ) return
     end do


     status = FuncUmask( stacks, MaxStacks, mode, low, high, val )

  ! parcel method
  !     [T] [QV] [P] FUNC(PARCEL,MODE,MODE2,LEVEL1,LEVEL2)
  !          MODE: PLIFT, PLCL, PLFC, PLNB, TLIFT, TLCL, TLFC, CAPE, CIN
  !         MODE2: unit of LEVEL1 and LEVEL2 ( 1: Pa, 2: grid number )
  !        LEVEL1: start level (  Pa or grid number )
  !        LEVEL2: end level (  Pa or grid number for PLIFT or TLIFT )

  !        format: TEXT, BIN1, BIN2, BIN4, GRD, V5D, CDF
  !          mode: H2D, Z2D, 3D, Z1D, PNT
  !         mode2: 1 (ijkt)    2(lon,lat,lev,t) or (i,lev)
  else if( FuncName(1:6) == 'PARCEL' ) then
     OutMode   = ''
     mode2 = 2
     low   = 1.
     high  = -1.
     status    = RPN_FUNC_ARG_ERROR
     if( ipos /= 0 ) return

     mode = AnalyzeFuncComponent( trim( operation ),       &
          &                       ',', OutMode, ip, ipos )

     if( ipos == 0 ) then
        is = 1
        mode = AnalyzeFuncComponent( trim( operation ),     &
             &                       ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, mode2 )
        if( ierr /= STRING_NOERROR ) return
     end if

     if( ipos == 0 ) then
        is = 1
        mode = AnalyzeFuncComponent( trim( operation ),     &
             &                       ',', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, low )
        if( ierr /= STRING_NOERROR ) return
     end if

     if( ipos == 0 ) then
        is = 1
        mode = AnalyzeFuncComponent( trim( operation ),     &
             &                       ')', FuncArg, ip, ipos )
        ierr = InquireNumber( trim( FuncArg ), is, high )
        if( ierr /= STRING_NOERROR ) return
     end if
     status = FuncParcel( stacks, MaxStacks, OutMode, mode2, low, high )

  ! user defined function
  else if( FuncName(1:4) == 'USER' ) then
     call UserFunction( operation, stacks, MaxStacks, status )

  else
     status = RPN_NOT_DEFINED
  end if

  return
end subroutine ReadFuncData
