  ! push 0-D data into stack
  integer function PushStack0D( stacks, MaxStacks, val, undef, model )
    integer                              , intent(IN)    :: MaxStacks
    type(RPN_STACK), dimension(MaxStacks), intent(INOUT) :: stacks
    real                                 , intent(IN)    :: val
    real           , optional            , intent(IN)    :: undef
    integer        , optional            , intent(IN)    :: model

    integer :: ic

    ic = stacks(1)%current
    if( ic == 0 ) ic = 1

    PushStack0D = RPN_PUSH_ERROR
    if( MaxStacks <= ic ) return

    ic = ic + 1
    allocate( stacks(ic)%val(1,1,1,1) )
    stacks(ic)%dim = 0
    stacks(ic)%ix  = 1
    stacks(ic)%jx  = 1
    stacks(ic)%kx  = 1
    stacks(ic)%it  = 1
    stacks(ic)%val(1,1,1,1) = val

    if( present( undef ) ) then
       stacks(ic)%undef = undef
    else
       stacks(ic)%undef = RPN_Undefined
    end if

    if( present( model ) ) then
       stacks(ic)%model = model
    else
       stacks(ic)%model = RPN_NO_MODEL_DATA
    end if
       
    stacks(1)%current = ic
    PushStack0D = RPN_NOERROR
    return
  end function PushStack0D

  ! push 4-D data into stack
  integer function PushStack4D( stacks, &
       &                        MaxStacks, &
       &                        val,       &
       &                        ix,        &
       &                        jx,        &
       &                        kx,        &
       &                        it,        &
       &                        undef,     &
       &                        model,     &
       &                        omodel,    &
       &                        olev,      &
       &                        cdate,     &
       &                        staggering )
    integer                                , intent(IN)    :: MaxStacks
    type(RPN_STACK), dimension(MaxStacks)  , intent(INOUT) :: stacks
    integer                                , intent(IN)    :: ix, jx, kx, it
    real           , dimension(ix,jx,kx,it), intent(IN)    :: val
    real           , optional              , intent(IN)    :: undef
    integer        , optional              , intent(IN)    :: model
    integer        , optional              , intent(IN)    :: omodel
    real           , optional, dimension(:), intent(IN)    :: olev(kx)
    character(len=24), optional            , intent(IN)    :: cdate
    logical        , optional              , intent(IN)    :: staggering

    integer :: ic, i, j, k, t

    ic = stacks(1)%current
    if( ic == 0 ) ic = 1

    PushStack4D = RPN_PUSH_ERROR
    if( MaxStacks <= ic ) return

    ic = ic + 1
    allocate( stacks(ic)%val(ix,jx,kx,it) )
    stacks(ic)%dim = 4
    stacks(ic)%ix  = ix
    stacks(ic)%jx  = jx
    stacks(ic)%kx  = kx
    stacks(ic)%it  = it
    if( ix == 1 ) stacks(ic)%dim = stacks(ic)%dim - 1
    if( jx == 1 ) stacks(ic)%dim = stacks(ic)%dim - 1
    if( kx == 1 ) stacks(ic)%dim = stacks(ic)%dim - 1
    if( it == 1 ) stacks(ic)%dim = stacks(ic)%dim - 1
    if( present( undef ) ) then
       stacks(ic)%undef = undef
    else
       stacks(ic)%undef = RPN_Undefined
    end if

    if( present( model ) ) then
       stacks(ic)%model = model
    else
       stacks(ic)%model = RPN_NO_MODEL_DATA
    end if

    do t = 1, it
       do k = 1, kx
          do j = 1, jx
             do i = 1, ix
                stacks(ic)%val(i,j,k,t) = val(i,j,k,t)
             end do
          end do
       end do
    end do

    if( present( omodel ) ) stacks(ic)%omodel = omodel
    if( present( olev ) ) then
!       if( associated( stacks(ic)%olev ) ) nullify( stacks(ic)%olev )
       if( associated( stacks(ic)%olev ) ) deallocate( stacks(ic)%olev )
       allocate( stacks(ic)%olev( kx ) )
       stacks(ic)%olev(1:kx) = olev(1:kx)
    end if

    if( present( cdate ) ) then
       stacks(ic)%cdate = cdate
    else
       stacks(ic)%cdate = 'YYYY-MM-DD_hh:mm:ss.ssss'
    end if

    if( present( staggering ) ) then
       stacks(ic)%staggering = staggering
    else
       stacks(ic)%staggering = .false.
    end if

    stacks(1)%current = ic
    PushStack4D = RPN_NOERROR
    return
  end function PushStack4D
