  integer function RPN_SIMILARITY( stacks, MaxStacks )
    integer                              , intent(IN)    :: MaxStacks
    type(RPN_STACK), dimension(MaxStacks), intent(INOUT) :: stacks

    integer :: i, j, k, t, ic
    integer :: ixm, jxm, kxm, itm, jx
    integer :: ixp, jxp, kxp, itp
    integer :: m

    real :: r1, r2

    integer, dimension(:,:)    , allocatable :: cnt
    real   , dimension(:,:,:,:), allocatable :: sim
    real   , dimension(:,:)    , allocatable :: sig1, sig2, corr, ave1, ave2

    ic  = stacks(1)%current

    RPN_SIMILARITY = RPN_STACK_ERROR
    if( ic < 3 ) return

    ixm = stacks(ic-1)%ix
    jxm = stacks(ic-1)%jx
    kxm = stacks(ic-1)%kx
    itm = stacks(ic-1)%it
    ixp = stacks(ic  )%ix
    jxp = stacks(ic  )%jx
    kxp = stacks(ic  )%kx
    itp = stacks(ic  )%it
    m   = stacks(ic)%model

    jx  = 6

    RPN_SIMILARITY = RPN_OPE_ERROR

    if( ixm /= ixp .or. jxm /= jxp .or. kxm /= kxp .or. itm /= itp ) return
!    if( stacks(ic-1)%model /= m ) return

    allocate( sim(1,jx,kxm,itm) )
    allocate( cnt(kxm,itm) )
    allocate( ave1(kxm,itm) )
    allocate( ave2(kxm,itm) )
    allocate( sig1(kxm,itm) )
    allocate( sig2(kxm,itm) )
    allocate( corr(kxm,itm) )

    sim = stacks(ic-1)%undef
    do t = 1, itm
       do k = 1, kxm
          ! sum of the difference
          cnt(k,t)     = 0
          ave1(k,t)    = 0
          ave2(k,t)    = 0
          do j = 1, jxm
             do i = 1, ixm
                if(    stacks(ic  )%val(i,j,k,t) /= stacks(ic  )%undef .and. &
                     & stacks(ic-1)%val(i,j,k,t) /= stacks(ic-1)%undef ) then
                   cnt(k,t) = cnt(k,t) + 1
                   if( sim(1,1,k,t) == stacks(ic-1)%undef ) then
                      sim(1,1,k,t) = ( stacks(ic-1)%val(i,j,k,t) &
                           &         - stacks(ic  )%val(i,j,k,t) )
                      sim(1,3,k,t) = abs( stacks(ic-1)%val(i,j,k,t) &
                           &            - stacks(ic  )%val(i,j,k,t) )
                      ave1(k,t) = stacks(ic-1)%val(i,j,k,t)
                      ave2(k,t) = stacks(ic  )%val(i,j,k,t)
                   else
                      sim(1,1,k,t) = sim(1,1,k,t) &
                           &            + ( stacks(ic-1)%val(i,j,k,t) &
                           &            - stacks(ic  )%val(i,j,k,t) )
                      sim(1,3,k,t) = sim(1,3,k,t) &
                           &            + abs( stacks(ic-1)%val(i,j,k,t) &
                           &                 - stacks(ic  )%val(i,j,k,t) )
                      ave1(k,t) = ave1(k,t) + stacks(ic-1)%val(i,j,k,t)
                      ave2(k,t) = ave2(k,t) + stacks(ic  )%val(i,j,k,t)
                   end if
                end if
             end do
          end do

          ! average of the difference
          if( cnt(k,t) > 0 ) then
             sim(1,2,k,t) = sim(1,1,k,t) / cnt(k,t)
             sim(1,4,k,t) = sim(1,3,k,t) / cnt(k,t)
             ave1(k,t)    = ave1(k,t) / cnt(k,t)
             ave2(k,t)    = ave2(k,t) / cnt(k,t)

             ! square error
             sig1 = 0.
             sig2 = 0.
             corr = 0.
             do j = 1, jxm
                do i = 1, ixm
                   if( stacks(ic)%val(i,j,k,t) /= stacks(ic)%undef .and. &
                        & stacks(ic-1)%val(i,j,k,t) /= stacks(ic-1)%undef ) then
                      r1 = stacks(ic-1)%val(i,j,k,t) - ave1(k,t)
                      r2 = stacks(ic  )%val(i,j,k,t) - ave2(k,t)
                      sig1(k,t) = sig1(k,t) + r1 * r1
                      sig2(k,t) = sig2(k,t) + r2 * r2
                   end if
                end do
             end do

             ! correlation
             do j = 1, jxm
                do i = 1, ixm
                   if( stacks(ic)%val(i,j,k,t) /= stacks(ic)%undef .and. &
                        & stacks(ic-1)%val(i,j,k,t) /= stacks(ic-1)%undef ) then
                      corr(k,t) = corr(k,t)                              &
                           & + ( stacks(ic-1)%val(i,j,k,t) - ave1(k,t) ) &
                           & * ( stacks(ic  )%val(i,j,k,t) - ave2(k,t) )
                   end if
                end do
             end do
             sim(1,5,k,t) = corr(k,t) / sqrt( sig1(k,t) * sig2(k,t) )

          else
             sim(1,2,k,t) = stacks(ic-1)%undef
             sim(1,4,k,t) = stacks(ic-1)%undef
          end if
          sim(1,6,k,t) = cnt(k,t)

       end do
    end do

    call ClearStacks( stacks, MaxStacks )
    stacks(ic-1)%ix = 1
    stacks(ic-1)%jx = jx
    stacks(ic-1)%model = -1
    stacks(ic-1)%omodel = -1
!    if( associated( stacks(ic-1)%val ) ) nullify( stacks(ic-1)%val )
    if( associated( stacks(ic-1)%val ) ) deallocate( stacks(ic-1)%val )
    allocate( stacks(ic-1)%val(1,jx,kxm,itm) )
    stacks(ic-1)%val = sim
    
    RPN_SIMILARITY = RPN_NOERROR
    return
  end function RPN_SIMILARITY

