#!/usr/bin/perl -w

use strict;
use Time::Local;

# output file name
#   def_out: model output
#   def_geo: model geographical data
my $def_out = 'def/model_info_CReSS_T0418c.def';
my $def_geo = 'def/model_geo_info_CReSS_T0418c.def';

# 1 for united data, 0 for node-dump data
my $united = 1;

# user.conf file for CReSS
my $user_conf = '/mnt/e3fs1/T0418c/user.conf.t0418c';

# format of model geography and output file name
my $out_tmpl  = '/mnt/e3fs1/T0418c/t0418c.dmp%06d.united.bin';
my $geo_tmpl  = '/mnt/e3fs1/T0418c/t0418c.geography.united.bin';

# model variable ( .2 or .3 means dimension of its variable )
my @variables = qw(
		   US.2
		   VS.2
		   PS.2
		   PTS.2
		   QVS.2
		   TG.2
		   SH.2
		   LH.2
		   DSR.2
		   DLR.2
		   ULR.2
		   STX.2
		   STY.2
		   SHF.2
		   SMF.2
		   U.3
		   V.3
		   W.3
		   P.3
		   PT.3
		   QV.3
		   QC.3
		   QR.3
		   QI.3
		   QS.3
		   QG.3
		   TKE.3
		   RR.2
		   RRA.2
		   RS.2
		   RSA.2
		   RG.2
		   RGA.2
		   Z.3
		   );

# geography variable ( .2 or .3 means dimension of its variable )
my @variables_geo = qw(
		       HGT.2
		       LAT.2
		       LON.2
		       MAP.2
		       COR.2
		       LAND.2
		       );
# model levels
my @levels = qw(
		0.100000E+03  0.305690E+03
		0.522305E+03  0.748955E+03
		0.984785E+03  0.122898E+04
		0.148077E+04  0.173940E+04
		0.200418E+04  0.227445E+04
		0.254958E+04  0.282898E+04
		0.311210E+04  0.339843E+04
		0.368749E+04  0.397886E+04
		0.427213E+04  0.456694E+04
		0.486297E+04  0.515993E+04
		0.545758E+04  0.575572E+04
		0.605416E+04  0.635278E+04
		0.665148E+04  0.695021E+04
		0.724894E+04  0.754770E+04
		0.784654E+04  0.814556E+04
		0.844489E+04  0.874470E+04
		0.904519E+04  0.934663E+04
		0.964928E+04  0.995347E+04
		0.102596E+05  0.105680E+05
		0.108791E+05  0.111934E+05
		0.115115E+05  0.118338E+05
		0.121610E+05  0.124937E+05
		0.128325E+05  0.131782E+05
		0.135315E+05  0.138931E+05
		0.142639E+05  0.146448E+05
		0.150365E+05  0.154370E+05
		0.158404E+05  0.162439E+05
		0.166473E+05  0.170507E+05
		0.174542E+05  0.178576E+05
		0.182611E+05  0.186645E+05
		0.190680E+05  0.194714E+05
		0.198748E+05  0.202783E+05
		);

###############################################################################
###############################################################################
my $conf = {};
&ReadUserConf( $user_conf, \$conf );

# Total times
my $ntimes = ( $conf->{etime} - $conf->{stime} ) / $conf->{dmpitv} + 1;
if( $united == 1 ){
    $ntimes = ( $conf->{etime} - $conf->{stime} ) / $conf->{flitv_uni} + 1;
}

# Start date
my $stime = -1;
{
    $conf->{sfcast} =~ /(\d+)\/(\d+)\/(\d+)\s+(\d+):(\d+)/;
    my $year   = $1;
    my $month  = $2;
    my $day    = $3;
    my $hour   = $4;
    my $minute = $5;
    $stime = timelocal( 0, $minute, $hour, $day, $month-1, $year-1900 );
}

# Total dates and files
if( $united != 1 ){
    if( $out_tmpl =~ /united/ ){
	$out_tmpl =~ s/united/pe%04d/;
    }
    if( $geo_tmpl =~ /united/ ){
	$geo_tmpl =~ s/united/pe%04d/;
    }
}
my $dates = '';
my $files = '';
for( my $i = 1 ; $i <= $ntimes ; $i++ ){
    my $time = ( $i - 1 ) * $conf->{dmpitv};
    ( $united == 1 ) and ( $time = ( $i - 1 ) * $conf->{flitv_uni} );

    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst )
	= localtime( $stime + $time );
    my $date = sprintf( "%04d-%02d-%02d_%02d:%02d:%02d.0000", $year+1900, $mon+1, $mday, $hour, $min, $sec);
    $dates .= "$date\n";

    my $out_file = '';
    if( $united == 1 ){
	$out_file = sprintf( $out_tmpl, $time );
	$files .= "1 $out_file $i\n";
    }else{
	my $nsep = $conf->{xpedim} * $conf->{ypedim};
	for( my $j = 0 ; $j < $nsep ; $j++ ){
	    $out_file = sprintf( $out_tmpl, $time, $j );
	    $files .= "1 $out_file $i\n";
	}
    }
}

open( DEF, "> $def_out" );
&WriteDefineFile( \*DEF, $dates, $files, \$conf, \@variables, \@levels );
close( DEF );

$dates = "stationary";
if( $united == 0 ){
    my $nsep = $conf->{xpedim} * $conf->{ypedim};
    $files = "";
    for( my $j = 0 ; $j < $nsep ; $j++ ){
	my $out_file .= sprintf( $geo_tmpl, $j );
	$files .= "1 $out_file 1\n";
    }
}else{
    $files = "1 $geo_tmpl 1";
}
@levels = ();
open( DEF, "> $def_geo" );
$ntimes = 1;
&WriteDefineFile( \*DEF, $dates, $files, \$conf, \@variables_geo, \@levels );
close( DEF );

###############################################################################
# write define file
sub WriteDefineFile{
    my ( $fh, $dates, $files, $conf, $var, $lev ) = @_;

    my $ndate = $ntimes;
    my $nfile = $ntimes;
    my $xsep  = 1;
    my $ysep  = 1;
    my $nx    = $$conf->{xdim} - 3;
    my $ny    = $$conf->{ydim} - 3;
    my $nz    = $$conf->{zdim} - 3;
    my $levels = join( "\n", @$lev );
    my $nvars  = @$var;
    my $grid_x = $nx;
    my $grid_y = $ny;
    my $grid_z = $nz;

    $dates =~ s/\n$//;
    $files =~ s/\n$//;

    if( $united != 1 ){
	$xsep  = $$conf->{xpedim};
	$ysep  = $$conf->{ypedim};
	$nfile = $nfile * $xsep * $ysep;

	my $grid_x = $nx;
	my $grid_y = $ny;
	for( my $i = 1 ; $i <= $xsep ; $i++ ){
	    $nx .= " " . $grid_x * $i / $xsep;
	}
	for( my $i = 1 ; $i <= $ysep ; $i++ ){
	    $ny .= " " . $grid_y * $i / $ysep;
	}
    }

    print $fh <<"END";
DataType BINARY
TotalDates $ndate
$dates
/TotalDates
xsep $xsep
ysep $ysep
TotalFiles $nfile
$files
/TotalFiles
EndianType BIG
ByteSize 4
VSLICE FALSE
Undefined -1e+35
GridSizeX $nx
GridSizeY $ny
GridSizeZ $nz
DeltaX $$conf->{dx}
DeltaY $$conf->{dy}
EarthRadius 6.371e+6
HUNIT m
VUNIT m
MPROJ $$conf->{mpopt}
TrueLat1 $$conf->{tlat1}
TrueLat2 $$conf->{tlat2}
TrueLon $$conf->{tlon}
RefLat  $$conf->{ulat}
RefLon  $$conf->{ulon}
RefX 1
RefY 1
END

    my $nlev = @$lev;
    if( $nlev < 1 ){
	print $fh "Level 0 1.\n";
    }else{
	print $fh "Level 100 0\n";
	print $fh "$levels\n";
	print $fh "/Level\n";
    }
    print $fh "TotalVariables $nvars\n";

    for( my $i = 0 ; $i < $nvars ; $i++ ){
	$$var[$i] =~ /([^\.]+)\.(\d+)/;
	my $key = $1;
	my $dim = $2;

	if( $dim == 2 ){
	    printf $fh "%-15s %5d %5d %5d %5d\n", $key, $grid_x, $grid_y, 1, 1;
	}elsif( $dim == 3 ){
	    printf $fh "%-15s %5d %5d %5d %5d\n", $key, $grid_x, $grid_y, $grid_z, 1;
	}
    }
    print $fh "/TotalVariables\n";
}

# read user configuration file
sub ReadUserConf{
    my ( $user_conf, $conf ) = @_;

    my %copt = ( 1 => 'POLSTR',
                 2 => 'LAMCON',
                 3 => 'MERCAT',
                 0 => 'LATLON',
                 4 => 'XY'
                );

    # default
    $$conf->{xpedim}    = 2;
    $$conf->{ypedim}    = 2;
    $$conf->{xdim}      = 503;
    $$conf->{ydim}      = 503;
    $$conf->{zdim}      = 67;
    $$conf->{mpopt}     = 'LAMCON';
    $$conf->{tlat1}     = 30.;
    $$conf->{tlat2}     = 60.;
    $$conf->{tlon}      = 130.;
    $$conf->{ulat}      = 26.0;
    $$conf->{ulon}      = 125.0;
    $$conf->{riu}       = 2.5;
    $$conf->{rju}       = 2.5;
    $$conf->{dx}        = 2500.;
    $$conf->{dy}        = 2500.;
    $$conf->{sfcast}    = '2004/09/06 --:--';
    $$conf->{dmpitv}    = 900.0;
    $$conf->{flitv_uni} = 900.0;
    $$conf->{stime}     = 0.0;
    $$conf->{etime}     = 172800.0;

    open( CONF, $user_conf ) || die;
    while( <CONF> ){
        chomp;
        next if( /^\!/ );
        if( /(\S+)\s*\=\s*\'?([^\']+)\'?/ ){
            my $key = $1;
            my $val = $2;

            if( defined( $$conf->{$key} ) ){
                $$conf->{$key} = $val;
            }
        }
    }
    close( CONF );

#    $$conf->{mpopt}     = $copt{$$conf->{mpopt}};
#    $$conf->{dx}        /= 1000.;
#    $$conf->{dy}        /= 1000.;
#    if( $$conf->{mpopt} eq 'LAMCON' ){
    if( $$conf->{mpopt} == 2 ){
        if( abs( $$conf->{tlat1} ) < abs( $$conf->{tlat2} ) ){
            my $tmp = $$conf->{tlat1};
            $$conf->{tlat1} = $$conf->{tlat2};
            $$conf->{tlat2} = $tmp;
        }
    }
}
