#!/usr/bin/perl -w

use strict;
use Time::Local;

my $config_file = 'hokuriku_5km_setup.txt';
if( @ARGV ){
    $config_file = shift @ARGV;
}

###############################################################################
# constant
my $pi = 3.14159265;
my $er = 6371000.;                # Earth radius in km
my $dr = $pi / 180.;              # coefficient of conversion degree to radian
my %CPROJ = ( 1 => 'LAMCON',      # Lambert Conformal
	      2 => 'POLSTR',      # Polar Stereographic
	      3 => 'MERCAT' );    # Mercator

my $data_dir    = '/Area5/cress/India/mm5/test/CReSS/work/';  # CReSS output (current)
my $work_dir    = '/Area5/cress/India/mm5/test/CReSS/MG/a';    # this directory

my $out_prefix = 'NOBI4KM';
my $dmp_tmpl   = 'Z.dmp';
my $geo_tmpl   = 'Z.geography';
my $userconf   = '';

my $ModelInfoPhys = 'def/model_info_CReSS_cron.def';
my $ModelInfoGeo  = 'def/model_geo_info_CReSS_cron.def';

my $CReSS_version = 2.3;

# model variable ( .2 or .3 means dimension of its variable )
my @variables = ();

#my @variables = qw(
#		   US.2
#		   VS.2
#		   PS.2
#		   PTS.2
#		   QVS.2
#		   TG.2
#		   SH.2
#		   LH.2
#		   DSR.2
#		   DLR.2
#		   ULR.2
#		   CLA.2
#		   STX.2
#		   STY.2
#		   SHF.2
#		   SMF.2
#		   CM.2
#		   CH.2
#		   RIB.2
#		   U.3
#		   V.3
#		   W.3
#                   PBAR.3
#		   PP.3
#                   PTBAR.3
#		   PTP.3
#		   QV.3
#		   QC.3
#		   QR.3
#		   QI.3
#		   QS.3
#		   QG.3
#                   NCI.3
#                   NCS.3
#                   NCG.3
#		   TKE.3
#		   RR.2
#		   RRA.2
#		   RS.2
#		   RSA.2
#		   RG.2
#		   RGA.2
#		   Z.3
#		   );

# geography variable ( .2 or .3 means dimension of its variable )
my @variables_geo = ();
#my @variables_geo = qw(
#		       HGT.2
#		       LAT.2
#		       LON.2
#		       MAP.2
#		       COR.2
#		       LAND.2
#		       );
# model levels

my @levels = ();
#my @levels = qw(
#		0.750000E+02
#		0.253427E+03
#		0.484793E+03
#		0.761560E+03
#		0.107677E+04
#		0.142404E+04
#		0.179758E+04
#		0.219216E+04
#		0.260315E+04
#		0.302648E+04
#		0.345868E+04
#		0.389685E+04
#		0.433867E+04
#		0.478240E+04
#		0.522688E+04
#		0.567152E+04
#		0.611634E+04
#		0.656190E+04
#		0.700937E+04
#		0.746049E+04
#		0.791758E+04
#		0.838354E+04
#		0.886184E+04
#		0.935655E+04
#		0.987231E+04
#		0.104143E+05
#		0.109884E+05
#		0.116009E+05
#		0.122589E+05
#		0.129697E+05
#		0.137278E+05
#		0.145045E+05
#		0.152813E+05
#		0.160581E+05 
#		0.168348E+05
#		0.176116E+05
#		);

###############################################################################
my @mon = qw( JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC );

if( -e $config_file ){
    my @list = ();
    my %def  = ();

    open( CFG, $config_file );
    while( <CFG> ){
        chomp;

	if( /^\#def\s+(\S+)\s+(\S.*)$/ ){
	    $def{$1} = $2;
	}
    
        s/([^\#]*)\#.*$/$1/;
        next unless ( /\S/ );

        if( />/ ){
            s/>/>\n/g;
            s/</\n</g;
            s/\n\n/\n/g;
            foreach my $line ( split( /\n/, $_ ) ){
                next unless ( $line =~ /\S/ );
                push( @list, $line );
            }
        }else{
            s/^\s+//;
            s/\s+$//;
            push( @list, $_ );
        }
	
    }
    close( CFG );

    my $nlist = @list;
    my $key   = '';
    for( my $i = 0 ; $i < $nlist ; $i++ ){
	if( $key eq '' && $list[$i] =~ /<([^>]+)>/ ){
	    $key = $1;
	    if( $key eq 'variables' ){
		@variables = ();
	    }elsif( $key eq 'variables_geo' ){
		@variables_geo = ();
	    }elsif( $key eq 'levels' ){
		@levels = ();
	    }
	    next;

	}elsif( $key ne '' && $list[$i] =~ /<\/${key}>/ ){
	    $key = '';
	    next;

	}else{
	    foreach my $def ( keys %def ){
		$list[$i] =~ s/\@${def}\@/$def{$def}/g;
	    }

	    if( $key eq 'work_dir' ){
		$work_dir = $list[$i];
	    }elsif( $key eq 'data_dir' ){
		$data_dir = $list[$i];
	    }elsif( $key eq 'userconf' ){
		$userconf = $list[$i];
	    }elsif( $key eq 'CReSS_version' ){
		$CReSS_version = $list[$i];
	    }elsif( $key eq 'out_prefix' ){
		$out_prefix = $list[$i];
	    }elsif( $key eq 'dmp_tmpl' ){
		$dmp_tmpl = $list[$i];
	    }elsif( $key eq 'geo_tmpl' ){
		$geo_tmpl = $list[$i];
	    }elsif( $key eq 'ModelInfoPhys' ){
		$ModelInfoPhys = $list[$i];
	    }elsif( $key eq 'ModelInfoGeo' ){
		$ModelInfoGeo = $list[$i];
	    }elsif( $key eq 'variables' ){
		push( @variables, $list[$i] );
	    }elsif( $key eq 'variables_geo' ){
		push( @variables_geo, $list[$i] );
	    }elsif( $key eq 'levels' ){
		push( @levels, $list[$i] );
	    }
	}
    }
}

$CReSS_version =~ /(\d+)\.(\d+)/;
my $CReSS_Major_Version = $1;
my $CReSS_Minor_Version = $2;

unless( -d $work_dir ){ &die_message( "$work_dir is not found" ); }
unless( -d $data_dir ){ &die_message( "$data_dir is not found" ); }


###############################################################################
chdir( $work_dir );

my $dir_list = {};
my $id = &inquire_dir( $data_dir, \$dir_list );

print "ID: $id ( $data_dir )\n";

exit if( $id == 0 );

###########
my $worked = {};
foreach my $stime ( %{$dir_list} ){
    next unless( defined( $dir_list->{$stime}->{conf} ) );

    my $yshift = $dir_list->{$stime}->{yshift};
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst )
	= localtime( $stime );
    $year -= $yshift;
    my $sdate = sprintf( "%04d%02d%02d%02d",
			 ${year}+1900, ${mon}+1, $mday, $hour);
    my $rname = "${out_prefix}${sdate}Z";
    my $nfile = @{$dir_list->{$stime}->{data}};

    # make define files
    &make_define_file( $data_dir,
		       $rname,
		       $dir_list->{$stime}->{phys_united},
		       $ModelInfoPhys,
		       $ModelInfoGeo,
		       $dir_list->{$stime}->{conf},
		       $yshift );

    exit;
}

###############################################################################
sub make_define_file{
    my ( $top_dir, $rname, $united,
	 $def_out, $def_geo, $user_conf, $yshift ) = @_;

    # format of model geography and output file name
    my $out_tmpl  = "${top_dir}/${rname}.dmp%08d.united.bin";
    if( $united == 2 ){
	$out_tmpl  = "${top_dir}/${rname}.dmp%08d.bin";
    }
    if( $CReSS_Major_Version == 2 &&
	$CReSS_Minor_Version == 2 ){
	$out_tmpl  = "${top_dir}/${rname}.dmp%06d.united.bin";
	if( $united == 2 ){
	    $out_tmpl  = "${top_dir}/${rname}.dmp%06d.bin";
	}
    }
    my $geo_tmpl  = "${top_dir}/${rname}.geography.united.bin";
    if( $united == 2 ){
	$geo_tmpl  = "${top_dir}/${rname}.geography.bin";
    }

    my $conf = {};
    &ReadUserConf( $user_conf, \$conf );

    if( $userconf ne '' ){
	my $exprim = $conf->{exprim};
	$out_tmpl  = "${top_dir}/${exprim}.dmp%08d.united.bin";
	if( $united == 2 ){
	    $out_tmpl  = "${top_dir}/${exprim}.dmp%08d.bin";
	}
	if( $CReSS_Major_Version == 2 &&
	    $CReSS_Minor_Version == 2 ){
	    $out_tmpl  = "${top_dir}/${exprim}.dmp%06d.united.bin";
	    if( $united == 2 ){
		$out_tmpl  = "${top_dir}/${exprim}.dmp%06d.bin";
	    }
	}
	$geo_tmpl  = "${top_dir}/${exprim}.geography.united.bin";
	if( $united == 2 ){
	    $geo_tmpl  = "${top_dir}/${exprim}.geography.bin";
	}
    }

    # Total times
    my $ntimes = ( $conf->{etime} - $conf->{stime} ) / $conf->{dmpitv} + 1;
    if( $united == 1 ){
	$ntimes = ( $conf->{etime} - $conf->{stime} ) / $conf->{flitv_uni} + 1;
    }

    # Start date
    my $stime = -1;
    {
	$conf->{sfcast} =~ /(\d+)\/(\d+)\/(\d+)\s+(\d+):(\d+)/;
	my $year   = $1;
	my $month  = $2;
	my $day    = $3;
	my $hour   = $4;
	my $minute = $5;
	$year += $yshift;
	$stime = timelocal( 0, $minute, $hour, $day, $month-1, $year-1900 );
    }

    # Total dates and files
    if( $united < 1 ){
	if( $out_tmpl =~ /united/ ){
	    $out_tmpl =~ s/united/pe%04d/;
	}
	if( $geo_tmpl =~ /united/ ){
	    $geo_tmpl =~ s/united/pe%04d/;
	}
    }

    my $dates = '';
    my $files = '';
    for( my $i = 1 ; $i <= $ntimes ; $i++ ){
	my $time = ( $i - 1 ) * $conf->{dmpitv};
	( $united == 1 ) and ( $time = ( $i - 1 ) * $conf->{flitv_uni} );

	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst )
	    = localtime( $stime + $time );
	my $date = sprintf( "%04d-%02d-%02d_%02d:%02d:%02d.0000",
			    $year-$yshift+1900, $mon+1, $mday, $hour, $min, $sec);
	$dates .= "$date\n";

	my $out_file = '';
	if( $united >= 1 ){
	    $out_file = sprintf( $out_tmpl, $time );
	    $files .= "1 $out_file $i\n";
	}else{
	    my $nsep = $conf->{xpedim} * $conf->{ypedim};
	    for( my $j = 0 ; $j < $nsep ; $j++ ){
		$out_file = sprintf( $out_tmpl, $time, $j );
		$files .= "1 $out_file $i\n";
	    }
	}
    }

    my $nlev = @levels;
    if( $nlev < 1 ){
	@levels = &getz( \$conf );
    }

    my $nvar = @variables;
    if( $nvar < 1 ){
	@variables = &inquire_variable( \$conf, 1 );
    }

    open( DEF, "> $def_out" );
    &WriteDefineFile( \*DEF,
		      $dates,
		      $files,
		      \$conf,
		      \@variables,
		      \@levels,
		      $ntimes,
		      $united );
    close( DEF );

    $dates = "stationary";
    if( $united == 0 ){
	my $nsep = $conf->{xpedim} * $conf->{ypedim};
	$files = "";
	for( my $j = 0 ; $j < $nsep ; $j++ ){
	    my $out_file .= sprintf( $geo_tmpl, $j );
	    $files .= "1 $out_file 1\n";
	}
    }else{
	$files = "1 $geo_tmpl 1";
    }
    @levels = ();

    my $nvar_geo = @variables_geo;
    if( $nvar_geo < 1 ){
	@variables_geo = &inquire_variable( \$conf, 2 );
    }

    open( DEF, "> $def_geo" );
    $ntimes = 1;
    &WriteDefineFile( \*DEF,
		      $dates,
		      $files,
		      \$conf,
		      \@variables_geo,
		      \@levels,
		      $ntimes,
		      $united );
    close( DEF );
}

###############################################################################
# set variables
sub inquire_variable{
    my ( $conf, $mode ) = @_;

    my @v = ();

    unless( $CReSS_Major_Version == 2 &&
	    $CReSS_Minor_Version == 3 ){
	return '';
    }

    if( $mode == 1 ){
	@v = &inquire_variable_2_3( $conf );
    }elsif( $mode == 2 ){
	@v = &inquire_geo_variable_2_3( $conf );
    }

    return @v;
}

sub inquire_geo_variable_2_3{
    my ( $conf ) = @_;

    my @v = ();
    my $dmpvar = $$conf->{dmpvar};

    push( @v, 'HT.2' );
    push( @v, 'ALAT.2' );
    push( @v, 'ALON.2' );

    if( $$conf->{mfcopt} == 1 ){
	push( @v, 'MAP.2' );
    }

    if( $$conf->{coropt} >= 1 ){
	push( @v, 'FS.2' );

	if( $$conf->{coropt} == 2 ){
	    push( @v, 'FC.2' );
	}
    }

    if( $$conf->{sfcopt} >= 1 ){
	push( @v, 'LAND.2' );
    }

    return @v;
}

sub inquire_variable_2_3{
    my ( $conf ) = @_;

    my @v = ();
    my $dmpvar = $$conf->{dmpvar};

    if( $$conf->{sfcopt} >= 1 &&
	substr( $dmpvar, 11,1 ) eq 'o' ){
	push( @v, 'US.2' );
	push( @v, 'VS.2' );
	push( @v, 'PS.2' );
	push( @v, 'PTS.2' );
	push( @v, 'QVS.2' );
	push( @v, 'TGS.2' );
	push( @v, 'HS.2' );
	push( @v, 'LE.2' );
	push( @v, 'RSD.2' );
	push( @v, 'RLD.2' );
	push( @v, 'RLU.2' );
	push( @v, 'CDALL.2' );
	push( @v, 'USFRC.2' );
	push( @v, 'VSFRC.2' );
	push( @v, 'PTSFRC.2' );
	push( @v, 'QVSFRC.2' );
    }

    if( substr( $dmpvar, 0, 1 ) eq 'o' ){
	push( @v, 'UBAR.3' );
	push( @v, 'U.3' );
    }elsif( substr( $dmpvar, 0, 1 ) eq '-' ){
	push( @v, 'U.3' );
    }

    if( substr( $dmpvar, 1, 1 ) eq 'o' ){
	push( @v, 'VBAR.3' );
	push( @v, 'V.3' );
    }elsif( substr( $dmpvar, 1, 1 ) eq '-' ){
	push( @v, 'V.3' );
    }

    if( substr( $dmpvar, 2, 1 ) eq 'o' ){
	push( @v, 'W.3' );
    }

    if( substr( $dmpvar, 3, 1 ) eq 'o' ){
	push( @v, 'PBAR.3' );
	push( @v, 'PP.3' );
    }elsif( substr( $dmpvar, 3, 1 ) eq '-' ){
	push( @v, 'P.3' );
    }

    if( substr( $dmpvar, 4, 1 ) eq 'o' ){
	push( @v, 'PTBAR.3' );
	push( @v, 'PTP.3' );
    }elsif( substr( $dmpvar, 4, 1 ) eq '-' ){
	push( @v, 'PT.3' );
    }

    if( substr( $dmpvar, 5, 1 ) eq 'o' ){
	push( @v, 'QVBAR.3' );
	push( @v, 'QVP.3' );
    }elsif( substr( $dmpvar, 5, 1 ) eq '-' ){
	push( @v, 'QV.3' );
    }

    if( $$conf->{cphopt} < 10 ){
	if( $$conf->{cphopt} >= 1 && substr( $dmpvar, 6, 1 ) eq 'o' ){
	    push( @v, 'QC.3' );
	    push( @v, 'QR.3' );
	}

	if( $$conf->{cphopt} >= 2 && substr( $dmpvar, 6, 1 ) eq 'o' ){
	    push( @v, 'QI.3' );
	    push( @v, 'QS.3' );
	    push( @v, 'QG.3' );
	}

	if( $$conf->{cphopt} >= 3 && substr( $dmpvar, 7, 1 ) eq 'o' ){
	    push( @v, 'NCI.3' );
	    push( @v, 'NCS.3' );
	    push( @v, 'NCG.3' );
	}

	if( $$conf->{cphopt} < 0 && substr( $dmpvar, 8, 1 ) eq 'o' ){
#	    if( $$conf->{qcgopt} == 2 ){
#		push( @v, 'QCC.3' );
#		push( @v, 'QCR.3' );
#	    }

	    push( @v, 'QCI.3' );
	    push( @v, 'QCS.3' );
	    push( @v, 'QCG.3' );
	}

    }else{
	if( abs( $$conf->{cphopt} ) >= 11 ){
	    if( substr( $dmpvar, 6, 1 ) eq 'o' ){
		push( @v, 'QLQ.3' );
	    }

	    if( substr( $dmpvar, 7, 1 ) eq 'o' ){
		push( @v, 'NLQ.3' );
	    }
	}

	if( abs( $$conf->{cphopt} ) == 12 ){
	    if( substr( $dmpvar, 6, 1 ) eq 'o' ){
		push( @v, 'QSL' );
	    }

	    if( substr( $dmpvar, 7, 1 ) eq 'o' ){
		push( @v, 'NSL' );
	    }
	}
    }

    if( abs( $$conf->{tubopt} ) >= 2 ){
	if( substr( $dmpvar, 9, 1 ) eq '+' || 
	    substr( $dmpvar, 9, 1 ) eq 'o' ){
	    push( @v, 'TKE.3' );
	}

	if( substr( $dmpvar, 9, 1 ) eq '+' || 
	    substr( $dmpvar, 9, 1 ) eq '-' ){
	    push( @v, 'MAXVL.3' );
	}
    }

    if( $$conf->{trkopt} >= 1 && substr( $dmpvar, 10, 1 ) eq 'o' ){
	push( @v, 'QT.3' );
    }

    if( abs( $$conf->{cphopt} ) < 10 ){
	if( abs( $$conf->{cphopt} ) >= 1 && substr( $dmpvar, 12, 1 ) eq 'o' ){
	    push( @v, 'QRR.2' );
	    push( @v, 'QRA.2' );
	}

	if( abs( $$conf->{cphopt} ) >= 2 && substr( $dmpvar, 12, 1 ) eq 'o' ){
	    push( @v, 'QSR.2' );
	    push( @v, 'QSA.2' );
	    push( @v, 'QGR.2' );
	    push( @v, 'QGA.2' );
	}

    }else{
	if( abs( $$conf->{cphopt} ) >= 11 && substr( $dmpvar, 12, 1 ) eq 'o' ){
	    push( @v, 'QWR.2' );
	    push( @v, 'QWA.2' );
	}

	if( abs( $$conf->{cphopt} ) >= 12 && substr( $dmpvar, 12, 1 ) eq 'o' ){
	    push( @v, 'QIR.2' );
	    push( @v, 'QIA.2' );
	}
    }

    if( $$conf->{dmpcmp} % 10  == 1 && substr( $dmpvar, 13, 1 ) eq 'o' ){
	push( @v, 'ZPH.3' );
    }

    return @v;
}

###############################################################################
# calculate zeta coordinate
sub getz{
    my ( $conf ) = @_;

    my $eps   = 1.e-20;
    my $tanh2 = 0.96403;

    my $nz     = $$conf->{zdim};
    my $dz     = $$conf->{dz};
    my $dzmin  = $$conf->{dzmin};
    my $zsfc   = $$conf->{zsfc};
    my $zflat  = $$conf->{zflat};
    my $layer1 = $$conf->{layer1};
    my $layer2 = $$conf->{layer2};
    my $sthopt = $$conf->{sthopt};
    my $dmpcmp = $$conf->{dmpcmp} % 10;

    my @z = ();
    for( my $k = 0 ; $k < $nz ; $k++ ){
	$z[$k] = $zsfc + ( $k - 1 ) * $dz
    }

    my $ztop1 = $z[$nz-3];
    my $ztop2 = $z[$nz-2];
    if( $layer1 > ( $ztop1 - $zsfc ) ){
	$layer1 = $ztop1 - $zsfc;
    }

    if( $layer1 < 0.0 ){
	$layer1 = 0.0;
    }

    my $z1 = $zsfc + $layer1;

    if( $layer2 < 0.0 ){
	$layer2 = 0.0;
    }

    my $z2 = $z1 + $layer2;

    my @zsth = ();
    if( int( ($z1+$eps)/($ztop2+$eps)+0.1 ) == 1 &&
	int( ($z1-$zsfc+$eps)/(($nz-3)*$dzmin+$eps)+0.1 ) == 1 ){

	my $dzall = ( $ztop2 - $zsfc ) / ( $nz - 3 );
	for( my $k = 0 ; $k < $nz ; $k++ ){
	    $zsth[$k] = $zsfc + ($k-1)*$dzall;
	}

    }else{
	if( $z1 < $zsfc ){
	    $z1 = $zsfc;
	}

	if( $z2 > $ztop2 ){
	    $z2 = $ztop2;
	}

	my $klow = int( ($z1-$zsfc)/$dzmin + 0.1 );

	my $dzmid  = $dz;
	my $dzhigh = $dz;
	my $kmid   = 0;
	if( $z2 >= $ztop2 ){
	    $dzmid = ( $ztop2 - $zsfc - $klow * $dzmin ) / ( $nz -3 - $klow );
	    $dzhigh = 2.0 * $dzmid - $dzmin;

	}elsif( $z2 < $ztop2 ){
	    my $a = 2.0 * ( $nz - 3 - $klow );
	    my $b = 2.0 * $zsfc - $ztop2 - $z2 - ($nz-3-3*$klow)*$dzmin;
	    my $c = $dzmin * ( $z2-$zsfc-$klow*$dzmin );

	    $dzmid = 0.5 * ( sqrt( $b*$b - 4.0*$a*$c ) - $b ) / $a;
	    $kmid  = int( ($ztop2-$z2)/(2.0*$dzmid-$dzmin) + 0.1 );

	    $a = $zsfc + ($klow-$kmid)*$dzmin + ($nz-3-$klow+$kmid)*$dzmid;

	    if( $kmid != 0 ){
		$dzhigh = ( 2.0*$dzmid - $dzmin ) + ( $ztop2 - $a ) / $kmid;
	    }elsif( $nz-3-$klow-$kmid <= 0 ){
		$dzmid = $dzmid + ( $ztop2 - $a ) / ( $nz-3-$klow-$kmid );
		$dzhigh = 2.0*$dzmid - $dzmin
	    }
	}

	my @dzsth = ();
	for( my $k = 0 ; $k < $klow+1 ; $k++ ){
	    $dzsth[$k] = $dzmin;
	}

	if( $sthopt == 1 ){
	    my $a = $dzmid - $dzmin;
	    my $b = 1.0 / ( $nz - 4 - $kmid - $klow );

	    for( my $k = $klow+1 ; $k < $nz-2-$kmid ; $k++ ){
		my $c = 2.0 * $b * ( $k-1-$klow ) - 1.0;
		$dzsth[$k] = $dzmid + $a * $c * $c * $c;
	    }

	}elsif( $sthopt == 2 ){
	    my $a = 0.5 * ( $nz-$kmid+$klow);

	    my $b = 0.0;
	    if( 2*($klow+2) - ($nz-$kmid+$klow) == 0 ){
		$b = 0.0;
	    }else{
		$b = 2.0 / ( $klow + 2 - $a );
	    }
	    my $c = ( $dzmid - $dzmid ) / $tanh2;

	    for( my $k = $klow+1 ; $k < $nz-2-$kmid ; $k++ ){
		$dzsth[$k] = $dzmid + $c*&tanh($b*($k+1)-$a);
	    }
	}

	for( my $k = $nz-2-$kmid ; $k < $nz-2 ; $k++ ){
	    $dzsth[$k] = $dzhigh;
	}
	$dzsth[$nz-2] = $dzsth[$nz-3];
	$dzsth[$nz-1] = $dzsth[$nz-2];

	$zsth[1] = $zsfc;

	for( my $k = 2 ; $k < $nz-2 ; $k++ ){
	    $zsth[$k] = $zsth[$k-1] + $dzsth[$k-1];
	}

	$zsth[0] = $zsth[1] - $dzsth[0];
	$zsth[$nz-2] = $ztop2;
	$zsth[$nz-1] = $zsth[$nz-2] + $dzsth[$nz-2];

	if( $dmpcmp == 1 ){
	    for( my $k = 0 ; $k < $nz ; $k++ ){
		$z[$k] = $k;
	    }
	}elsif( $dmpcmp == 2 ){
	    for( my $k = 1; $k < $nz-2 ; $k++ ){
		$z[$k] = $dz * ( $k - 0.5 );
	    }
	}elsif( $dmpcmp == 3 ){
	    for( my $k = 1 ; $k < $nz-2 ; $k++ ){
		$z[$k] = 0.5 * ( $zsth[$k] + $zsth[$k+1] );
	    }
	}
	shift( @z );
	pop( @z );
	pop( @z );

    }

    return @z;
}

###############################################################################
sub tanh{
    my ( $x ) = @_;

    my $out = ( exp( $x ) - exp( - $x ) ) / ( exp( $x ) + exp( - $x ) );
    return $out;
}

###############################################################################
# inquire informations of directory
sub inquire_dir{
    my ( $data_dir, $dir_list ) = @_;

    my $expname = '';
    my $estime  = '';
    if( $userconf ne '' && -e $userconf ){
	open( CNF, $userconf ) || die "$userconf\n";
	while( <CNF> ){
	    if( /exprim\s*=\s*\'([^\']+)\'/ ){
		$expname = $1;

	    }elsif( /sfcast\s*=\s*\'([^\']+)\'/ ){
		my $sfcast = $1;
		$sfcast =~ /(\d+)\/(\d+)\/(\d+)\s+(\d+)\:(\d+)/;
		my $year   = $1;
		my $month  = $2;
		my $day    = $3;
		my $hour   = $4;
		my $minute = $5;
		my $yshift = 0;
		if( $year > 2036 ){
		    $year  -= 100;
		    $yshift = -100;
		}elsif( $year < 1936 ){
		    $year  += 100;
		    $yshift = 100;
		}elsif( $year < 1970 ){
		    $year  += 40;
		    $yshift = 40;
		}


		$estime = timelocal( 0, $minute, $hour, $day,
				     ${month}-1, ${year}-1900 );
		$$dir_list->{$estime}->{yshift} = $yshift;

		# default values
		unless( defined( $$dir_list->{$estime}->{phys_united} ) ){
		    $$dir_list->{$estime}->{phys_united} = -1;
		}

		unless( defined( $$dir_list->{$estime}->{geo_united} ) ){
		    $$dir_list->{$estime}->{geo_united} = -1;
		}
	    }
	}
	close( CNF );
	$$dir_list->{$estime}->{conf} = $userconf;
    }

    my $id = 0;
    my @id = ( 0, 0 );
    while( <${data_dir}/*> ){
	my $file = $_;

	if( $expname ne '' ){
	    # physical data
	    if( $file =~ /${expname}\.dmp(\d+)\.(\S+)$/ ){
		$id = 1;
		my $ctime  = $1;
		my $united = $2;

		# united data
		if( $united =~ /united/i ){
                    $id[0] = 1;
                    return 0 if( $id[1] == 1 );
		    $$dir_list->{$estime}->{phys_united} = 1;
		    push( @{$$dir_list->{$estime}->{data}}, $ctime );
		
		# parted data
		}elsif( $united =~ /pe/i ){
                    $id[1] = 1;
                    return 0 if( $id[0] == 1 );
		    $$dir_list->{$estime}->{phys_united} = 0;
		    if( $united =~ /pe0000/ ){
			push( @{$$dir_list->{$estime}->{data}}, $ctime );
		    }

		# single data
		}elsif( $united =~ /bin/i ){
                    $id[0] = 1;
                    return 0 if( $id[1] == 1 );
		    $$dir_list->{$estime}->{phys_united} = 2;
		    push( @{$$dir_list->{$estime}->{data}}, $ctime );

		}

	    }elsif( $file =~ /${expname}\.geography\.(\S+)$/ ){
		my $united = $1;

		# united data
		if( $united =~ /united/i ){
		    $$dir_list->{$estime}->{geo_united} = 1;
		
		# parted data
		}elsif( $united =~ /pe/i ){
		    $$dir_list->{$estime}->{geo_united} = 0;

		# single data
		}elsif( $united =~ /bin/i ){
		    $$dir_list->{$estime}->{geo_united} = 2;

		}
	    }

	}elsif( $file =~ /$out_prefix(\d{4})(\d\d)(\d\d)(\d\d)/ ){
	    my $year  = $1;
	    my $month = $2;
	    my $day   = $3;
	    my $hour  = $4;
	    my $stime = timelocal( 0, 0, $hour, $day,
				   ${month}-1, ${year}-1900 );

	    # default values
	    unless( defined( $$dir_list->{$stime}->{phys_united} ) ){
		$$dir_list->{$stime}->{phys_united} = -1;
	    }

	    unless( defined( $$dir_list->{$stime}->{geo_united} ) ){
		$$dir_list->{$stime}->{geo_united} = -1;
	    }

	    # physical data
	    if( $file =~ /${dmp_tmpl}(\d+)\.(\S+)$/ ){
	        $id = 1;
		my $ctime  = $1;
		my $united = $2;

		# united data
		if( $united =~ /united/i ){
                    $id[0] = 1;
                    return 0 if( $id[1] == 1 );
		    $$dir_list->{$stime}->{phys_united} = 1;
		    push( @{$$dir_list->{$stime}->{data}}, $ctime );
		
		# parted data
		}elsif( $united =~ /pe/i ){
                    $id[1] = 1;
                    return 0 if( $id[0] == 1 );
		    $$dir_list->{$stime}->{phys_united} = 0;
		    if( $united =~ /pe0000/ ){
			push( @{$$dir_list->{$stime}->{data}}, $ctime );
		    }

		# single data
		}elsif( $united =~ /bin/i ){
                    $id[0] = 1;
                    return 0 if( $id[1] == 1 );
		    $$dir_list->{$stime}->{phys_united} = 2;
		    push( @{$$dir_list->{$stime}->{data}}, $ctime );

		}

	    # geographical data
	    }elsif( $file =~ /${geo_tmpl}\.(\S+)$/ ){
		my $united = $1;

		# united data
		if( $united =~ /united/i ){
		    $$dir_list->{$stime}->{geo_united} = 1;
		
		# parted data
		}elsif( $united =~ /pe/i ){
		    $$dir_list->{$stime}->{geo_united} = 0;

		# single data
		}elsif( $united =~ /bin/i ){
		    $$dir_list->{$stime}->{geo_united} = 2;

		}

	    # user conf
	    }elsif( $file =~ /user\.conf\.${out_prefix}(\d{4})(\d\d)(\d\d)(\d\d)/ ){
                $file =~ s/\/\//\//g;
		$$dir_list->{$stime}->{conf} = $file;

	    }

	}
    }

    if( $id[0] == 1 && $id[1] == 1 ){ $id = 0; }
    return $id;
}

###############################################################################
# write define file
sub WriteDefineFile{
    my ( $fh, $dates, $files, $conf, $var, $lev, $ntimes, $united ) = @_;

    my $ndate = $ntimes;
    my $nfile = $ntimes;
    my $xsep  = 1;
    my $ysep  = 1;
    my $nx    = $$conf->{xdim} - 3;
    my $ny    = $$conf->{ydim} - 3;
    my $nz    = $$conf->{zdim} - 3;
    my $riu   = $$conf->{riu} - 1.5;
    my $rju   = $$conf->{rju} - 1.5;
    $nz = @$lev;
    ( $nz == 0 ) and ( $nz = 1 );
    my $levels = join( "\n", @$lev );
    my $nvars  = @$var;
    my $grid_x = $nx;
    my $grid_y = $ny;
    my $grid_z = $nz;

    $dates =~ s/\n$//;
    $files =~ s/\n$//;

    if( $united < 1 ){
	$xsep  = $$conf->{xpedim};
	$ysep  = $$conf->{ypedim};
	$nfile = $nfile * $xsep * $ysep;

	my $grid_x = $nx;
	my $grid_y = $ny;
	for( my $i = 1 ; $i <= $xsep ; $i++ ){
	    $nx .= " " . $grid_x * $i / $xsep;
	}
	for( my $i = 1 ; $i <= $ysep ; $i++ ){
	    $ny .= " " . $grid_y * $i / $ysep;
	}
    }

    print $fh <<"END";
DataType BINARY
TotalDates $ndate
$dates
/TotalDates
xsep $xsep
ysep $ysep
TotalFiles $nfile
$files
/TotalFiles
EndianType BIG
ByteSize 4
VSLICE FALSE
Undefined -1e+35
GridSizeX $nx
GridSizeY $ny
GridSizeZ $nz
DeltaX $$conf->{dx}
DeltaY $$conf->{dy}
EarthRadius 6.371e+6
HUNIT m
VUNIT m
MPROJ $$conf->{mpopt}
TrueLat1 $$conf->{tlat1}
TrueLat2 $$conf->{tlat2}
TrueLon $$conf->{tlon}
RefLat  $$conf->{ulat}
RefLon  $$conf->{ulon}
RefX $riu
RefY $rju
END

    my $nlev = @$lev;
    if( $nlev < 1 ){
	print $fh "Level 0 1.\n";
    }else{
	print $fh "Level 100 0\n";
	print $fh "$levels\n";
	print $fh "/Level\n";
    }
    print $fh "TotalVariables $nvars\n";

    for( my $i = 0 ; $i < $nvars ; $i++ ){
	$$var[$i] =~ /([^\.]+)\.(\d+)/;
	my $key = $1;
	my $dim = $2;

	if( $dim == 2 ){
	    printf $fh "%-15s %5d %5d %5d %5d\n", $key, $grid_x, $grid_y, 1, 1;
	}elsif( $dim == 3 ){
	    printf $fh "%-15s %5d %5d %5d %5d\n", $key, $grid_x, $grid_y, $grid_z, 1;
	}
    }
    print $fh "/TotalVariables\n";
}

###############################################################################
# read user configuration file
sub ReadUserConf{
    my ( $user_conf, $conf ) = @_;

    my %copt = ( 1 => 'POLSTR',
                 2 => 'LAMCON',
                 3 => 'MERCAT',
                 0 => 'LATLON',
                 4 => 'XY'
                );

    # default
    $$conf->{exprim}    = '';
    $$conf->{xpedim}    = 2;
    $$conf->{ypedim}    = 2;
    $$conf->{xdim}      = 503;
    $$conf->{ydim}      = 503;
    $$conf->{zdim}      = 67;
    $$conf->{mpopt}     = 'LAMCON';
    $$conf->{mfcopt}    = 1;
    $$conf->{tlat1}     = 30.;
    $$conf->{tlat2}     = 60.;
    $$conf->{tlon}      = 130.;
    $$conf->{ulat}      = 26.0;
    $$conf->{ulon}      = 125.0;
    $$conf->{riu}       = 2.5;
    $$conf->{rju}       = 2.5;
    $$conf->{dx}        = 2500.;
    $$conf->{dy}        = 2500.;
    $$conf->{dz}        = 320.;
    $$conf->{zsfc}      = 0.;
    $$conf->{zflat}     = 18000.;
    $$conf->{sthopt}    = 1;
    $$conf->{dzmin}     = 200.;
    $$conf->{layer1}    = 0.;
    $$conf->{layer2}    = 18000.;
    $$conf->{sfcast}    = '2004/09/06 --:--';
    $$conf->{sfcopt}    = 1;
    $$conf->{cphopt}    = 1;
    $$conf->{tubopt}    = 1;
    $$conf->{trkopt}    = 0;
    $$conf->{coropt}    = 1;
    $$conf->{dmpcmp}    = 1;
    $$conf->{dmpvar}    = '';
    $$conf->{dmpitv}    = 900.0;
    $$conf->{flitv_uni} = 900.0;
    $$conf->{stime}     = 0.0;
    $$conf->{etime}     = 172800.0;

    open( CONF, $user_conf ) || die;
    while( <CONF> ){
        chomp;
        next if( /^\!/ );
	s/\s*\!.*$//;
        if( /(\S+)\s*\=\s*\'?([^\']+)\'?/ ){
            my $key = $1;
            my $val = $2;

            if( defined( $$conf->{$key} ) ){
                $$conf->{$key} = $val;
            }
        }
    }
    close( CONF );

#    $$conf->{mpopt}     = $copt{$$conf->{mpopt}};
#    $$conf->{dx}        /= 1000.;
#    $$conf->{dy}        /= 1000.;
#    if( $$conf->{mpopt} eq 'LAMCON' ){
    if( $$conf->{mpopt} == 1 ){
         $$conf->{mpopt} = 3;
    }elsif( $$conf->{mpopt} == 2 ){
        if( abs( $$conf->{tlat1} ) < abs( $$conf->{tlat2} ) ){
            my $tmp = $$conf->{tlat1};
            $$conf->{tlat1} = $$conf->{tlat2};
            $$conf->{tlat2} = $tmp;
        }
    }elsif( $$conf->{mpopt} == 3 ){
         $$conf->{mpopt} = 1;
    }
}

###############################################################################
sub die_message{
    my ( $message ) = @_;

    print STDERR "$message\n";
    exit;
}
