#!/usr/bin/perl -w

use strict;

###############################################################################
# top directory of GMT
my $gmt_top = '/usr/local/gmt4';

###############################################################################
# the order of drawing picture
#     grdimage must be first
my @order = qw( grdimage psscale grdcontour psbasemap pstext );

###############################################################################
# do not change next line
my $GMT = {};

###############################################################################
# grdimage
#   grd, cpt, and opt( optional )
$GMT->{grdimage}->{grd}  = 'z_hvor.grd';
$GMT->{grdimage}->{cpt}  = 'z_hvor.cpt';
$GMT->{grdimage}->{opt}  = '-X1.5i';

###############################################################################
# psscale
#   cpt, position, opt( optional ), and format( optional )
$GMT->{psscale}->{cpt}      = 'z_hvor.cpt';
$GMT->{psscale}->{position} = '-D7.5i/2i/4i/0.2i';
$GMT->{psscale}->{opt}      = '-L -E';
$GMT->{psscale}->{format}   = '%7.3f';

###############################################################################
# grdcontour
#   grd and opt
$GMT->{grdcontour}->{grd}  = 'z_wspd.grd';
$GMT->{grdcontour}->{opt}  = '-C5 -A10';

###############################################################################
# pstext
#   width  ( in inch, optional )
#   height ( in inch, optional )
#   opt    ( optional )
#   text
#     x y size angle fontno justify text
#                             +-  9  10  11
#                                 5   6   7
#                                 1   2   3
$GMT->{pstext}->{width}  = 8.5;
$GMT->{pstext}->{height} = 5.4;
$GMT->{pstext}->{opt}    = '';
$GMT->{pstext}->{text}   = <<"TEXTEND";
0.2 5.1 14 0 0 1 VORTICITY AND WIND SPEED
7.6 4.2 14 0 0 2 (s\@+-1\@+)
TEXTEND

###############################################################################
# psbasemap
#   tick
#   x1, z1, dx, dz, opt ( optional )
$GMT->{psbasemap}->{tick} = '-BWSne50g10:"DISTANCE (km)":/5000g1000:"HEIGHT (m)":';
$GMT->{psbasemap}->{x1}   = -75;
$GMT->{psbasemap}->{z1}   = 0;
$GMT->{psbasemap}->{dx}   = 2.5 * 150/320;
$GMT->{psbasemap}->{dz}   = 100;
$GMT->{psbasemap}->{opt}  = '';


###############################################################################
# other setting
my $eps = 0;           # 0: PostScript output   1: Encapsulated Post Script
my $ps = 'z.ps';       # output filename
my $width  = 7;        # the width of cross section in inch
my $height = 5;        # the height of cross section in inch
my $debug = 1;         # 1: debug messages in standard output

# user settings are finished
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
###############################################################################
my $psxy       = $gmt_top . '/bin/psxy';
my $pstext     = $gmt_top . '/bin/pstext';
my $grdcontour = $gmt_top . '/bin/grdcontour';
my $grdimage   = $gmt_top . '/bin/grdimage';
my $grd2xyz    = $gmt_top . '/bin/grd2xyz';
my $minmax     = $gmt_top . '/bin/minmax';
my $gmtset     = $gmt_top . '/bin/gmtset';
my $psscale    = $gmt_top . '/bin/psscale';
my $psbasemap  = $gmt_top . '/bin/psbasemap';

###############################################################################
unlink( <.gmtdefault*> );
if( $eps > 0 ){
    if( $debug > 0 ){
	print( "$gmtset PAPER_MEDIA a4+\n" );
    }
    system( "$gmtset PAPER_MEDIA a4+" );
}

###############################################################################
my $n = @order;
$GMT->{debug}  = $debug;
$GMT->{ps}     = $ps;
$GMT->{width}  = $width;
$GMT->{height} = $height;
for( my $i = 0 ; $i < $n ; $i++ ){
    if( $order[$i] eq 'psbasemap' ){
	if( defined( $GMT->{xmin} ) && defined( $GMT->{xmax} ) &&
	    defined( $GMT->{ymin} ) && defined( $GMT->{ymax} ) ){

	    my $xmin = $GMT->{xmin};
	    my $xmax = $GMT->{xmax};
	    my $ymin = $GMT->{ymin};
	    my $ymax = $GMT->{ymax};

	    if( defined( $GMT->{psbasemap}->{dx} ) ){
		if( defined( $GMT->{psbasemap}->{x1} ) ){
		    $xmin = $GMT->{psbasemap}->{x1};
		    $xmax = $xmin + ( $GMT->{xmax} - 1 ) * $GMT->{psbasemap}->{dx};
		}else{
		    $xmin = 0;
		    $xmax = ( $xmax - 1 ) * $GMT->{psbasemap}->{dx};
		}
	    }

	    if( defined( $GMT->{psbasemap}->{dz} ) ){
		$ymin = 0;
		$ymax = ( $ymax - 1 ) * $GMT->{psbasemap}->{dz};
	    }

	    $GMT->{reg} = "-JX${width}i/${height}i -R${xmin}/${xmax}/${ymin}/${ymax}";

	    &psbasemap( \$GMT, $i, $n );
	}else{
	    print "currently unsupported.\n";
	    exit;
	}
	next;

    }elsif( $order[$i] =~ /pstext/i ){
	if( defined( $GMT->{$order[$i]}->{width} ) ){
	    $GMT->{width} = $GMT->{$order[$i]}->{width};
	}
	if( defined( $GMT->{$order[$i]}->{height} ) ){
	    $GMT->{height} = $GMT->{$order[$i]}->{height};
	}

	$GMT->{opt} = '';
	if( defined( $GMT->{$order[$i]}->{opt} ) ){
	    $GMT->{opt} = $GMT->{$order[$i]}->{opt};
	}

	$GMT->{reg} = "-JX$GMT->{width}i/$GMT->{height}i -R0/$GMT->{width}/0/$GMT->{height}";
	&pstext( \$GMT, $i, $n, $order[$i] );

	$GMT->{width}  = $width;
	$GMT->{height} = $height;
	$GMT->{opt}    = undef;
	$GMT->{reg}    = undef;
	next;

    }elsif( $order[$i] =~ /psscale/i ){
	unless( defined( $GMT->{$order[$i]}->{cpt} ) ){
	    print "CPT file is not defined.\n";
	    exit;
	}
	&check_file( $GMT->{$order[$i]}->{cpt}, 'exit' );

	&psscale( \$GMT, $i, $n, $order[$i] );
	next;
    }

    if( defined( $GMT->{$order[$i]}->{grd} ) ){
	my $grd = $GMT->{$order[$i]}->{grd};
	&check_file( $grd, 'exit' );

	open( MINMAX, "$grd2xyz $grd | $minmax |" );
	while( <MINMAX> ){
	    chomp;
	    /<([^\/>]+)\/([^\/>]+)>\s*<([^\/>]+)\/([^\/>]+)>\s*<([^\/>]+)\/([^\/>]+)>/;
	    my $xmin = $1;
	    my $xmax = $2;
	    my $ymin = $3;
	    my $ymax = $4;
	    my $zmin = $5;
	    my $zmax = $6;

	    $GMT->{xmin} = $xmin;
	    $GMT->{xmax} = $xmax;
	    $GMT->{ymin} = $ymin;
	    $GMT->{ymax} = $ymax;

	    $GMT->{reg} = "-JX${width}i/${height}i -R${xmin}/${xmax}/${ymin}/${ymax}";
	    $GMT->{grd} = $grd;

	    if( $debug > 0 ){
		print "summary of ${grd}\;\n";
		print "   XMIN/XMAX: ${xmin}/${xmax}\n";
		print "   YMIN/YMAX: ${ymin}/${ymax}\n";
		print "   VMIN/VMAX: ${zmin}/${zmax}\n";
	    }
	}

	if( $order[$i] eq 'grdimage' ){
	    my $cpt = $GMT->{$order[$i]}->{cpt};
	    &check_file( $cpt, 'exit' );

	    $GMT->{cpt} = $cpt;
	    &grdimage( \$GMT, $i, $n );

	    $GMT->{reg} = undef;
	    $GMT->{cpt} = undef;

	}elsif( $order[$i] =~ /grdcontour/ ){
	    $GMT->{opt} = $GMT->{$order[$i]}->{opt};
	    &grdcontour( \$GMT, $i, $n, $order[$i] );

	    $GMT->{opt} = undef;
	}

	$GMT->{grd} = undef;
	next;
    }

}
unlink( <.gmtdefault*> );

###############################################################################
# file check
sub check_file{
    my ( $file, $mode ) = @_;

    unless( -e $file ){
	if( $mode =~ /exit/i ){
	    print "ERROR: $file is not found.\n";
	    exit;
	}else{
	    print "WARNING: $file is not found.\n";
	}
    }
}

###############################################################################
# psbasemap
sub psbasemap{
    my ( $GMT, $i, $n ) = @_;

    my $end = "-O -K >> $$GMT->{ps}";
    if( $i == 0 ){
	if( $n > 1 ){
	    $end = "-K > $$GMT->{ps}";
	}else{
	    $end = "> $$GMT->{ps}";
	}

    }elsif( $i == $n-1 ){
	$end = "-O >> $$GMT->{ps}";
    }

    my $opt = '';
    if( defined( $$GMT->{psbasemap}->{opt} ) ){
	$opt = $$GMT->{psbasemap}->{opt};
    }

    if( $$GMT->{debug} > 0 ){
	print( "$psbasemap $$GMT->{reg} $$GMT->{psbasemap}->{tick} $opt $end\n" );
    }
    system( "$psbasemap $$GMT->{reg} $$GMT->{psbasemap}->{tick} $opt $end" );

}

###############################################################################
# grdimage
sub grdimage{
    my ( $GMT, $i, $n ) = @_;

    my $end = "-O -K >> $$GMT->{ps}";
    if( $i == 0 ){
	if( $n > 1 ){
	    $end = "-K > $$GMT->{ps}";
	}else{
	    $end = "> $$GMT->{ps}";
	}
    }elsif( $i == $n-1 ){
	$end = "-O >> $$GMT->{ps}";
    }

    my $tick = '';
    if( defined( $$GMT->{grdimage}->{tick} ) ){
	$tick = $$GMT->{grdimage}->{tick};
    }

    my $opt = '';
    if( defined( $$GMT->{grdimage}->{opt} ) ){
	$opt = $$GMT->{grdimage}->{opt};
    }

    if( $$GMT->{debug} > 0 ){
	print( "$grdimage $$GMT->{grd} $$GMT->{reg} ${tick} $opt -C$$GMT->{cpt} $end\n" );
    }
    system( "$grdimage $$GMT->{grd} $$GMT->{reg} ${tick} $opt -C$$GMT->{cpt} $end" );

}

###############################################################################
# grdcontour
sub grdcontour{
    my ( $GMT, $i, $n, $key ) = @_;

    my $end = "-O -K >> $$GMT->{ps}";
    if( $i == 0 ){
	if( $n > 1 ){
	    $end = "-K > $$GMT->{ps}";
	}else{
	    $end = "> $$GMT->{ps}";
	}
    }elsif( $i == $n-1 ){
	$end = "-O >> $$GMT->{ps}";
    }

    my $tick = '';
    if( defined( $$GMT->{$key}->{tick} ) ){
	$tick = $$GMT->{$key}->{tick};
    }

    if( $$GMT->{debug} > 0 ){
	print( "$grdcontour $$GMT->{grd} $$GMT->{reg} ${tick} $$GMT->{opt} $end\n" );
    }
    system( "$grdcontour $$GMT->{grd} $$GMT->{reg} ${tick} $$GMT->{opt} $end" );

}

###############################################################################
# pstext
sub pstext{
    my ( $GMT, $i, $n, $key ) = @_;

    my $end = "-O -K >> $$GMT->{ps}";
    if( $i == 0 ){
	if( $n > 1 ){
	    $end = "-K > $$GMT->{ps}";
	}else{
	    $end = "> $$GMT->{ps}";
	}
    }elsif( $i == $n-1 ){
	$end = "-O >> $$GMT->{ps}";
    }

    my $text = '';
    if( defined( $$GMT->{$key}->{text} ) ){
	$text = $$GMT->{$key}->{text};
    }

    if( $$GMT->{debug} > 0 ){
	print( "| $pstext $$GMT->{reg} $$GMT->{opt} $end\n" );
	print $text;
    }

    open( TEXT, "| $pstext $$GMT->{reg} $$GMT->{opt} $end" );
    print TEXT $text;
    close( TEXT );

}

###############################################################################
# psscale
sub psscale{
    my ( $GMT, $i, $n, $key ) = @_;

    my $end = "-O -K >> $$GMT->{ps}";
    if( $i == 0 ){
	if( $n > 1 ){
	    $end = "-K > $$GMT->{ps}";
	}else{
	    $end = "> $$GMT->{ps}";
	}
    }elsif( $i == $n-1 ){
	$end = "-O >> $$GMT->{ps}";
    }

    my $opt = '';
    if( defined( $$GMT->{$key}->{opt} ) ){
	$opt = $$GMT->{$key}->{opt};
    }

    if( defined( $$GMT->{$key}->{format} ) ){
	if( $$GMT->{debug} > 0 ){
	    print( "$gmtset D_FORMAT $$GMT->{$key}->{format}\n" );
	}
	system( "$gmtset D_FORMAT $$GMT->{$key}->{format}" );
    }

    if( $$GMT->{debug} > 0 ){
	print( "$psscale $$GMT->{$key}->{cpt} $opt $$GMT->{$key}->{position} $end\n" );
    }

    system( "$psscale -C$$GMT->{$key}->{cpt} $opt $$GMT->{$key}->{position} $end" );

    if( defined( $$GMT->{$key}->{format} ) ){
	if( $$GMT->{debug} > 0 ){
	    print( "$gmtset D_FORMAT %lg\n" );
	}
	system( "$gmtset D_FORMAT %lg" );
    }

}
